@@ -1,4 +1,11 @@
-//  This program is free software; you can redistribute it and/or modiry
+//  Copyright (C) 2002, 2003 Michael Bartl
+//  Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
+//  Copyright (C) 2004, 2005, 2006 Andrea Paternesi
+//  Copyright (C) 2005 Josef Spillner
+//  Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//  Copyright (C) 2007 Ole Laursen
+//
+//  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
 //  (at your option) any later version.
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdlib.h>
 #include <fstream>
@@ -46,6 +54,7 @@ int Configuration::s_displaySpeedDelay = 0;
 Uint32 Configuration::s_surfaceFlags = SDL_SWSURFACE;
 Uint32 Configuration::s_cacheSize = 1000000;
 bool Configuration::s_zipfiles = false;
+int Configuration::s_autosave_policy = 1;
 bool Configuration::s_hardware = false;
 bool Configuration::s_ggz = false;
 bool Configuration::s_musicenable = true;
@@ -116,6 +125,7 @@ bool Configuration::saveConfigurationFile(string filename)
     retval &= helper.saveData("cachesize", s_cacheSize);
     retval &= helper.saveData("hardware", s_hardware);
     retval &= helper.saveData("zipfiles", s_zipfiles);
+    retval &= helper.saveData("autosave_policy", s_autosave_policy);
     retval &= helper.saveData("speeddelay", s_displaySpeedDelay);
     retval &= helper.saveData("shownextplayer", s_showNextPlayer);
     retval &= helper.saveData("musicenable", s_musicenable);
@@ -155,6 +165,7 @@ bool Configuration::parseConfiguration(string tag, XML_Helper* helper)
     
     string temp;
     bool retval,zipping;
+    int autosave_policy;
     
     if (helper->getVersion() != LORDSAWAR_CONFIG_VERSION)
     {
@@ -230,6 +241,11 @@ bool Configuration::parseConfiguration(string tag, XML_Helper* helper)
     if (retval)
         s_zipfiles = zipping;
 
+    //parse when and how to save autosave files
+    retval = helper->getData(autosave_policy, "autosave_policy");
+    if (retval)
+        s_autosave_policy = autosave_policy;
+
     //parse the speed delay
     helper->getData(s_displaySpeedDelay, "speeddelay");
 
