@@ -1,3 +1,10 @@
+//  Copyright (C) 2002, 2003 Michael Bartl
+//  Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
+//  Copyright (C) 2004, 2005 Andrea Paternesi
+//  Copyright (C) 2005 Josef Spillner
+//  Copyright (C) 2006 Ben Asselstine
+//  Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef CONFIGURATION_H
 #define CONFIGURATION_H
@@ -81,6 +89,11 @@ class Configuration : public sigc::trackable
         //zip and obfuscate save files
         static bool s_zipfiles;
 
+	// when to save autosave files
+	// 0 = never, 1 = once a round overwrting, 
+	// 2 = once a round not-overwriting
+	static int s_autosave_policy;
+
         //run game in GGZ mode
         static bool s_ggz;
 
@@ -103,6 +116,7 @@ class Configuration : public sigc::trackable
         static bool s_quick_start;
         static bool s_cusp_of_war;
 
+	//autos
     private:
         /** \brief The callback for the XML_Helper
           * 
