@@ -1,3 +1,10 @@
+// Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2004, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sstream>
 
@@ -63,35 +71,12 @@ CreateScenario::CreateScenario(int width, int height)
     Portlist::deleteInstance();
     Bridgelist::deleteInstance();
     Citylist::deleteInstance();
+    Itemlist::deleteInstance();
 
     QuestsManager::deleteInstance();
 
     fl_counter = new FL_Counter();
 
-    // Fill the namelists 
-    bool success = true;
-    
-    std::ifstream file(File::getMiscFile("citynames").c_str());
-    success &= loadNames(d_citynames, file);
-    file.close();
-
-    file.open(File::getMiscFile("templenames").c_str());
-    success &= loadNames(d_templenames, file);
-    file.close();
-
-    file.open(File::getMiscFile("ruinnames").c_str());
-    success &= loadNames(d_ruinnames, file);
-    file.close();
-
-    file.open(File::getMiscFile("signposts").c_str());
-    success &= loadNames(d_signposts, file);
-    file.close();
-
-    if (!success)
-    {
-        std::cerr <<"CreateScenario: Didn't succeed in reading object names. Aborting!\n";
-        exit(-1);
-    }
     setWidth(width);
     setHeight(height);
 }
@@ -370,6 +355,9 @@ bool CreateScenario::create(const GameParameters &g)
     if (!setupPlayers(g.diplomacy, g.random_turns, base_gold))
         return false;
 
+    if (!setupItems())
+        return false;
+
     if (!setupRewards())
         return false;
 
@@ -416,10 +404,12 @@ bool CreateScenario::createMap()
                     Signpostlist::getInstance()->push_back(Signpost(Vector<int>(x,y)));
                     break;
                 case Maptile::TEMPLE:
-                    Templelist::getInstance()->push_back(Temple(Vector<int>(x,y)));
+                    Templelist::getInstance()->push_back
+		      (Temple(Vector<int>(x,y), popRandomTempleName()));
                     break;
                 case Maptile::RUIN:
-		    Ruinlist::getInstance()->push_back(Ruin(Vector<int>(x,y)));
+		    Ruinlist::getInstance()->push_back
+		      (Ruin(Vector<int>(x,y), popRandomRuinName()));
 		    break;
                 case Maptile::CITY:
                     Citylist::getInstance()->push_back(City(Vector<int>(x,y)));
@@ -477,7 +467,7 @@ bool CreateScenario::distributePlayers()
                 pit++;
         }
         else
-            (*cit).setPlayer(pl->getNeutral());
+            (*cit).setOwner(pl->getNeutral());
 
     }
 
@@ -497,19 +487,14 @@ bool CreateScenario::setupCities(bool quick_start,
         it != Citylist::getInstance()->end(); it++)
     {
         //1. set a reasonable cityname
-        int randno = rand() % d_citynames.size();
-        (*it).setName(d_citynames[randno]);
-
-        //remove the used name
-        d_citynames[randno] = d_citynames[d_citynames.size() - 1];
-        d_citynames.pop_back();
+        (*it).setName(popRandomCityName());
 
         //2. distribute the income a bit (TBD)
 
         //3. set the city production
         (*it).setRandomArmytypes(cities_can_produce_allies, 
 				 number_of_armies_factor);
-        if ((*it).getPlayer() == Playerlist::getInstance()->getNeutral())
+        if ((*it).getOwner() == Playerlist::getInstance()->getNeutral())
         {
             switch (GameScenario::s_neutral_cities)
               {
@@ -517,33 +502,30 @@ bool CreateScenario::setupCities(bool quick_start,
                 (*it).produceScout();
                 break;
               case GameParameters::STRONG:
-                (*it).produceStrongestArmy();
+                (*it).produceStrongestProductionBase();
                 break;
               case GameParameters::ACTIVE:
                 if (rand () % 100 >  20)
-                  (*it).produceStrongestArmy();
+                  (*it).produceStrongestProductionBase();
                 else
-                  (*it).produceWeakestArmy();
+                  (*it).produceWeakestProductionBase();
                 break;
               }
-            (*it).setProduction(-1);
+            (*it).setActiveProductionSlot(-1);
         }
         else
         {
           if ((*it).isCapital())
-            (*it).produceStrongestArmy();
+            (*it).produceStrongestProductionBase();
           else
-            (*it).produceWeakestArmy();
+            (*it).produceWeakestProductionBase();
 
-            (*it).setProduction(0);
+            (*it).setActiveProductionSlot(0);
         }
 
 	if (rand() % 2 == 0)
 	  (*it).raiseDefense();
-	if ((*it).isCapital())
-	  (*it).setGold(33 + (rand() % 8));
-	else
-	  (*it).setGold(15 + (rand() % 12));
+	(*it).setGold(getRandomCityIncome((*it).isCapital()));
     }
 
     return true;
@@ -562,17 +544,10 @@ bool CreateScenario::setupTemples()
     Templelist* tl = Templelist::getInstance();
     for (Templelist::iterator it = tl->begin(); it != tl->end(); it++)
     {
-        // set a temple name
-        int randno = rand() % d_templenames.size();
-        (*it).setName(d_templenames[randno]);
-
         // set a random temple type
         int type= (int) ((TEMPLE_TYPES*1.0) * (rand() / (RAND_MAX + 1.0)));
         (*it).setType(type);
 
-        //remove the used name
-        d_templenames[randno] = d_templenames[d_templenames.size() - 1];
-        d_templenames.pop_back();
     }
 
     return true;
@@ -585,24 +560,9 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
     //The aim of this function is to put a strong stack as sentinel in all
     //ruins.
 
-    // list all the army types that can be a sentinel.
-    std::vector<const Army*> occupants;
-    Armysetlist *al = Armysetlist::getInstance();
-    Player *p = Playerlist::getInstance()->getNeutral();
-    for (unsigned int j = 0; j < al->getSize(p->getArmyset()); j++)
-      {
-        const Army *a = al->getArmy (p->getArmyset(), j);
-        if (a->getDefendsRuins())
-          occupants.push_back(a);
-      }
-
     for (Ruinlist::iterator it = Ruinlist::getInstance()->begin();
         it != Ruinlist::getInstance()->end(); it++)
     {
-        // set a ruin name
-        int randno = rand() % d_ruinnames.size();
-        (*it).setName(d_ruinnames[randno]);
-
         // set a random ruin type
         if (rand() % stronghold_factor == 0) //one in six ruins is a stronghold
           {
@@ -616,10 +576,6 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
         else
           (*it).setType(Ruin::RUIN);
 
-        //remove the used name
-        d_ruinnames[randno] = d_ruinnames[d_ruinnames.size() - 1];
-        d_ruinnames.pop_back();
-
         //one in twenty ruins is a sage
         if (rand() % sage_factor == 0 && (*it).getType() == Ruin::RUIN) 
           {
@@ -634,16 +590,16 @@ bool CreateScenario::setupRuins(bool strongholds_invisible, int sage_factor,
 
         // and set a guardian
         Stack* s;
-        const Army* a = 0;
+        Army* a = 0;
         Vector<int> pos = (*it).getPos();
         
-        if (!occupants.empty())
+	a = getRandomRuinKeeper(Playerlist::getInstance()->getNeutral());
+        if (a)
           {
             //create a stack:
             s = new Stack(0, pos);
             
-            a = occupants[rand() % occupants.size()];
-            s->push_back(new Army(*a, p));
+            s->push_back(a);
             a = 0;
 
             //now mark this stack as guard
@@ -659,53 +615,22 @@ bool CreateScenario::setupSignposts(int ratio)
     int randno;
     int dynamicPercent = static_cast<int>(1.0 / ratio * 100);
     debug("CreateScenario::setupSignposts")
+    Signpostlist *sl = Signpostlist::getInstance();
 
-    for (Signpostlist::iterator it = Signpostlist::getInstance()->begin();
-        it != Signpostlist::getInstance()->end(); it++)
+    for (Signpostlist::iterator it = sl->begin(); it != sl->end(); it++)
     {
-	if (d_signposts.size() == 0)
+	if (randomSignpostsEmpty())
 	    randno = dynamicPercent;
 	else
 	    randno = rand() % 100;
 	if (randno < dynamicPercent)
 	{
             // set up a signpost from the list of signposts
-            randno = rand() % d_signposts.size();
-            (*it).setName(d_signposts[randno]);
-
-            //remove the used name
-            d_signposts[randno] = d_signposts[d_signposts.size() - 1];
-            d_signposts.pop_back();
+	    (*it).setName(popRandomSignpost());
 	}
 	else
 	{
-	    char *dir = NULL;
-	    int xdir, ydir;
-	    char buf[101]; buf[100] = '\0';
-            Vector<int> signpostPos = (*it).getPos();
-	    City *nearCity = Citylist::getInstance()->getNearestCity(signpostPos);
-            Vector<int> cityPos = nearCity->getPos();
-	    xdir = cityPos.x - signpostPos.x;
-	    ydir = cityPos.y - signpostPos.y;
-	    if (xdir >= 1 && ydir >= 1)
-		dir = _("southeast");
-	    else if (xdir >= 1 && ydir == 0)
-		dir = _("east");
-	    else if (xdir >= 1 && ydir <= -1)
-		dir = _("northeast");
-	    else if (xdir == 0 && ydir >= 1)
-		dir = _("south");
-	    else if (xdir == 0 && ydir <= -1)
-		dir = _("north");
-	    else if (xdir <= -1 && ydir >= 1)
-		dir = _("southwest");
-	    else if (xdir <= -1 && ydir == 0)
-		dir = _("west");
-	    else if (xdir <= -1 && ydir <= -1)
-		dir = _("northwest");
-	    snprintf(buf,100,_("%s lies to the %s"), 
-		             nearCity->getName().c_str(), dir);
-            (*it).setName(buf);
+            (*it).setName(getDynamicSignpost(&*it));
 	}
     }
 
@@ -754,36 +679,6 @@ bool CreateScenario::setupPlayers(bool diplomacy, bool random_turns,
     return true;
 }
 
-bool CreateScenario::loadNames(std::vector<std::string>& list, std::ifstream& namefile)
-{
-    debug("CreateScenario::loadNames")
-
-    int counter;
-    char buffer[101];
-    buffer[100] = '\0';
-    
-    if (!namefile)
-    {
-        std::cerr <<_("Critical Error: Couldn't open citynames data file\n");
-        return false;
-    }
-
-    namefile >> counter;
-    list.resize(counter);
-
-    //with getline, the first call will get the first line to the end, i.e.
-    //a newline character. Thus, we throw away the result of the first call.
-    namefile.getline(buffer, 100);
-
-    for (counter--; counter >= 0; counter--)
-    {
-        namefile.getline(buffer, 100);
-        list[counter] = std::string(buffer);
-    }
-
-    return true;
-}
-
 bool CreateScenario::setupRewards()
 {
   debug("CreateScenario::setupRewards")
@@ -828,13 +723,10 @@ bool CreateScenario::setupMapRewards()
 	  else
 	    continue;
 
-	  Vector<int> pos;
-	  pos.x = i;
-	  pos.y = j;
-	  Location *loc = new Location(name, pos);
-	  Reward_Map *reward = new Reward_Map(loc, 
+	  Reward_Map *reward = new Reward_Map(Vector<int>(i, j), name,
 					      GameMap::getHeight() / 3, 
 					      GameMap::getWidth() / 3);
+	    
 	  Rewardlist::getInstance()->push_back(reward); //add it
 	  w_count++;
 	}
@@ -853,6 +745,7 @@ bool CreateScenario::setupRuinRewards()
 	  {
 	    //add it to the reward list
 	    Reward_Ruin *newReward = new Reward_Ruin(&(*it)); //make a reward
+	    newReward->setName(newReward->getDescription());
 	    Rewardlist::getInstance()->push_back(newReward); //add it
 	  }
 	if ((*it).hasSage() == false)
@@ -861,10 +754,15 @@ bool CreateScenario::setupRuinRewards()
   return true;
 }
 
+bool CreateScenario::setupItems()
+{
+  Itemlist::createStandardInstance();
+  return true;
+}
+
 bool CreateScenario::setupItemRewards()
 {
   debug("CreateScenario::setupItemRewards")
-    Itemlist::createInstance();
   Itemlist *il = Itemlist::getInstance();
   Itemlist::iterator iter;
   for (iter = il->begin(); iter != il->end(); iter++)
@@ -872,10 +770,10 @@ bool CreateScenario::setupItemRewards()
       Item templateItem = *iter->second;
       Item *newItem = new Item(templateItem); //instantiate it
       Reward_Item *newReward = new Reward_Item(newItem); //make a reward
+      newReward->setName(newReward->getDescription());
       Rewardlist::getInstance()->push_back(newReward); //add it
     }
 
-  Itemlist::deleteInstance();
   return true;
 }
 
