@@ -1,3 +1,8 @@
+// Copyright (C) 2003, 2004, 2005 Ulf Lorenz
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef CREATE_SCENARIO_H
 #define CREATE_SCENARIO_H
@@ -18,6 +24,7 @@
 #include <string>
 #include <vector>
 #include <SDL.h>
+#include "CreateScenarioRandomize.h"
 #include "game-parameters.h"
 
 class MapGenerator;
@@ -34,7 +41,7 @@ class Player;
   * e.g. to the player class have to be reflected here, too.
   */
 
-class CreateScenario
+class CreateScenario : public CreateScenarioRandomize
 {
     public:
         //! This represents the class of the map (hills, islands etc.).
@@ -179,6 +186,10 @@ class CreateScenario
 	bool setupRuins(bool strongholds_invisible, int sage_factor, 
 			int no_guardian_factor, int stronghold_factor);
 
+
+	//! Set up the standard set of items
+	bool setupItems();
+
         //! Set up signposts
 	//! @param ratio - how many signposts reference nearby cities vs
 	//                 signposts that don't.
@@ -190,12 +201,6 @@ class CreateScenario
         bool setupPlayers(bool dimplomacy, bool random_turns, int base_gold);
 
 	bool setupRoads();
-        /** Loads a list of possible city, ruin or temple names
-          * 
-          * @param list     the list to be filled with the names
-          * @param namefile an ifstream of the file containing the names
-          */
-        bool loadNames(std::vector<std::string>& list, std::ifstream& namefile);
 
         //! Set up rewards to be given out for quests, ruins and sages
         bool setupRewards();
@@ -234,10 +239,6 @@ class CreateScenario
         int d_width;
         int d_height;
         bool d_turnmode;
-
-        //the namelists
-        std::vector<std::string> d_citynames, d_signposts;
-        std::vector<std::string> d_templenames, d_ruinnames;
 };
 
 #endif  //CREATE_SCENARIO_H
