@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <sstream>
@@ -19,6 +22,7 @@
 
 #include "playerlist.h"
 #include "xmlhelper.h"
+#include "GameScenario.h"
 
 using namespace std;
 
@@ -189,4 +193,20 @@ void FogMap::smooth()
     }
 }
 
+bool FogMap::isFogged(Vector <int> pos)
+{
+  //is this tile visible, or not?
+  FogMap *fogmap = Playerlist::getActiveplayer()->getFogMap();
+  if (fogmap->getFogTile(pos) == FogMap::CLOSED)
+    return true;
+  if (Playerlist::getActiveplayer())
+    if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+	GameScenario::s_hidden_map == true)
+      return true;
+                
+  if (fogmap->isLoneFogTile(pos) == true)
+    return false;
+
+  return false;
+}
 // End of file
