@@ -1,3 +1,9 @@
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +16,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sstream>
 #include <assert.h>
@@ -365,7 +372,7 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
     max = s_width;
   max--;
 
-  Location l("", pos, 1);
+  Location l(pos, 1);
   s = l.addArmy(a);
   if (s)
     return s;
@@ -396,7 +403,7 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
                     continue;
                   //is there somebody else's city here?
                   City *c = Citylist::getInstance()->getObjectAt(x, y);
-                  if (c && c->getPlayer() != a->getPlayer())
+                  if (c && c->getOwner() != a->getOwner())
                     {
                       if (c->isBurnt() == false)
                         continue;
@@ -406,11 +413,15 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
                     continue;
                   if (!land && getTile(x, y)->getType() != Tile::WATER)
                     continue;
+                  if (land && getTile(x, y)->getType() == Tile::MOUNTAIN &&
+		      (a->getStat(Army::MOVE_BONUS) & Tile::MOUNTAIN) == 
+		      Tile::MOUNTAIN)
+                    continue;
                   //is there somebody else's stack here?
                   s = Stacklist::getObjectAt(x, y);
                   if (s)
                     { 
-                      if (s->getPlayer() != a->getPlayer())
+                      if (s->getOwner() != a->getOwner())
                         continue;
                       //is it our stack, but too full?
                       if (s->size() >= MAX_STACK_SIZE)
@@ -420,8 +431,8 @@ Stack* GameMap::addArmyAtPos(Vector<int> pos, Army *a)
                   else  //but there isn't a stack here
                     {
                       Vector<int> pos(x, y);
-                      s = new Stack(a->getPlayer(), pos);
-                      a->getPlayer()->addStack(s);
+                      s = new Stack(a->getOwner(), pos);
+                      a->getOwner()->addStack(s);
                     }
                   s->push_front(a);
                   added_army = true;
@@ -509,6 +520,11 @@ bool GameMap::isBlockedAvenue(int x, int y, int destx, int desty)
       if (to->getMaptileType() == Tile::MOUNTAIN &&
 	  Roadlist::getInstance()->getObjectAt(destx, desty) == NULL)
         return true;
+
+      //am i on a mountain without a road?
+      if (from->getMaptileType() == Tile::MOUNTAIN &&
+	  Roadlist::getInstance()->getObjectAt(x, y) == NULL)
+        return true;
     }
  return false;
 }
@@ -592,9 +608,9 @@ TileStyle *GameMap::calculatePreferredStyle(int i, int j)
     for (int l = -1; l <= +1; l++)
       {
 	box[k+1][l+1] = 1;
-	if (i+k >= s_height || i+k < 0)
+	if ((i+k) >= s_height || (i+k) < 0)
 	  continue;
-	if (j+l >= s_width || j+l < 0)
+	if ((j+l) >= s_width || (j+l) < 0)
 	  continue;
 	box[k+1][l+1] = (getTile(j+l, i+k)->getType() == mtile->getType());
       }
@@ -754,12 +770,11 @@ int GameMap::tile_is_connected_to_other_like_tiles (Tile::Type tile, int i, int
   for (int k = -1; k <= +1; k++)
     for (int l = -1; l <= +1; l++)
       {
-	if (i+k >= s_height || i+k < 0)
+	if ((i+k) >= s_height || (i+k) < 0)
 	  continue;
-	if (j+l >= s_width || j+l < 0)
+	if ((j+l) >= s_width || (j+l) < 0)
 	  continue;
-	box[k+1][l+1] = 
-	  (d_map[(i+k)*s_width + (j+l)]->getMaptileType() == tile);
+	box[k+1][l+1] = (getTile(j+l, i+k)->getMaptileType() == tile);
       }
   if (box[0][0] && box[0][1] && box[1][0] && box[1][1])
     return 1;
@@ -784,7 +799,7 @@ void GameMap::demote_lone_tile(int minx, int miny, int maxx, int maxy,
 	  continue;
 	if (j < 0 || j >= s_width)
 	  continue;
-	Tile::Type tile = d_map[i*s_width + j]->getMaptileType();
+	Tile::Type tile = getTile(j, i)->getMaptileType();
 	if (tile == intype)
 	  {
 	    //if we're not connected in a square of
@@ -793,11 +808,8 @@ void GameMap::demote_lone_tile(int minx, int miny, int maxx, int maxy,
 	      {
 		//okay, this is a lone tile.
 		//downgrade it
-		delete d_map[i*s_width + j];
-
-		d_map[i*s_width + j] = 
-		  new Maptile(d_tileSet, j, i, d_tileSet->getIndex(outtype), 
-			      NULL);
+		setTile(j, i, new Maptile
+			(d_tileSet, j, i, d_tileSet->getIndex(outtype), NULL));
 	      }
 	  }
       }
