@@ -1,3 +1,9 @@
+// Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +16,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <fstream>
@@ -78,7 +85,6 @@ GameScenario::GameScenario(std::string name,std::string comment, bool turnmode)
     Tilesetlist::getInstance()->instantiatePixmaps();
     Shieldsetlist::getInstance();
     Shieldsetlist::getInstance()->instantiatePixmaps();
-    Itemlist::createInstance();
 
     if (fl_counter == 0)
         fl_counter = new FL_Counter();
@@ -87,48 +93,60 @@ GameScenario::GameScenario(std::string name,std::string comment, bool turnmode)
 // savegame is a filename with absolute path!
 
 GameScenario::GameScenario(string savegame, bool& broken)
-    :d_turnmode(true)
+  :d_turnmode(true)
 {
-    Armysetlist::getInstance();
-    Armysetlist::getInstance()->instantiatePixmaps();
-    Tilesetlist::getInstance();
-    Tilesetlist::getInstance()->instantiatePixmaps();
-    Shieldsetlist::getInstance();
-    Shieldsetlist::getInstance()->instantiatePixmaps();
-    Itemlist::createInstance();
-
-    broken = false;
-    XML_Helper helper(savegame, ios::in, Configuration::s_zipfiles);
-
-    helper.registerTag("scenario", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("playerlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("map", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("citylist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("templelist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("ruinlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("rewardlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("signpostlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("roadlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("counter", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("questlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("bridgelist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("portlist", sigc::mem_fun(this, &GameScenario::load));
-    helper.registerTag("vectoredunitlist", sigc::mem_fun(this, &GameScenario::load));
-
-    //now parse the document and close the file afterwards
-    if (!helper.parse())
-    {
-        broken = true;
-    }
-
-    helper.close();
-    GameMap::getInstance()->calculateBlockedAvenues();
+  XML_Helper helper(savegame, ios::in, Configuration::s_zipfiles);
+  broken = loadWithHelper(helper);
+  helper.close();
 }
 
+GameScenario::GameScenario(XML_Helper &helper, bool& broken)
+  : d_turnmode(true)
+{
+  broken = loadWithHelper(helper);
+}
+
+bool GameScenario::loadWithHelper(XML_Helper& helper)
+{
+  Armysetlist::getInstance();
+  Armysetlist::getInstance()->instantiatePixmaps();
+  Tilesetlist::getInstance();
+  Tilesetlist::getInstance()->instantiatePixmaps();
+  Shieldsetlist::getInstance();
+  Shieldsetlist::getInstance()->instantiatePixmaps();
+
+  bool broken = false;
+
+  helper.registerTag("scenario", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("itemlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("playerlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("map", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("citylist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("templelist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("ruinlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("rewardlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("signpostlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("roadlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("counter", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("questlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("bridgelist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("portlist", sigc::mem_fun(this, &GameScenario::load));
+  helper.registerTag("vectoredunitlist", sigc::mem_fun(this, &GameScenario::load));
+
+  if (!helper.parse())
+    broken = true;
+    
+  GameMap::getInstance()->calculateBlockedAvenues();
+
+  return broken;
+}
+
+
 GameScenario::~GameScenario()
 {
     // GameMap is a Singleton so we need a function to delete it
     GameMap::deleteInstance();
+    Itemlist::deleteInstance();
     Playerlist::deleteInstance();
     Citylist::deleteInstance();
     Templelist::deleteInstance();
@@ -139,7 +157,6 @@ GameScenario::~GameScenario()
     Bridgelist::deleteInstance();
     Roadlist::deleteInstance();
     QuestsManager::deleteInstance();
-    Itemlist::deleteInstance();
     VectoredUnitlist::deleteInstance();
 
     if (fl_counter)
@@ -169,6 +186,13 @@ void GameScenario::nextRound()
 {
     d_round++;
 
+    char filename[1024];
+    if (Configuration::s_autosave_policy == 2)
+      snprintf(filename,sizeof(filename), "autosave-%03d.sav", d_round - 1);
+    else if (Configuration::s_autosave_policy == 1)
+      snprintf(filename,sizeof(filename), "autosave.sav");
+    else
+      return;
     // autosave to the file "autosave.sav". This is crude, but should work
     //
     // As a more enhanced version: autosave to a temporary file, then rename
@@ -180,7 +204,7 @@ void GameScenario::nextRound()
         return;
     }
     if (rename(std::string(File::getSavePath() + "tmp.sav").c_str(),
-               std::string(File::getSavePath() + "autosave.sav").c_str()))
+               std::string(File::getSavePath() + filename).c_str()))
     {
         char* err = strerror(errno);
         std::cerr <<_("Error while trying to rename the temporary file to autosave.sav\n");
@@ -209,7 +233,20 @@ bool GameScenario::saveGame(string filename, string extension) const
     delete strtoken;
 
     XML_Helper helper(goodfilename, ios::out, Configuration::s_zipfiles);
+    retval &= saveWithHelper(helper);
+    helper.close();
 
+    if (retval)
+        return true;
+        
+    std::cerr <<_("GameScenario: Something went wrong with saving.\n");
+    return false;
+}
+
+bool GameScenario::saveWithHelper(XML_Helper &helper) const
+{
+    bool retval = true;
+    
     //start writing
     retval &= helper.begin(LORDSAWAR_SAVEGAME_VERSION);
     retval &= helper.openTag("lordsawar");
@@ -217,6 +254,7 @@ bool GameScenario::saveGame(string filename, string extension) const
     //if retval is still true it propably doesn't change throughout the rest
     //now save the single object's data
     retval &= fl_counter->save(&helper);
+    retval &= Itemlist::getInstance()->save(&helper);
     retval &= Playerlist::getInstance()->save(&helper);
     retval &= GameMap::getInstance()->save(&helper);
     retval &= Citylist::getInstance()->save(&helper);
@@ -252,21 +290,9 @@ bool GameScenario::saveGame(string filename, string extension) const
     
     retval &= helper.closeTag();
     
-    if (!retval)
-    {
-        std::cerr <<_("GameScenario: Something went wrong with saving.\n");
-        return false;
-    }
-    
     retval &= helper.closeTag();
-
-    helper.close();
-
-    if (retval)
-        return true;
-        
-    std::cerr <<_("GameScenario: Something went wrong with saving.\n");
-    return false;
+    
+    return retval;
 }
 
 bool GameScenario::load(std::string tag, XML_Helper* helper)
@@ -314,6 +340,13 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
         return true;
     }
 
+    if (tag == "itemlist")
+    {
+        debug("loading items");
+        Itemlist::getInstance(helper);
+        return true;
+    }
+
     if (tag == "playerlist")
     {
         debug("loading players");
