@@ -1,3 +1,9 @@
+// Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2006 Andrea Paternesi
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +16,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef GAME_SCENARIO_H
 #define GAME_SCENARIO_H
@@ -22,17 +29,17 @@
 
 class XML_Helper;
 
-/** Class to hold several scenario options
-  * 
-  * This class has two functions. On the one hand side, it holds some data
-  * about the current scenario being played (such as the name), on the other
-  * hand it has a kind of supervisor function. Loading and saving works in
-  * a hierarchical way with superior objects (such as the playerlist) saving
-  * their data and then telling inferior objects (such as players) to save
-  * their data as well. GameScenario is kind of the root of the saving or
-  * loading process. For more information about the saving procedure, have
-  * a look at XML_Helper.
-  */
+//! A class to hold several scenario options.
+/** 
+ * This class has two functions. On the one hand side, it holds some data
+ * about the current scenario being played (such as the name), on the other
+ * hand it has a kind of supervisor function. Loading and saving works in
+ * a hierarchical way with superior objects (such as the playerlist) saving
+ * their data and then telling inferior objects (such as players) to save
+ * their data as well. GameScenario is kind of the root of the saving or
+ * loading process. For more information about the saving procedure, have
+ * a look at XML_Helper.
+ */
 
 class GameScenario: public sigc::trackable
 {
@@ -52,6 +59,9 @@ class GameScenario: public sigc::trackable
           * @param broken       set to true if something goes wrong
           */
         GameScenario(std::string savegame, bool& broken);
+
+        GameScenario(XML_Helper &helper, bool &broken);
+
         ~GameScenario();
 
         //! Returns the number of the current turn.
@@ -82,6 +92,10 @@ class GameScenario: public sigc::trackable
           * @return true if all went well, false otherwise
           */
         bool saveGame(std::string filename, std::string extension = "sav") const;
+
+        bool loadWithHelper(XML_Helper &helper);
+        bool saveWithHelper(XML_Helper &helper) const;
+        
         static bool s_see_opponents_stacks;
         static bool s_see_opponents_production;
         static bool s_play_with_quests;
