@@ -1,3 +1,7 @@
+// Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
+// Copyright (C) 2004, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +14,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <assert.h>
 
@@ -241,24 +246,24 @@ GraphicsCache::~GraphicsCache()
         SDL_FreeSurface(d_flagmask[i]);
     }
 
-    for (int i = 0; i < 6; i++)
+    for (int i = 0; i < SELECTOR_FRAMES; i++)
     {
         SDL_FreeSurface(d_selector[i]);
         SDL_FreeSurface(d_selectormask[i]);
     }
 
-    for (int i = 0; i < 4; i++)
+    for (int i = 0; i < SMALL_SELECTOR_FRAMES; i++)
     {
         SDL_FreeSurface(d_smallselector[i]);
         SDL_FreeSurface(d_smallselectormask[i]);
     }
 
-    for (int i = 0; i < 7; i++)
+    for (int i = 0; i < PRODUCTION_SHIELD_TYPES; i++)
     {
         SDL_FreeSurface(d_prodshieldpic[i]);
     }
 
-    for (int i = 0; i < 6; i++)
+    for (int i = 0; i < MOVE_BONUS_TYPES; i++)
     {
         SDL_FreeSurface(d_movebonuspic[i]);
     }
@@ -690,9 +695,9 @@ SDL_Surface* GraphicsCache::getCityPic(const City* city)
     if (!city)
         return 0;
     if (city->isBurnt())
-      return d_razedpic[city->getPlayer()->getId()];
+      return d_razedpic[city->getOwner()->getId()];
     else
-      return d_citypic[city->getPlayer()->getId()];
+      return d_citypic[city->getOwner()->getId()];
 }
 
 SDL_Surface* GraphicsCache::getCityPic(int type, const Player* p)
@@ -713,7 +718,7 @@ SDL_Surface* GraphicsCache::getTowerPic(const Player* p)
 
 SDL_Surface* GraphicsCache::getFlagPic(const Stack* s)
 {
-    debug("GraphicsCache::getFlagPic " <<s->getId() <<", player" <<s->getPlayer()->getName())
+    debug("GraphicsCache::getFlagPic " <<s->getId() <<", player" <<s->getOwner()->getName())
 
     if (!s)
     {
@@ -727,7 +732,7 @@ SDL_Surface* GraphicsCache::getFlagPic(const Stack* s)
     for (it = d_flaglist.begin(); it != d_flaglist.end(); it++)
     {
         myitem = *it;
-        if (myitem->size == s->size() - 1 && myitem->player == s->getPlayer())
+        if (myitem->size == s->size() - 1 && myitem->player == s->getOwner())
         {
             // put the item in last place (last touched)
             d_flaglist.erase(it);
@@ -738,7 +743,7 @@ SDL_Surface* GraphicsCache::getFlagPic(const Stack* s)
     }
 
     // no item found => create a new one
-    myitem = addFlagPic(s->size(), s->getPlayer());
+    myitem = addFlagPic(s->size(), s->getOwner());
 
     return myitem->surface;
 }
@@ -746,7 +751,7 @@ SDL_Surface* GraphicsCache::getFlagPic(const Stack* s)
 SDL_Surface* GraphicsCache::getSelectorPic(Uint32 type, Uint32 frame, 
 					   const Player *p)
 {
-    debug("GraphicsCache::getSelectorPic " <<type <<", " << frame << ", player" <<s->getPlayer()->getName())
+    debug("GraphicsCache::getSelectorPic " <<type <<", " << frame << ", player" <<s->getOwner()->getName())
 
     if (!p)
     {
@@ -2145,7 +2150,7 @@ void GraphicsCache::loadSelectors()
 
   // copy alpha values, don't use them
   SDL_SetAlpha(selpics, 0, 0);
-  for (i = 0; i < 6; i++)
+  for (i = 0; i < SELECTOR_FRAMES; i++)
     {
       SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 
 					      fmt->BitsPerPixel, fmt->Rmask, 
@@ -2159,7 +2164,7 @@ void GraphicsCache::loadSelectors()
       d_selector[i] = SDL_DisplayFormatAlpha(tmp);
       SDL_FreeSurface(tmp);
 
-      d_selectormask[i]=  SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 32,
+      d_selectormask[i] = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 32,
 					       0xFF000000, 0xFF0000, 0xFF00, 
 					       0xFF);
       selrect.y = size;
@@ -2173,7 +2178,7 @@ void GraphicsCache::loadSelectors()
   fmt = selpics->format;
   // copy alpha values, don't use them
   SDL_SetAlpha(selpics, 0, 0);
-  for (i = 0; i < 4; i++)
+  for (i = 0; i < SMALL_SELECTOR_FRAMES; i++)
     {
       SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 
 					      fmt->BitsPerPixel, fmt->Rmask, 
@@ -2210,16 +2215,18 @@ void GraphicsCache::loadProdShields()
   // copy alpha values, don't use them
   SDL_SetAlpha(prodshieldpics, 0, 0);
   SDL_PixelFormat* fmt = prodshieldpics->format;
-  int size = 10; /*FIXME hardcoded size, should be in .defs? */
-  for (i = 0; i < 7; i++)
+  int xsize = PRODUCTION_SHIELD_WIDTH;
+  int ysize = PRODUCTION_SHIELD_HEIGHT;
+  for (i = 0; i < PRODUCTION_SHIELD_TYPES; i++)
     {
-      SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, size, size, 
+      SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 
 					      fmt->BitsPerPixel, fmt->Rmask, 
 					      fmt->Gmask, fmt->Bmask, 
 					      fmt->Amask);
-      prodshieldrect.x = i*size;
+      prodshieldrect.x = i * xsize;
       prodshieldrect.y = 0;
-      prodshieldrect.w = prodshieldrect.h = size;
+      prodshieldrect.w = xsize;
+      prodshieldrect.h = ysize;
       SDL_BlitSurface(prodshieldpics, &prodshieldrect, tmp, 0);
       d_prodshieldpic[i] = SDL_DisplayFormatAlpha(tmp);
       SDL_FreeSurface(tmp);
@@ -2237,15 +2244,15 @@ void GraphicsCache::loadMoveBonusPics()
   // copy alpha values, don't use them
   SDL_SetAlpha(movebonuspics, 0, 0);
   SDL_PixelFormat* fmt = movebonuspics->format;
-  int xsize = 32; /*FIXME hardcoded size, should be in .defs? */
-  int ysize = 10; /*FIXME hardcoded size, should be in .defs? */
-  for (i = 0; i < 6; i++)
+  int xsize = MOVE_BONUS_WIDTH;
+  int ysize = MOVE_BONUS_HEIGHT;
+  for (i = 0; i < MOVE_BONUS_TYPES; i++)
     {
       SDL_Surface* tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 
 					      fmt->BitsPerPixel, fmt->Rmask, 
 					      fmt->Gmask, fmt->Bmask, 
 					      fmt->Amask);
-      movebonusrect.x = i*xsize;
+      movebonusrect.x = i * xsize;
       movebonusrect.y = 0;
       movebonusrect.w = xsize;
       movebonusrect.h = ysize;
