@@ -1,3 +1,7 @@
+// Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
+// Copyright (C) 2004, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +14,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef GRAPHICS_CACHE_H
 #define GRAPHICS_CACHE_H
@@ -488,14 +493,14 @@ class GraphicsCache
         SDL_Surface* d_razedpic[MAX_PLAYERS + 1]; //+1 for neutral
         SDL_Surface* d_flagpic[MAX_STACK_SIZE];
         SDL_Surface* d_flagmask[MAX_STACK_SIZE];
-	SDL_Surface* d_selector[6];
-	SDL_Surface* d_selectormask[6];
-	SDL_Surface* d_smallselector[4];
-	SDL_Surface* d_smallselectormask[4];
-        SDL_Surface* d_prodshieldpic[7];
+	SDL_Surface* d_selector[SELECTOR_FRAMES];
+	SDL_Surface* d_selectormask[SELECTOR_FRAMES];
+	SDL_Surface* d_smallselector[SMALL_SELECTOR_FRAMES];
+	SDL_Surface* d_smallselectormask[SMALL_SELECTOR_FRAMES];
+        SDL_Surface* d_prodshieldpic[PRODUCTION_SHIELD_TYPES];
 	SDL_Surface* d_smallruinedcity;
 	SDL_Surface* d_smallhero;
-        SDL_Surface* d_movebonuspic[6];
+        SDL_Surface* d_movebonuspic[MOVE_BONUS_TYPES];
 	SDL_Surface* d_port;
 	SDL_Surface* d_explosion;
 	SDL_Surface* d_signpost;
