@@ -1,4 +1,11 @@
-//  This program is free software; you can redistribute it d_terrainand/or modify
+// Copyright (C) 2002 Vibhu Rishi
+// Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004 David Barnsdale
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2004, 2005 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//
+//  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
 //  (at your option) any later version.
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdlib.h>
 #include <iostream>
@@ -902,7 +910,8 @@ void MapGenerator::makeRoads()
       Vector<int> dest = c->getPos();
       Vector<int> src = (*it).getPos();
       //does it already have a road going to it?
-      if (Roadlist::getInstance()->getNearestRoad(dest, c->getSize() + 1))
+      if (Roadlist::getInstance()->getNearestObjectBefore(dest, 
+							  c->getSize() + 1))
 	continue;
 
       makeRoad(src, dest);
@@ -917,7 +926,6 @@ void MapGenerator::makeRoads()
 	continue;
       if (isAccessible(center->getPos(), (*it).getPos()) == false)
 	makeAccessible(center->getPos(), (*it).getPos());
-      //fixme: if one city is accessible, then all i have to do is make it to the nearest accessible city
     }
 
   Roadlist::deleteInstance();
