@@ -1,3 +1,8 @@
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2004, 2005 Ulf Lorenz
+// Copyright (C) 2005 Andrea Paternesi
+// Copyright (C) 2007 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <algorithm>
@@ -119,7 +125,7 @@ void Threatlist::addStack(Stack *stack)
     for (iterator it = begin(); it != end(); it++)
     {
         Threat *threat = *it;
-        if (threat->Near(stack->getPos(), stack->getPlayer()))
+        if (threat->Near(stack->getPos(), stack->getOwner()))
         {
             threat->addStack(stack);
             return;
