@@ -1,3 +1,8 @@
+// Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdlib.h>
 #include <sstream>
@@ -45,10 +51,31 @@ Action::Action(Type type)
 {
 }
 
+Action::Action(XML_Helper *helper)
+{
+  Uint32 t;
+  helper->getData(t, "type");
+  d_type = Action::Type(t);
+  helper->getData(d_player, "player");
+}
+
 Action::~Action()
 {
 }
 
+bool Action::save(XML_Helper* helper) const
+{
+    bool retval = true;
+
+    retval &= helper->openTag("action");
+    retval &= helper->saveData("type", d_type);
+    retval &= helper->saveData("player", d_player);
+    retval &= doSave(helper);
+    retval &= helper->closeTag();
+
+    return retval;
+}
+
 Action* Action::handle_load(XML_Helper* helper)
 {
     Uint32 t;
@@ -114,6 +141,12 @@ Action* Action::handle_load(XML_Helper* helper)
             return (new Action_DiplomacyProposal(helper));
 	case DIPLOMATIC_SCORE:
             return (new Action_DiplomacyScore(helper));
+        case END_TURN:
+            return (new Action_EndTurn(helper));
+        case CITY_CONQUER:
+            return (new Action_ConquerCity(helper));
+        case RECRUIT_HERO:
+            return (new Action_RecruitHero(helper));
     }
 
     return 0;
@@ -189,6 +222,18 @@ Action* Action::copy(const Action* a)
             return 
               (new Action_DiplomacyScore
                 (*dynamic_cast<const Action_DiplomacyScore*>(a)));
+        case END_TURN:
+            return 
+              (new Action_EndTurn
+                (*dynamic_cast<const Action_EndTurn*>(a)));
+        case CITY_CONQUER:
+            return 
+              (new Action_ConquerCity
+                (*dynamic_cast<const Action_ConquerCity*>(a)));
+        case RECRUIT_HERO:
+            return 
+              (new Action_RecruitHero
+                (*dynamic_cast<const Action_RecruitHero*>(a)));
     }
 
     return 0;
@@ -204,7 +249,7 @@ Action_Move::Action_Move()
 }
 
 Action_Move::Action_Move(XML_Helper* helper)
-    :Action(Action::STACK_MOVE)
+    :Action(helper)
 {
     helper->getData(d_stack, "stack");
 
@@ -229,16 +274,13 @@ std::string Action_Move::dump() const
     return s.str();
 }
 
-bool Action_Move::save(XML_Helper* helper) const
+bool Action_Move::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", d_type);
     retval &= helper->saveData("stack", d_stack);
     retval &= helper->saveData("x", d_dest.x);
     retval &= helper->saveData("y", d_dest.y);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -261,7 +303,7 @@ Action_Split::Action_Split()
 }
 
 Action_Split::Action_Split(XML_Helper* helper)
-    :Action(Action::STACK_SPLIT)
+    :Action(helper)
 {
     helper->getData(d_orig, "orig_stack");
     helper->getData(d_added, "new_stack");
@@ -294,7 +336,7 @@ std::string Action_Split::dump() const
     return s.str();
 }
 
-bool Action_Split::save(XML_Helper* helper) const
+bool Action_Split::doSave(XML_Helper* helper) const
 {
     bool retval = true;
     std::stringstream s;
@@ -303,12 +345,9 @@ bool Action_Split::save(XML_Helper* helper) const
         s <<d_armies_moved[i] <<" ";
     s <<d_armies_moved[MAX_STACK_SIZE - 1];
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::STACK_SPLIT);
     retval &= helper->saveData("orig_stack", d_orig);
     retval &= helper->saveData("new_stack", d_added);
     retval &= helper->saveData("moved", s.str());
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -346,7 +385,7 @@ Action_Fight::Action_Fight()
 }
 
 Action_Fight::Action_Fight(XML_Helper* helper)
-    :Action(Action::STACK_FIGHT)
+    :Action(helper)
 {
     std::string s;
     std::istringstream si;
@@ -359,6 +398,7 @@ Action_Fight::Action_Fight(XML_Helper* helper)
     si.str(s);
     while (si >> ui)
         d_attackers.push_back(ui);
+    si.clear();
 
     helper->getData(s, "defenders");
     si.str(s);
@@ -386,14 +426,12 @@ std::string Action_Fight::dump() const
     return s.str();
 }
 
-bool Action_Fight::save(XML_Helper* helper) const
+bool Action_Fight::doSave(XML_Helper* helper) const
 {
     std::stringstream si;
     std::list<Uint32>::const_iterator uit;
     bool retval = true;
     
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::STACK_FIGHT);
 
     // save the stack's ids
     for (uit = d_attackers.begin(); uit != d_attackers.end(); uit++)
@@ -415,8 +453,6 @@ bool Action_Fight::save(XML_Helper* helper) const
         retval &= helper->closeTag();
     }
 
-    retval &= helper->closeTag();
-
     return retval;
 }
 
@@ -431,9 +467,9 @@ bool Action_Fight::fillData(const Fight* f)
     list = f->getDefenders();
 
     for (it = list.begin(); it != list.end(); it++)
-        d_attackers.push_back((*it)->getId());
+        d_defenders.push_back((*it)->getId());
     
-    d_history   = f->getCourseOfEvents();
+    d_history = f->getCourseOfEvents();
 
     return true;
 }
@@ -446,6 +482,8 @@ bool Action_Fight::loadItem(std::string tag, XML_Helper* helper)
     helper->getData(item.id, "id");
     helper->getData(item.damage, "damage");
 
+    d_history.push_back(item);
+
     return true;
 }
 
@@ -458,7 +496,7 @@ Action_Join::Action_Join()
 }
 
 Action_Join::Action_Join(XML_Helper* helper)
-    :Action(Action::STACK_JOIN)
+    :Action(helper)
 {
     helper->getData(d_orig_id, "receiver");
     helper->getData(d_joining_id, "joining");
@@ -477,15 +515,12 @@ std::string Action_Join::dump() const
     return s.str();
 }
 
-bool Action_Join::save(XML_Helper* helper) const
+bool Action_Join::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", d_type);
     retval &= helper->saveData("receiver", d_orig_id);
     retval &= helper->saveData("joining", d_joining_id);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -517,7 +552,7 @@ Action_Ruin::Action_Ruin()
 }
 
 Action_Ruin::Action_Ruin(XML_Helper* helper)
-    :Action(Action::RUIN_SEARCH)
+    :Action(helper)
 {
     helper->getData(d_ruin, "ruin");
     helper->getData(d_stack, "seeker");
@@ -538,16 +573,13 @@ std::string Action_Ruin::dump() const
     return s.str();
 }
 
-bool Action_Ruin::save(XML_Helper* helper) const
+bool Action_Ruin::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::RUIN_SEARCH);
     retval &= helper->saveData("ruin", d_ruin);
     retval &= helper->saveData("seeker", d_stack);
     retval &= helper->saveData("searched", d_searched);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -571,7 +603,7 @@ Action_Temple::Action_Temple()
 }
 
 Action_Temple::Action_Temple(XML_Helper* helper)
-    :Action(Action::TEMPLE_SEARCH)
+    :Action(helper)
 {
     helper->getData(d_temple, "temple");
     helper->getData(d_stack, "stack");
@@ -590,15 +622,12 @@ std::string Action_Temple::dump() const
     return s.str();
 }
 
-bool Action_Temple::save(XML_Helper* helper) const
+bool Action_Temple::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::TEMPLE_SEARCH);
     retval &= helper->saveData("temple", d_temple);
     retval &= helper->saveData("stack", d_stack);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -621,7 +650,7 @@ Action_Occupy::Action_Occupy()
 }
 
 Action_Occupy::Action_Occupy(XML_Helper* helper)
-    :Action(Action::CITY_OCCUPY)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
 }
@@ -639,14 +668,11 @@ std::string Action_Occupy::dump() const
     return s.str();
 }
 
-bool Action_Occupy::save(XML_Helper* helper) const
+bool Action_Occupy::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_OCCUPY);
     retval &= helper->saveData("city", d_city);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -666,7 +692,7 @@ Action_Pillage::Action_Pillage()
 }
 
 Action_Pillage::Action_Pillage(XML_Helper* helper)
-    :Action(Action::CITY_PILLAGE)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
 }
@@ -682,14 +708,11 @@ std::string Action_Pillage::dump() const
     return s.str();
 }
 
-bool Action_Pillage::save(XML_Helper* helper) const
+bool Action_Pillage::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_PILLAGE);
     retval &= helper->saveData("city", d_city);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -709,7 +732,7 @@ Action_Sack::Action_Sack()
 }
 
 Action_Sack::Action_Sack(XML_Helper* helper)
-    :Action(Action::CITY_SACK)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
 }
@@ -725,14 +748,11 @@ std::string Action_Sack::dump() const
     return s.str();
 }
 
-bool Action_Sack::save(XML_Helper* helper) const
+bool Action_Sack::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_SACK);
     retval &= helper->saveData("city", d_city);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -752,7 +772,7 @@ Action_Raze::Action_Raze()
 }
 
 Action_Raze::Action_Raze(XML_Helper* helper)
-    :Action(Action::CITY_RAZE)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
 }
@@ -770,14 +790,11 @@ std::string Action_Raze::dump() const
     return s.str();
 }
 
-bool Action_Raze::save(XML_Helper* helper) const
+bool Action_Raze::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_RAZE);
     retval &= helper->saveData("city", d_city);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -797,7 +814,7 @@ Action_Upgrade::Action_Upgrade()
 }
 
 Action_Upgrade::Action_Upgrade(XML_Helper* helper)
-    :Action(Action::CITY_UPGRADE)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
 }
@@ -815,14 +832,11 @@ std::string Action_Upgrade::dump() const
     return s.str();
 }
 
-bool Action_Upgrade::save(XML_Helper* helper) const
+bool Action_Upgrade::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_UPGRADE);
     retval &= helper->saveData("city", d_city);
-    retval &= helper->closeTag();
 
     return retval;
 }
@@ -842,7 +856,7 @@ Action_Buy::Action_Buy()
 }
 
 Action_Buy::Action_Buy(XML_Helper* helper)
-    :Action(Action::CITY_BUY)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
     helper->getData(d_slot, "slot");
@@ -863,25 +877,22 @@ std::string Action_Buy::dump() const
     return s.str();
 }
 
-bool Action_Buy::save(XML_Helper* helper) const
+bool Action_Buy::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_BUY);
     retval &= helper->saveData("city", d_city);
     retval &= helper->saveData("slot", d_slot);
     retval &= helper->saveData("production", d_prod);
-    retval &= helper->closeTag();
 
     return retval;
 }
 
-bool Action_Buy::fillData(City* c, int slot, int prod)
+bool Action_Buy::fillData(City* c, int slot, const Army *prod)
 {
     d_city = c->getId();
     d_slot = slot;
-    d_prod = prod;
+    d_prod = prod->getType();
 
     return true;
 }
@@ -895,7 +906,7 @@ Action_Production::Action_Production()
 }
 
 Action_Production::Action_Production(XML_Helper* helper)
-    :Action(Action::CITY_PROD)
+    :Action(helper)
 {
     helper->getData(d_city, "city");
     helper->getData(d_prod, "production");
@@ -915,23 +926,20 @@ std::string Action_Production::dump() const
     return s.str();
 }
 
-bool Action_Production::save(XML_Helper* helper) const
+bool Action_Production::doSave(XML_Helper* helper) const
 {
     bool retval = true;
 
-    retval &= helper->openTag("action");
-    retval &= helper->saveData("type", Action::CITY_PROD);
     retval &= helper->saveData("city", d_city);
     retval &= helper->saveData("production", d_prod);
-    retval &= helper->closeTag();
 
     return retval;
 }
 
-bool Action_Production::fillData(City* c, int prod)
+bool Action_Production::fillData(City* c, int slot)
 {
     d_city = c->getId();
-    d_prod = prod;
+    d_prod = slot;
 
     return true;
 }
@@ -968,17 +976,19 @@ bool Action_Reward::load(std::string tag, XML_Helper *helper)
 }
 
 Action_Reward::Action_Reward(XML_Helper* helper)
-:Action(Action::REWARD)
+:Action(helper)
 {
-  helper->registerTag("reward", sigc::mem_fun((*this), &Action_Reward::load));
+  helper->getData(d_stack, "stack");
+  helper->registerTag("reward", sigc::mem_fun(this, &Action_Reward::load));
 }
 
 Action_Reward::~Action_Reward()
 {
 }
 
-bool Action_Reward::fillData(Reward* r)
+bool Action_Reward::fillData(Stack *s, Reward* r)
 {
+  d_stack = s->getId();
   d_reward = r;
   return true;
 }
@@ -993,12 +1003,11 @@ std::string Action_Reward::dump() const
   return s.str();
 }
 
-bool Action_Reward::save(XML_Helper* helper) const
+bool Action_Reward::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", Action::REWARD);
+  retval &= helper->saveData("stack", d_stack);
   if (d_reward->getType() == Reward::GOLD)
     static_cast<Reward_Gold*>(d_reward)->save(helper);
   else if (d_reward->getType() == Reward::ALLIES)
@@ -1009,7 +1018,6 @@ bool Action_Reward::save(XML_Helper* helper) const
     static_cast<Reward_Ruin*>(d_reward)->save(helper);
   else if (d_reward->getType() == Reward::MAP)
     static_cast<Reward_Map*>(d_reward)->save(helper);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1023,7 +1031,7 @@ Action_Quest::Action_Quest()
 }
 
 Action_Quest::Action_Quest(XML_Helper* helper)
-:Action(Action::QUEST)
+:Action(helper)
 {
 
   helper->getData(d_hero, "hero");
@@ -1046,17 +1054,14 @@ std::string Action_Quest::dump() const
   return ss.str();
 }
 
-bool Action_Quest::save(XML_Helper* helper) const
+bool Action_Quest::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("quest", d_questtype);
   retval &= helper->saveData("data", d_data);
   retval &= helper->saveData("victim_player", d_victim_player);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1105,7 +1110,7 @@ Action_Equip::Action_Equip()
 }
 
 Action_Equip::Action_Equip(XML_Helper* helper)
-:Action(Action::HERO_EQUIP)
+:Action(helper)
 {
   helper->getData(d_hero, "hero");
   helper->getData(d_item, "item");
@@ -1126,24 +1131,21 @@ std::string Action_Equip::dump() const
   return ss.str();
 }
 
-bool Action_Equip::save(XML_Helper* helper) const
+bool Action_Equip::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("item", d_item);
   retval &= helper->saveData("dest", d_slot);
-  retval &= helper->closeTag();
 
   return retval;
 }
 
-bool Action_Equip::fillData(Uint32 hero, Uint32 item, Action_Equip::Slot slot)
+bool Action_Equip::fillData(Hero *hero, Item *item, Action_Equip::Slot slot)
 {
-  d_hero = hero;
-  d_item = item;
+  d_hero = hero->getId();
+  d_item = item->getId();
   d_slot = slot;
 
   return true;
@@ -1158,7 +1160,7 @@ Action_Level::Action_Level()
 }
 
 Action_Level::Action_Level(XML_Helper* helper)
-:Action(Action::UNIT_ADVANCE)
+:Action(helper)
 {
   helper->getData(d_army, "army");
   helper->getData(d_stat, "stat");
@@ -1178,22 +1180,19 @@ std::string Action_Level::dump() const
   return ss.str();
 }
 
-bool Action_Level::save(XML_Helper* helper) const
+bool Action_Level::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("army", d_army);
   retval &= helper->saveData("stat", d_stat);
-  retval &= helper->closeTag();
 
   return retval;
 }
 
-bool Action_Level::fillData(Uint32 unit, Army::Stat raised)
+bool Action_Level::fillData(Army *unit, Army::Stat raised)
 {
-  d_army = unit;
+  d_army = unit->getId();
   d_stat = raised;
 
   return true;
@@ -1208,7 +1207,7 @@ Action_Disband::Action_Disband()
 }
 
 Action_Disband::Action_Disband(XML_Helper* helper)
-:Action(Action::STACK_DISBAND)
+:Action(helper)
 {
   helper->getData(d_stack, "stack");
 }
@@ -1226,14 +1225,11 @@ std::string Action_Disband::dump() const
   return s.str();
 }
 
-bool Action_Disband::save(XML_Helper* helper) const
+bool Action_Disband::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("stack", d_stack);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1253,7 +1249,7 @@ Action_ModifySignpost::Action_ModifySignpost()
 }
 
 Action_ModifySignpost::Action_ModifySignpost(XML_Helper* helper)
-:Action(Action::MODIFY_SIGNPOST)
+:Action(helper)
 {
   helper->getData(d_signpost, "signpost");
   helper->getData(d_message, "message");
@@ -1272,15 +1268,12 @@ std::string Action_ModifySignpost::dump() const
   return s.str();
 }
 
-bool Action_ModifySignpost::save(XML_Helper* helper) const
+bool Action_ModifySignpost::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("signpost", d_signpost);
   retval &= helper->saveData("message", d_message);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1301,7 +1294,7 @@ Action_RenameCity::Action_RenameCity()
 }
 
 Action_RenameCity::Action_RenameCity(XML_Helper* helper)
-:Action(Action::CITY_RENAME)
+:Action(helper)
 {
   helper->getData(d_city, "city");
   helper->getData(d_name, "name");
@@ -1320,15 +1313,12 @@ std::string Action_RenameCity::dump() const
   return s.str();
 }
 
-bool Action_RenameCity::save(XML_Helper* helper) const
+bool Action_RenameCity::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("city", d_city);
   retval &= helper->saveData("name", d_name);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1349,7 +1339,7 @@ Action_Vector::Action_Vector()
 }
 
 Action_Vector::Action_Vector(XML_Helper* helper)
-:Action(Action::CITY_VECTOR)
+:Action(helper)
 {
   helper->getData(d_city, "city");
   int i;
@@ -1373,16 +1363,13 @@ std::string Action_Vector::dump() const
   return s.str();
 }
 
-bool Action_Vector::save(XML_Helper* helper) const
+bool Action_Vector::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("city", d_city);
   retval &= helper->saveData("x", d_dest.x);
   retval &= helper->saveData("y", d_dest.y);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1403,7 +1390,7 @@ Action_FightOrder::Action_FightOrder()
 }
 
 Action_FightOrder::Action_FightOrder(XML_Helper* helper)
-:Action(Action::FIGHT_ORDER)
+:Action(helper)
 {
   std::string fight_order;
   std::stringstream sfight_order;
@@ -1437,12 +1424,10 @@ std::string Action_FightOrder::dump() const
   return s.str();
 }
 
-bool Action_FightOrder::save(XML_Helper* helper) const
+bool Action_FightOrder::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   std::stringstream fight_order;
   for (std::list<Uint32>::const_iterator it = d_order.begin();
        it != d_order.end(); it++)
@@ -1450,7 +1435,6 @@ bool Action_FightOrder::save(XML_Helper* helper) const
       fight_order << (*it) << " ";
     }
   retval &= helper->saveData("order", fight_order.str());
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1470,7 +1454,7 @@ Action_Resign::Action_Resign()
 }
 
 Action_Resign::Action_Resign(XML_Helper* helper)
-:Action(Action::RESIGN)
+:Action(helper)
 {
 }
 
@@ -1486,14 +1470,10 @@ std::string Action_Resign::dump() const
   return s.str();
 }
 
-bool Action_Resign::save(XML_Helper* helper) const
+bool Action_Resign::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
-  retval &= helper->closeTag();
-
   return retval;
 }
 
@@ -1511,7 +1491,7 @@ Action_Plant::Action_Plant()
 }
 
 Action_Plant::Action_Plant(XML_Helper* helper)
-:Action(Action::ITEM_PLANT)
+:Action(helper)
 {
   helper->getData(d_hero, "hero");
   helper->getData(d_item, "item");
@@ -1529,23 +1509,20 @@ std::string Action_Plant::dump() const
   return s.str();
 }
 
-bool Action_Plant::save(XML_Helper* helper) const
+bool Action_Plant::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("hero", d_hero);
   retval &= helper->saveData("item", d_item);
-  retval &= helper->closeTag();
 
   return retval;
 }
 
-bool Action_Plant::fillData(Uint32 hero, Uint32 item)
+bool Action_Plant::fillData(Hero *hero, Item *item)
 {
-  d_hero = hero;
-  d_item = item;
+  d_hero = hero->getId();
+  d_item = item->getId();
   return true;
 }
 
@@ -1558,7 +1535,7 @@ Action_Produce::Action_Produce()
 }
 
 Action_Produce::Action_Produce(XML_Helper* helper)
-:Action(Action::PRODUCE_UNIT)
+:Action(helper)
 {
   helper->getData(d_army_type, "army_type");
   helper->getData(d_city, "city");
@@ -1580,23 +1557,23 @@ std::string Action_Produce::dump() const
   return s.str();
 }
 
-bool Action_Produce::save(XML_Helper* helper) const
+bool Action_Produce::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("army_type", d_army_type);
   retval &= helper->saveData("city", d_city);
   retval &= helper->saveData("vectored", d_vectored);
-  retval &= helper->closeTag();
 
   return retval;
 }
 
-bool Action_Produce::fillData(Uint32 army_type, City *city, bool vectored)
+bool Action_Produce::fillData(const Army *army, City *city, bool vectored)
 {
-  d_army_type = army_type;
+  if (army == NULL)
+    d_army_type = -1;
+  else
+    d_army_type = army->getType();
   d_city = city->getId();
   d_vectored = vectored;
   return true;
@@ -1611,7 +1588,7 @@ Action_ProduceVectored::Action_ProduceVectored()
 }
 
 Action_ProduceVectored::Action_ProduceVectored(XML_Helper* helper)
-:Action(Action::PRODUCE_VECTORED_UNIT)
+:Action(helper)
 {
   helper->getData(d_army_type, "army_type");
   int i;
@@ -1634,16 +1611,13 @@ std::string Action_ProduceVectored::dump() const
   return s.str();
 }
 
-bool Action_ProduceVectored::save(XML_Helper* helper) const
+bool Action_ProduceVectored::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("army_type", d_army_type);
   retval &= helper->saveData("x", d_dest.x);
   retval &= helper->saveData("y", d_dest.y);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1664,7 +1638,7 @@ Action_DiplomacyState::Action_DiplomacyState()
 }
 
 Action_DiplomacyState::Action_DiplomacyState(XML_Helper* helper)
-:Action(Action::DIPLOMATIC_STATE)
+:Action(helper)
 {
   Uint32 diplomatic_state;
   helper->getData(d_opponent_id, "opponent_id");
@@ -1691,15 +1665,12 @@ std::string Action_DiplomacyState::dump() const
   return s.str();
 }
 
-bool Action_DiplomacyState::save(XML_Helper* helper) const
+bool Action_DiplomacyState::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
   retval &= helper->saveData("state", (Uint32)d_diplomatic_state);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1721,7 +1692,7 @@ Action_DiplomacyProposal::Action_DiplomacyProposal()
 }
 
 Action_DiplomacyProposal::Action_DiplomacyProposal(XML_Helper* helper)
-:Action(Action::DIPLOMATIC_PROPOSAL)
+:Action(helper)
 {
   Uint32 diplomatic_proposal;
   helper->getData(d_opponent_id, "opponent_id");
@@ -1749,15 +1720,12 @@ std::string Action_DiplomacyProposal::dump() const
   return s.str();
 }
 
-bool Action_DiplomacyProposal::save(XML_Helper* helper) const
+bool Action_DiplomacyProposal::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
   retval &= helper->saveData("proposal", (Uint32)d_diplomatic_proposal);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1779,7 +1747,7 @@ Action_DiplomacyScore::Action_DiplomacyScore()
 }
 
 Action_DiplomacyScore::Action_DiplomacyScore(XML_Helper* helper)
-:Action(Action::DIPLOMATIC_SCORE)
+:Action(helper)
 {
   helper->getData(d_opponent_id, "opponent_id");
   helper->getData(d_amount, "amount");
@@ -1801,15 +1769,12 @@ std::string Action_DiplomacyScore::dump() const
   return s.str();
 }
 
-bool Action_DiplomacyScore::save(XML_Helper* helper) const
+bool Action_DiplomacyScore::doSave(XML_Helper* helper) const
 {
   bool retval = true;
 
-  retval &= helper->openTag("action");
-  retval &= helper->saveData("type", d_type);
   retval &= helper->saveData("opponent_id", d_opponent_id);
   retval &= helper->saveData("amount", d_amount);
-  retval &= helper->closeTag();
 
   return retval;
 }
@@ -1820,3 +1785,145 @@ bool Action_DiplomacyScore::fillData(Player *opponent, int amount)
   d_amount = amount;
   return true;
 }
+
+//-----------------------------------------------------------------------------
+//Action_EndTurn
+
+Action_EndTurn::Action_EndTurn()
+:Action(Action::END_TURN)
+{
+}
+
+Action_EndTurn::Action_EndTurn(XML_Helper* helper)
+:Action(helper)
+{
+}
+
+Action_EndTurn::~Action_EndTurn()
+{
+}
+
+std::string Action_EndTurn::dump() const
+{
+  return "ending turn\n";
+}
+
+bool Action_EndTurn::doSave(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  return retval;
+}
+
+//-----------------------------------------------------------------------------
+//Action_ConquerCity
+
+Action_ConquerCity::Action_ConquerCity()
+  :Action(Action::CITY_CONQUER), d_city(0), d_stack(0)
+{
+}
+
+Action_ConquerCity::Action_ConquerCity(XML_Helper* helper)
+  :Action(helper)
+{
+    helper->getData(d_city, "city");
+    helper->getData(d_stack, "stack");
+}
+
+Action_ConquerCity::~Action_ConquerCity()
+{
+}
+
+std::string Action_ConquerCity::dump() const
+{
+    std::stringstream s;
+
+    s <<"City " <<d_city <<" occupied by " << d_stack << "\n";
+
+    return s.str();
+}
+
+bool Action_ConquerCity::doSave(XML_Helper* helper) const
+{
+    bool retval = true;
+
+    retval &= helper->saveData("city", d_city);
+    retval &= helper->saveData("stack", d_stack);
+
+    return retval;
+}
+
+bool Action_ConquerCity::fillData(City* c, Stack *s)
+{
+    d_city = c->getId();
+    d_stack = s->getId();
+    return true;
+}
+
+//-----------------------------------------------------------------------------
+//Action_RecruitHero
+
+Action_RecruitHero::Action_RecruitHero()
+  :Action(Action::RECRUIT_HERO), d_hero(0)
+{
+}
+
+Action_RecruitHero::Action_RecruitHero(XML_Helper* helper)
+  :Action(helper)
+{
+    helper->getData(d_city, "city");
+    helper->getData(d_cost, "cost");
+    helper->getData(d_allies, "allies");
+    helper->getData(d_ally_army_type, "ally_army_type");
+    helper->registerTag("hero", sigc::mem_fun(this, &Action_RecruitHero::load));
+}
+
+bool Action_RecruitHero::load(std::string tag, XML_Helper *helper)
+{
+    if (tag == "hero")
+      {
+	d_hero = new Hero(helper);
+
+	return true;
+      }
+    return false;
+}
+
+Action_RecruitHero::~Action_RecruitHero()
+{
+}
+
+std::string Action_RecruitHero::dump() const
+{
+    std::stringstream s;
+
+    s << "Hero " << d_hero->getId() << " recruited with " << d_allies << "allies\n";
+
+    return s.str();
+}
+
+bool Action_RecruitHero::doSave(XML_Helper* helper) const
+{
+    bool retval = true;
+
+    retval &= helper->saveData("city", d_city);
+    retval &= helper->saveData("cost", d_cost);
+    retval &= helper->saveData("allies", d_allies);
+    retval &= helper->saveData("ally_army_type", d_ally_army_type);
+    retval &= d_hero->save(helper);
+
+    return retval;
+}
+
+bool Action_RecruitHero::fillData(Hero* hero, City *city, int cost, int alliesCount, const Army *ally)
+{
+    d_hero = hero;
+    d_city = city->getId();
+    d_cost = cost;
+    d_allies = alliesCount;
+    if (alliesCount > 0)
+      d_ally_army_type = ally->getType();
+    else
+      d_ally_army_type = 0;
+    return true;
+}
