@@ -1,3 +1,10 @@
+// Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2004, 2006 Andrea Paternesi
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdlib.h>
 #include <algorithm>
@@ -78,14 +86,18 @@ bool AI_Fast::save(XML_Helper* helper) const
 
 bool AI_Fast::startTurn()
 {
+    // maniac AI's never recruit heroes, otherwise take everything we can get
+    if (!d_maniac)
+      maybeRecruitHero();
+    
     debug(getName() << ": AI_Fast::start_turn")
     debug("being in " <<(d_maniac?"maniac":"normal") <<" mode")
     debug((d_join?"":"not ") <<"joining armies")
 
-    RealPlayer::startTurn();
-
     d_analysis = new AI_Analysis(this);
     d_diplomacy = new AI_Diplomacy(this);
+
+    d_diplomacy->considerCuspOfWar();
     
     // this is a recursively-programmed quite staightforward AI,we just call:
     computerTurn();
@@ -100,56 +112,36 @@ bool AI_Fast::startTurn()
     return true;
 }
 
-bool AI_Fast::invadeCity(City* c)
+void AI_Fast::invadeCity(City* c)
 {
-    debug("Invaded city " <<c->getName())
+  debug("Invaded city " <<c->getName());
 
-    // There are two modes: maniac razes all cities, non-maniac just
-    // occupies them
-    if (d_maniac)
-    {
-        debug ("Razing it")
-        bool retval = cityRaze(c);
-        sinvadingCity.emit(c);
-        return retval;
-    }
-
-    debug("Occupying it")
-    bool retval = cityOccupy(c);
-    sinvadingCity.emit(c);
-    soccupyingCity.emit(c, getActivestack());
-
-    return retval;
+  // There are two modes: maniac razes all cities, non-maniac just
+  // occupies them
+  if (d_maniac)
+  {
+    debug ("Razing it");
+    cityRaze(c);
+  }
+  else
+  {
+    debug("Occupying it");
+    cityOccupy(c);
+  }
 }
 
-bool AI_Fast::recruitHero(Hero* hero, City *city, int cost)
+void AI_Fast::levelArmy(Army* a)
 {
-    debug("AI_Fast: hero offers service")
-
-    // maniac AI's never recruit heroes, otherwise take everything we can get
-    if (d_maniac)
-        return false;
-
-    return true;
-}
-
-bool AI_Fast::levelArmy(Army* a)
-{
-    if (!a->canGainLevel())
-        return false;
- 
     debug("Army raised a level, id = " <<a->getId())
     
     //advancing a level
     // increase the strength attack (uninnovative, but enough here)
     Army::Stat stat = Army::STRENGTH;
-    a->gainLevel(stat);
+    doLevelArmy(a, stat);
 
     Action_Level* item = new Action_Level();
-    item->fillData(a->getId(), stat);
-    d_actions.push_back(item);
-
-    return true;
+    item->fillData(a, stat);
+    addAction(item);
 }
 
 void AI_Fast::computerTurn()
@@ -262,6 +254,8 @@ void AI_Fast::computerTurn()
 		target2 = cl->getNearestForeignCity(s->getPos());
 		if (target1)
 		  target1_path->calculate (s, target1->getPos());
+		if (!target2)
+		  return; //it's game over and we're still moving
 		target2_path->calculate (s, target2->getPos());
 		if (!target1)
 		  target = target2;
@@ -272,7 +266,7 @@ void AI_Fast::computerTurn()
 
 		if (target == target2)
 		  {
-		    d_diplomacy->needNewEnemy(target->getPlayer());
+		    d_diplomacy->needNewEnemy(target->getOwner());
 		    // try to wait a turn until we're at war
 		    if (target1)
 		      target = target1;
@@ -364,9 +358,6 @@ void AI_Fast::computerTurn()
 		City *friendly_city = cl->getNearestFriendlyCity(s->getPos());
 		s->getPath()->calculate(s, friendly_city->getPos());
 		stackMove(s);
-		//FIXME: again, we should be able to reach any city,
-		//but we can't reach this one.  just go to the nearest city
-		//instead.  this is an error in map generation
 	      }
 
             if (!d_stacklist->getActivestack())
