@@ -1,3 +1,9 @@
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +16,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdlib.h>
 #include <algorithm>
@@ -57,8 +64,13 @@ AI_Smart::~AI_Smart()
 
 bool AI_Smart::startTurn()
 {
+  maybeRecruitHero();
+  
     debug(getName() << " start_turn")
-    RealPlayer::startTurn();
+
+    AI_Diplomacy diplomacy (this);
+
+    diplomacy.considerCuspOfWar();
 
     // the real stuff
     examineCities();
@@ -86,44 +98,29 @@ bool AI_Smart::startTurn()
     }
     d_stacklist->setActivestack(0);
 
-    AI_Diplomacy *diplomacy = new AI_Diplomacy (this);
-    diplomacy->makeProposals();
-    delete diplomacy;
+    diplomacy.makeProposals();
+    
     return true;
 }
 
-bool AI_Smart::invadeCity(City* c)
+void AI_Smart::invadeCity(City* c)
 {
     // always occupy an enemy city
-    bool retval = cityOccupy(c);
-    sinvadingCity.emit(c);
-    soccupyingCity.emit(c, getActivestack());
+    cityOccupy(c);
 
     // Update its production
     maybeBuyProduction(c);
     setBestProduction(c);
-    return retval;
 }
 
-bool AI_Smart::recruitHero(Hero* hero, City *city, int cost)
+void AI_Smart::levelArmy(Army* a)
 {
-    debug("AI_Smart: hero recruited")
-    return true;    //always recruit heroes
-}
-
-bool AI_Smart::levelArmy(Army* a)
-{
-    if (!a->canGainLevel())
-        return false;
-    
     Army::Stat stat = Army::STRENGTH;
-    a->gainLevel(stat);
+    doLevelArmy(a, stat);
 
     Action_Level* item = new Action_Level();
-    item->fillData(a->getId(), stat);
-    d_actions.push_back(item);
-    
-    return true;
+    item->fillData(a, stat);
+    addAction(item);
 }
 
 int AI_Smart::maybeBuyProduction(City *c)
@@ -168,12 +165,12 @@ int AI_Smart::setBestProduction(City *c)
     int scorebasic = -1;
 
     // we try to determine the most attractive basic production
-    for (int i = 0; i < c->getMaxNoOfBasicProd(); i++)
+    for (int i = 0; i < c->getMaxNoOfProductionBases(); i++)
     {
         if (c->getArmytype(i) == -1)    // no production in this slot
             continue;
 
-        const Army *proto = c->getArmy(i);
+        const Army *proto = c->getProductionBase(i);
         if (scoreArmyType(proto) > scorebasic)
         {
             selectbasic = i;
@@ -184,13 +181,13 @@ int AI_Smart::setBestProduction(City *c)
 
     select=selectbasic;
 
-    if (select != c->getProductionIndex())
+    if (select != c->getActiveProductionSlot())
     {
         cityChangeProduction(c, select);
         debug(getName() << " Set production to BASIC" << select << " in " << c->getName())
     }
 
-    return c->getProductionIndex();
+    return c->getActiveProductionSlot();
 }
 
 int AI_Smart::chooseArmyTypeToBuy(City *c)
@@ -198,13 +195,6 @@ int AI_Smart::chooseArmyTypeToBuy(City *c)
     int bestScore, bestIndex;
     Uint32 size = 0;
 
-    // Andrea: cannot understand why was addes this piece of code
-    //         i comment it
-    //if (c->//getProductionIndex() > 0)
-    //{
-    //    return -1;
-    //}
-    
     const Armysetlist* al = Armysetlist::getInstance();
 
     size = al->getSize(getArmyset());
@@ -232,7 +222,7 @@ int AI_Smart::chooseArmyTypeToBuy(City *c)
             continue;
 	}
         
-       if (c->hasProduction(proto->getType(), getArmyset())==false)
+       if (c->hasProductionBase(proto->getType(), getArmyset())==false)
        {
          debug("I can buy it " << i)
          int score = scoreArmyType(proto);
