@@ -1,3 +1,9 @@
+// Copyright (C) 2000, 2001, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2005 Andrea Paternesi
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +16,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <sstream>
@@ -21,14 +28,15 @@
 #include "xmlhelper.h"
 #include "stacklist.h"
 #include "templelist.h"
+#include "ucompose.hpp"
 
-//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
+//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
 
 sigc::signal<void, Army*> Army::sdying;
 
-Army::Army(const Army& a, Player* p)
-    :d_type(a.d_type), d_armyset(a.d_armyset), d_pixmap(0),
+Army::Army(const Army& a, Player* p, bool for_template)
+    :Ownable(p), d_type(a.d_type), d_armyset(a.d_armyset), d_pixmap(0),
      d_mask(0), d_name(a.d_name), d_description(a.d_description), 
      d_production(a.d_production),
      d_production_cost(a.d_production_cost), d_upkeep(a.d_upkeep),
@@ -39,7 +47,7 @@ Army::Army(const Army& a, Player* p)
      d_sight(a.d_sight),
      d_xp_value(a.d_xp_value), d_move_bonus(a.d_move_bonus),
      d_army_bonus(a.d_army_bonus), d_ship(a.d_ship), d_gender(a.d_gender), 
-     d_player(p), d_id(a.d_id), d_hp(a.d_hp), d_moves(a.d_moves), d_xp(a.d_xp),
+     d_id(a.d_id), d_hp(a.d_hp), d_moves(a.d_moves), d_xp(a.d_xp),
      d_level(a.d_level), d_grouped(a.d_grouped),
      d_battles_number(a.d_battles_number), d_number_hashit(a.d_number_hashit),
      d_number_hasbeenhit(a.d_number_hasbeenhit), 
@@ -47,9 +55,10 @@ Army::Army(const Army& a, Player* p)
      d_visitedTemples(a.d_visitedTemples), d_hero(a.d_hero)
 {
     // if we have been copied from an army prototype, initialise several values
-    if (d_id == 0)
+    if (d_id == 0 && !for_template)
     {
         d_id = fl_counter->getNextId();
+        debug("army created with id " << d_id);
         d_hp = d_max_hp;
         d_moves = d_max_moves;
 	d_max_moves_multiplier = 1;
@@ -62,7 +71,7 @@ Army::Army(const Army& a, Player* p)
 }
 
 Army::Army()
-  :d_pixmap(0), d_mask(0), d_name("Untitled"), d_description(""),
+  :Ownable((Player *)0), d_pixmap(0), d_mask(0), d_name("Untitled"), d_description(""),
     d_production(0), d_production_cost(0), d_upkeep(0), d_strength(0),
     d_max_hp(0), d_max_moves(0), d_max_moves_multiplier(1), d_sight(0), 
     d_gender(NONE), d_level(1), d_defends_ruins(false), d_awardable(false), 
@@ -71,11 +80,10 @@ Army::Army()
 }
 
 Army::Army(XML_Helper* helper, enum ArmyContents contents)
-  :d_pixmap(0), d_mask(0), d_name(""), d_description(""), d_ship(false),
-   d_gender(NONE), d_player(0), 
-   d_id(0), d_xp(0), d_level(1), d_grouped(true),
-   d_number_hashit(0), d_number_hasbeenhit(0), d_defends_ruins(false),
-   d_awardable(false), d_hero(false)
+  :Ownable((XML_Helper*) 0), d_pixmap(0), d_mask(0), d_name(""), 
+    d_description(""), d_ship(false), d_gender(NONE), d_id(0), d_xp(0), 
+    d_level(1), d_grouped(true), d_number_hashit(0), d_number_hasbeenhit(0), 
+    d_defends_ruins(false), d_awardable(false), d_hero(false)
 {
     d_max_hp = 2;
     d_visitedTemples.clear();
@@ -219,7 +227,7 @@ SDL_Surface* Army::getPixmap() const
     
     //use the GraphicsCache to get a picture of the army's armyset_army
     return GraphicsCache::getInstance()->getArmyPic(d_armyset, d_type,
-                                         d_player, d_medal_bonus);
+                                         d_owner, d_medal_bonus);
 }
 
 Uint32 Army::getStat(Stat stat, bool modified) const
@@ -265,7 +273,7 @@ void Army::resetMoves()
 bool Army::bless()
 {
   bool visited = false;
-  Stack *stack = d_player->getStacklist()->getActivestack();
+  Stack *stack = d_owner->getStacklist()->getActivestack();
   Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());
 
   if (!temple)
@@ -509,7 +517,12 @@ void  Army::printAllDebugInfo() const
     std::cerr << "battle number = "     << d_battles_number    << std::endl;
     std::cerr << "has hit = "           << d_number_hashit     << std::endl;
     std::cerr << "has been hit = "      << d_number_hasbeenhit << std::endl;
-//XXX FIXME: show the visited temple info
+    std::stringstream temples;
+    std::list<unsigned int>::const_iterator tit = d_visitedTemples.begin();
+    std::list<unsigned int>::const_iterator tend = d_visitedTemples.end();
+    for(;tit != tend;++tit)
+        temples << (*tit) << " ";
+    std::cerr << "visited temples with ids = " << temples << std::endl;
 }
 
 
@@ -527,7 +540,7 @@ void Army::copyVals(const Army* a)
     d_defends_ruins = a->getDefendsRuins();
     d_awardable = a->getAwardable();
     d_visitedTemples = a->d_visitedTemples;
-    d_player = a->d_player;
+    setOwner(a->getOwner());
     d_hero = a->d_hero;
 }
 
@@ -568,3 +581,47 @@ void Army::setMask(SDL_Surface* mask)
     SDL_FreeSurface(d_mask);
   d_mask = mask;
 }
+
+std::string Army::getArmyBonusDescription() const
+{
+  Uint32 bonus = getStat(Army::ARMY_BONUS, false);
+  Glib::ustring s = "";
+  if (bonus & Army::ADD1STRINOPEN)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : "& ", 
+			  _("+1 str in open"));
+  if (bonus & Army::ADD2STRINOPEN)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : "& ", 
+			  _("+2 str in open"));
+  if (bonus & Army::ADD1STRINFOREST)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : "& ", 
+			  _("+1 str in woods"));
+  if (bonus & Army::ADD1STRINHILLS)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("+1 str in hills"));
+  if (bonus & Army::ADD1STRINCITY)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("+1 str in city"));
+  if (bonus & Army::ADD2STRINCITY)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("+2 str in city"));
+  if (bonus & Army::ADD1STACKINHILLS)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("+1 stack in hills"));
+  if (bonus & Army::SUBALLCITYBONUS)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("Cancel city bonus"));
+  if (bonus & Army::SUB1ENEMYSTACK)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("-1 enemy stack"));
+  if (bonus & Army::ADD1STACK)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", _("+1 stack"));
+  if (bonus & Army::ADD2STACK)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", _("+2 stack"));
+  if (bonus & Army::SUBALLNONHEROBONUS)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("cancel non-hero"));
+  if (bonus & Army::SUBALLHEROBONUS)
+    s += String::ucompose(_("%1%2"), s == "" ? " " : " & ", 
+			  _("cancel hero"));
+  return s;
+}
