@@ -1,3 +1,8 @@
+// Copyright (C) 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2005 Andrea Paternesi
+// Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <expat.h>
@@ -74,16 +80,17 @@ Armysetlist::~Armysetlist()
     delete (*it);
 
     // remove all army entries
-    for (ArmyMap::iterator it = d_armies.begin(); it != d_armies.end(); it++)
+    for (ArmyPrototypeMap::iterator it = d_armies.begin(); 
+	 it != d_armies.end(); it++)
         while (!(*it).second.empty())
             delete ((*it).second)[0];
 }
 
 Army* Armysetlist::getArmy(Uint32 id, Uint32 index) const
 {
-    // always use ArmyMap::find for searching, else a default entry is created,
-    // which can produce really bad results!!
-    ArmyMap::const_iterator it = d_armies.find(id);
+    // always use ArmyProtoMap::find for searching, else a default entry is 
+    // created, which can produce really bad results!!
+    ArmyPrototypeMap::const_iterator it = d_armies.find(id);
 
     // armyset does not exist
     if (it == d_armies.end())
@@ -98,7 +105,7 @@ Army* Armysetlist::getArmy(Uint32 id, Uint32 index) const
 
 Uint32 Armysetlist::getSize(Uint32 id) const
 {
-    ArmyMap::const_iterator it = d_armies.find(id);
+    ArmyPrototypeMap::const_iterator it = d_armies.find(id);
 
     // armyset does not exist
     if (it == d_armies.end())
