@@ -1,3 +1,10 @@
+// Copyright (C) 2003 Michael Bartl
+// Copyright (C) 2003, 2004, 2005, 2006, 2007 Ulf Lorenz
+// Copyright (C) 2004, 2005 Bryan Duff
+// Copyright (C) 2004, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -202,6 +210,11 @@ Vector<int> BigMap::mouse_pos_to_tile_offset(Vector<int> pos)
     return (view_pos + pos) % ts;
 }
 
+MapTipPosition BigMap::map_tip_position(Vector<int> tile)
+{
+  return map_tip_position (Rectangle(tile.x, tile.y, 1, 1)); 
+}
+
 MapTipPosition BigMap::map_tip_position(Rectangle tile_area)
 {
     // convert area to pixels on the SDL screen
@@ -251,7 +264,7 @@ MapTipPosition BigMap::map_tip_position(Rectangle tile_area)
     return m;
 }
 
-void BigMap::blit_if_inside_buffer(const Object &obj, SDL_Surface *image)
+void BigMap::blit_if_inside_buffer(const Location &obj, SDL_Surface *image)
 {
     if (is_overlapping(buffer_view, obj.get_area()))
     {
@@ -372,6 +385,10 @@ bit position.
 110      100      011     000
 (105)    (43)     (232)   (15)
 
+011      110
+1x1 = 14 1x1 = 15
+110      011
+(126)    (219)
 
 special note:
 none of these sets contain a so-called "lone" tile.
@@ -379,7 +396,6 @@ a lone tile is a fogged tile surrounded by two unfogged tiles on either side.
  */
 void BigMap::drawFogTile (int x, int y)
 {
-  FogMap *fogmap = Playerlist::getActiveplayer()->getFogMap();
   int idx = 0;
   int count = 0;
   bool foggyTile;
@@ -397,12 +413,7 @@ void BigMap::drawFogTile (int x, int y)
             Vector<int> pos;
             pos.x = i;
             pos.y = j;
-            foggyTile = fogmap->getFogTile(pos) == FogMap::CLOSED;
-            if (foggyTile)
-              {
-                if (fogmap->isLoneFogTile(pos) == true)
-                  foggyTile = false;
-              }
+            foggyTile = FogMap::isFogged(pos);
           }
         if (foggyTile)
           {
@@ -438,8 +449,9 @@ void BigMap::drawFogTile (int x, int y)
     case  31: case 63: case 159: case 191: type = 10; break;
     case 214: case 215: case 246: case 247: type = 11; break;
     case 107: case 111: case 235: case 239: type = 12; break;
+    case 126: type = 14; break;
+    case 219: type = 15; break;
     case 255: type = 13; break;
-
     }
   if (type)
     {
@@ -471,7 +483,7 @@ void BigMap::draw_stack(Stack *s)
   GraphicsCache *gc = GraphicsCache::getInstance();
   int tilesize = GameMap::getInstance()->getTileset()->getTileSize();
   Vector<int> p = s->getPos();
-  Player *player = s->getPlayer();
+  Player *player = s->getOwner();
   int army_tilesize;
 
   // check if the object lies in the viewed part of the map
@@ -645,12 +657,10 @@ void BigMap::draw_buffer()
 	  {
 	    if (x < GameMap::getWidth() && y < GameMap::getHeight())
 	      {
-		Player *p = Playerlist::getActiveplayer();
 		Vector<int> pos;
 		pos.x = x;
 		pos.y = y;
-		if (p->getFogMap()->getFogTile(pos) == FogMap::CLOSED &&
-		    p->getFogMap()->isLoneFogTile(pos) == false)
+		if (FogMap::isFogged(pos))
 		  drawFogTile (x, y);
 	      }
 	  }
@@ -659,3 +669,21 @@ void BigMap::draw_buffer()
     after_draw();
 
 }
+
+void BigMap::blank ()
+{
+    // fog it up
+    for (int x = buffer_view.x; x < buffer_view.x + buffer_view.w; x++)
+      {
+	for (int y = buffer_view.y; y < buffer_view.y + buffer_view.h; y++)
+	  {
+	    if (x < GameMap::getWidth() && y < GameMap::getHeight())
+	      {
+		Vector<int> pos;
+		pos.x = x;
+		pos.y = y;
+		drawFogTile (x, y);
+	      }
+	  }
+      }
+}
