@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,47 +12,55 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef BRIDGELIST_H
 #define BRIDGELIST_H
 
 #include <sigc++/trackable.h>
-#include "ObjectList.h"
+#include "LocationList.h"
 #include "bridge.h"
 
-/** The bridgelist just keeps track of the bridges located on the game map. It
-  * is also implemented as a singleton since many classes use it for looking up
-  * bridges.
-  */
-
-class Bridgelist : public ObjectList<Bridge>, public sigc::trackable
+//! A list of Bridge objects located on the game map.
+/** 
+ * The bridgelist keeps track of the bridges located on the game map. It
+ * is implemented as a singleton because many classes use it for looking up
+ * bridges.
+ */
+class Bridgelist : public LocationList<Bridge>, public sigc::trackable
 {
     public:
-        //! Return the singleton instance. Create a new one if needed.
+        //! Return the singleton instance.  Create a new one if needed.
         static Bridgelist* getInstance();
 
-        //! Load the singleton instance with the given savegame
+        //! Load the singleton instance from the opened saved-game file.
         static Bridgelist* getInstance(XML_Helper* helper);
 
-        //! Explicitly delete the singleton instance
+        //! Explicitly delete the singleton instance.
         static void deleteInstance();
-        
 
-        //! Saves the game data. See XML_Helper for details.
+        //! Saves the list of Bridge objects to the opened saved-game file.
         bool save(XML_Helper* helper) const;
 
     protected:
-        //! Default constructor
+        //! Default constructor.
         Bridgelist();
 
-        //! Loading constructor
+        //! Loading constructor.
+	/**
+	 * Load the list of Bridge objects from the opened saved-game file.
+	 *
+	 * @param helper  The opened saved-game file to load the Bridge objects
+	 *                from.
+	 */
         Bridgelist(XML_Helper* helper);
 
     private:
-        //! Callback for loading
+        //! Callback for loading Bridge objects into the list of bridges.
         bool load(std::string tag, XML_Helper* helper);
 
+        //! A static pointer for the singleton instance.
         static Bridgelist* s_instance;
 };
 
