@@ -1,3 +1,10 @@
+//  Copyright (C) 2000, 2001, 2003 Michael Bartl
+//  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+//  Copyright (C) 2002 Mark L. Amidon
+//  Copyright (C) 2005 Andrea Paternesi
+//  Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//  Copyright (C) 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <stdio.h>
 #include <algorithm>
@@ -35,49 +43,48 @@ using namespace std;
 #define debug(x)
 
 City::City(Vector<int> pos, string name, Uint32 gold)
-    :Location(name, pos, 2), d_player(0), d_numbasic(4),
-     d_production(-1),
+    :Ownable((Player *)0), Location(pos, 2), Renamable(name),
+    d_numprodbase(MAX_PRODUCTION_SLOTS_IN_A_CITY),
+     d_active_production_slot(-1),
      d_duration(-1), d_gold(gold),
      d_defense_level(1), d_burnt(false), d_vectoring(false),
      d_capital(false), d_capital_owner(0)
 
 {
     // Initialise armytypes
-    for (int i = 0; i < 4; i++)
-        d_basicprod[i] = NULL; 
+    for (int i = 0; i < getMaxNoOfProductionBases(); i++)
+        d_prodbase[i] = NULL; 
 
     // set the tiles to city type
     for (int i = 0; i < 2; i++)
         for (int j = 0; j < 2; j++)
-            GameMap::getInstance()->getTile(d_pos.x + i, d_pos.y + j)
+            GameMap::getInstance()->getTile(getPos().x + i, getPos().y + j)
                                   ->setBuilding(Maptile::CITY);
     d_vector.x=-1;
     d_vector.y=-1; 
 }
 
 City::City(XML_Helper* helper)
-    :Location(helper, 2), d_numbasic(4)
+    :Ownable(helper), Location(helper, 2), Renamable(helper), 
+    d_numprodbase(MAX_PRODUCTION_SLOTS_IN_A_CITY)
 {
   //note: the armies get loaded in citylist
 
     //initialize the city
-    Uint32 ui;
 
-    for (int i = 0; i < 4; i++)
-      d_basicprod[i] = NULL;
-
-    helper->getData(ui, "owner");
-    d_player = Playerlist::getInstance()->getPlayer(ui);
+    for (int i = 0; i < getMaxNoOfProductionBases(); i++)
+      d_prodbase[i] = NULL;
 
     helper->getData(d_defense_level, "defense");
     
-    helper->getData(d_production, "production");
+    helper->getData(d_active_production_slot, "active_production_slot");
     helper->getData(d_duration, "duration");
     helper->getData(d_gold, "gold");
     helper->getData(d_burnt, "burnt");
     helper->getData(d_capital, "capital");
     if (d_capital)
       {
+	Uint32 ui;
         helper->getData(ui, "capital_owner");
         d_capital_owner = Playerlist::getInstance()->getPlayer(ui);
       }
@@ -99,32 +106,33 @@ City::City(XML_Helper* helper)
     //mark the positions on the map as being occupied by a city
     for (unsigned int i = 0; i < d_size; i++)
         for (unsigned int j = 0; j < d_size; j++)
-            GameMap::getInstance()->getTile(d_pos.x+i, d_pos.y+j)
+            GameMap::getInstance()->getTile(getPos().x+i, getPos().y+j)
                                   ->setBuilding(Maptile::CITY);
     
 }
 
 City::City(const City& c)
-    :Location(c), d_player(c.d_player), d_numbasic(c.d_numbasic),
-    d_production(c.d_production), d_duration(c.d_duration), d_gold(c.d_gold), 
+    :Ownable(c), Location(c), Renamable(c), d_numprodbase(c.d_numprodbase),
+    d_active_production_slot(c.d_active_production_slot), 
+    d_duration(c.d_duration), d_gold(c.d_gold), 
     d_defense_level(c.d_defense_level), d_burnt(c.d_burnt),
     d_vectoring(c.d_vectoring),d_vector(c.d_vector), d_capital(c.d_capital), 
     d_capital_owner(c.d_capital_owner)
 {
-    for (int i = 0; i < 4; i++)
+    for (int i = 0; i < getMaxNoOfProductionBases(); i++)
       {
-	if (c.d_basicprod[i] != NULL)
-	  d_basicprod[i] = new Army(*c.d_basicprod[i]);
+	if (c.d_prodbase[i] != NULL)
+	  d_prodbase[i] = new Army(*c.d_prodbase[i]);
 	else
-	  d_basicprod[i] = NULL;
+	  d_prodbase[i] = NULL;
       }
 }
 
 City::~City()
 {
-    for (int i = 0; i < 4; i++)
-      if (d_basicprod[i])
-	delete d_basicprod[i];
+    for (int i = 0; i < getMaxNoOfProductionBases(); i++)
+      if (d_prodbase[i])
+	delete d_prodbase[i];
 }
 
 bool City::save(XML_Helper* helper) const
@@ -137,12 +145,13 @@ bool City::save(XML_Helper* helper) const
 
     retval &= helper->openTag("city");
     retval &= helper->saveData("id", d_id);
-    retval &= helper->saveData("name", d_name);
-    retval &= helper->saveData("x", d_pos.x);
-    retval &= helper->saveData("y", d_pos.y);
-    retval &= helper->saveData("owner", d_player->getId());
+    retval &= helper->saveData("x", getPos().x);
+    retval &= helper->saveData("y", getPos().y);
+    retval &= helper->saveData("name", getName());
+    retval &= helper->saveData("owner", d_owner->getId());
     retval &= helper->saveData("defense", d_defense_level);
-    retval &= helper->saveData("production", d_production);
+    retval &= helper->saveData("active_production_slot", 
+			       d_active_production_slot);
     retval &= helper->saveData("duration", d_duration);
     retval &= helper->saveData("gold", d_gold);
     retval &= helper->saveData("burnt", d_burnt);
@@ -151,42 +160,42 @@ bool City::save(XML_Helper* helper) const
       retval &= helper->saveData("capital_owner", d_capital_owner->getId());
     retval &= helper->saveData("vectoring", svect.str());
 
-    for (int i = 0; i < d_numbasic; i++)
+    for (int i = 0; i < d_numprodbase; i++)
       {
-	if (d_basicprod[i] == NULL)
+	if (d_prodbase[i] == NULL)
 	  continue;
-	retval &= d_basicprod[i]->save(helper, Army::PRODUCTION_BASE);
+	retval &= d_prodbase[i]->save(helper, Army::PRODUCTION_BASE);
       }
     retval &= helper->closeTag();
     return retval;
 }
 
-int City::getNoOfBasicProd()
+int City::getNoOfProductionBases()
 {
   int i, max = 0;
-  for (i = 0; i < this->getMaxNoOfBasicProd(); i++)
+  for (i = 0; i < this->getMaxNoOfProductionBases(); i++)
     {
-      if (this->getArmy(i))
+      if (getProductionBase(i))
         max++;
     }
   return max;
 }
 
-void City::setProduction(int index)
+void City::setActiveProductionSlot(int index)
 {
     if (index == -1)
     {
-        d_production = index;
+        d_active_production_slot = index;
         d_duration = -1;
         return;
     }
 
     // return on wrong data
-    if (((index >= d_numbasic)) || (index >= 0 && getArmytype(index) == -1))
+    if (((index >= d_numprodbase)) || (index >= 0 && getArmytype(index) == -1))
         return;
 
-    d_production = index;
-    const Army* a = getArmy(index);
+    d_active_production_slot = index;
+    const Army* a = getProductionBase(index);
 
     // set the duration to produce this armytype
     if (a)
@@ -226,11 +235,11 @@ int City::getFreeBasicSlot()
 {
      int index=-1;
 
-     debug(d_name << " BASIC SLOTS=" << d_numbasic) 
-     for (int i = 0; i < d_numbasic; i++)
+     debug(getName()<< " BASIC SLOTS=" << d_numprodbase) 
+     for (int i = 0; i < d_numprodbase; i++)
      {
-         debug(d_name << " Index Value=" << d_basicprod[i])
-         if (d_basicprod[i] == NULL)
+         debug(getName()<< " Index Value=" << d_prodbase[i])
+         if (d_prodbase[i] == NULL)
          {
              index=i;
              return i;
@@ -240,46 +249,21 @@ int City::getFreeBasicSlot()
      return index;
 }
 
-bool City::isAlreadyBought(const Army * army)
+bool City::hasProductionBase(const Army * army)
 {
-    int max=-1;
-
-    max=d_numbasic;
- 
-    for (int i = 0; i < max; i++)
-    {
-        
-        debug("Value of production slot=" << d_basicprod[i] << " of index " << i << " max index=" << max)
-        if (d_basicprod[i] != NULL)
-	{
-	    int type1 = army->getType();
-	    int type2 = d_basicprod[i]->getType();
-	    string name = d_basicprod[i]->getName();
-           
-            debug("army in list " << type2 << " - " << name)          
-	    debug("army in city " << type1 << " - " << army->getName())          
- 
-	    if(type1 == type2) return true;
-	}
-        else debug("basic prod was nil index=" << i);
-    }
- 
-    return false;
+  return hasProductionBase(army->getType(), army->getArmyset());
 }
 
-bool City::addBasicProd(int index, Army *army)
+void City::addProductionBase(int index, Army *army)
 {
-    if (index >= d_numbasic)
-        return false;
-
-    army->setPlayer(this->getPlayer());
+    army->setOwner(d_owner);
 
     if (index < 0)
     {
         // try to find an unoccupied production slot. If there is none, pick 
         // the slot with the highest index.
-        for (int i = 0; i < d_numbasic; i++)
-            if (d_basicprod[i] == NULL)
+        for (int i = 0; i < d_numprodbase; i++)
+            if (d_prodbase[i] == NULL)
             {
                 index = i;
                 break;
@@ -287,43 +271,42 @@ bool City::addBasicProd(int index, Army *army)
 
         if (index < 0)
         {
-            index = d_numbasic - 1;
+            index = d_numprodbase - 1;
         }
     }
     
     bool restore_production = false;
-    if (d_production == index)
+    if (d_active_production_slot == index)
       restore_production = true;
-    removeBasicProd(index);
-    d_basicprod[index] = army;
+    removeProductionBase(index);
+    d_prodbase[index] = army;
     if (restore_production)
-      setProduction(index);
-    return true;
+      setActiveProductionSlot(index);
 }
 
-void City::removeBasicProd(int index)
+void City::removeProductionBase(int index)
 {
-    if ((index < 0) || (index > 3))
+    if ((index < 0) || (index > (getMaxNoOfProductionBases() - 1)))
         return;
 
-    if (d_basicprod[index])
-      delete d_basicprod[index];
-    d_basicprod[index] = NULL;
+    if (d_prodbase[index])
+      delete d_prodbase[index];
+    d_prodbase[index] = NULL;
 
-    if (d_production == index)
-        setProduction(-1);
+    if (d_active_production_slot == index)
+        setActiveProductionSlot(-1);
 }
 
 void City::conquer(Player* newowner)
 {
   Citylist::getInstance()->stopVectoringTo(this);
-    d_player = newowner;
+  setOwner(newowner);
 
     // remove vectoring info 
     setVectoring(Vector<int>(-1,-1));
 
-    for (int j = 0; j < getNoOfBasicProd(); j++)
-      d_basicprod[j]->setPlayer(newowner);
+    for (int j = 0; j < getNoOfProductionBases(); j++)
+      d_prodbase[j]->setOwner(newowner);
 
     deFog(newowner);
 
@@ -367,17 +350,17 @@ bool armyCompareStrength (const Army *lhs, const Army *rhs)
 void City::sortProduction()
 {
   //sort them by strength
-  if (getNoOfBasicProd() > 1)
+  if (getNoOfProductionBases() > 1)
     {
       std::list<Army*> productibles;
       int j;
-      for (j = 0; j < getNoOfBasicProd(); j++)
-        productibles.push_back(d_basicprod[j]);
+      for (j = 0; j < getNoOfProductionBases(); j++)
+        productibles.push_back(d_prodbase[j]);
       productibles.sort(armyCompareStrength);
       j = 0;
       for (std::list<Army*>::iterator it = productibles.begin();
 	   it != productibles.end(); it++, j++)
-       	d_basicprod[j] = *it;
+       	d_prodbase[j] = *it;
     }
   return;
 }
@@ -385,11 +368,11 @@ void City::sortProduction()
 void City::setRandomArmytypes(bool produce_allies, int likely)
 {
   //remove armies any that happen to be being produced
-  for (int i = 0; i < getMaxNoOfBasicProd(); i++)
-    removeBasicProd(i);
+  for (int i = 0; i < getMaxNoOfProductionBases(); i++)
+    removeProductionBase(i);
 
   const Armysetlist* al = Armysetlist::getInstance();
-  Uint32 set = getPlayer()->getArmyset();
+  Uint32 set = d_owner->getArmyset();
 
   int army_type;
   int num = rand() % 10;
@@ -409,7 +392,7 @@ void City::setRandomArmytypes(bool produce_allies, int likely)
     }
   Army *army = new Army (*template_army);
   randomlyImproveOrDegradeArmy(army);
-  addBasicProd(0, army);
+  addProductionBase(0, army);
 
   if ((rand() % 10) < 3 && !isCapital() && likely < 1 )
     {
@@ -428,7 +411,7 @@ void City::setRandomArmytypes(bool produce_allies, int likely)
     }
   army = new Army (*template_army);
   randomlyImproveOrDegradeArmy(army);
-  addBasicProd(1, army);
+  addProductionBase(1, army);
 
   if ((rand() % 10) < 4 && !isCapital() && likely < 2)
     {
@@ -450,7 +433,7 @@ void City::setRandomArmytypes(bool produce_allies, int likely)
     }
   army = new Army (*template_army);
   randomlyImproveOrDegradeArmy(army);
-  addBasicProd(2, army);
+  addProductionBase(2, army);
 
   if ((rand() % 10) < 6 && !isCapital() && likely < 3)
     {
@@ -469,36 +452,38 @@ void City::setRandomArmytypes(bool produce_allies, int likely)
     }
   army = new Army (*template_army);
   randomlyImproveOrDegradeArmy(army);
-  addBasicProd(3, army);
+  addProductionBase(3, army);
   sortProduction();
 }
 
-void City::produceStrongestArmy()
+void City::produceStrongestProductionBase()
 {
-  debug("produceStrongestArmy()");
+  debug("produceStrongestProductionBase()");
 
-  Stack* stack = getFreeStack(d_player);
+  Stack* stack = getFreeStack(d_owner);
   if (stack)
     {
       unsigned int max_strength = 0;
       int strong_idx = -1;
-      for (int i = 0; i < 4; i++)
+      for (int i = 0; i < getMaxNoOfProductionBases(); i++)
 	{
-	  if (d_basicprod[i] == NULL)
+	  if (d_prodbase[i] == NULL)
 	    continue;
-	  if (getArmy(i)->getStat(Army::STRENGTH,false) > max_strength)
+	  if (getProductionBase(i)->getStat(Army::STRENGTH,false) > 
+	      max_strength)
 	    {
 	      strong_idx = i;
-	      max_strength = getArmy(i)->getStat(Army::STRENGTH,false);
+	      max_strength = 
+		getProductionBase(i)->getStat(Army::STRENGTH,false);
 	    }
 	}
       if (strong_idx == -1)
 	return;
 
-      int savep = d_production;
-      setProduction(strong_idx);
+      int savep = d_active_production_slot;
+      setActiveProductionSlot(strong_idx);
       produceArmy();
-      setProduction(savep);
+      setActiveProductionSlot(savep);
       return;
     }
 }
@@ -506,39 +491,41 @@ void City::produceStrongestArmy()
 void City::produceScout()
 {
   const Armysetlist* al = Armysetlist::getInstance();
-  Uint32 set = getPlayer()->getArmyset();
+  Uint32 set = d_owner->getArmyset();
   Army *scout = al->getArmy(set, 0);
-  Army *a = new Army(*scout, d_player);
+  Army *a = new Army(*scout, d_owner);
   GameMap::getInstance()->addArmy(this, a);
 
 }
 
-void City::produceWeakestArmy()
+void City::produceWeakestProductionBase()
 {
-  debug("produceWeakestArmy()");
+  debug("produceWeakestProductionBase()");
 
-  Stack* stack = getFreeStack(d_player);
+  Stack* stack = getFreeStack(d_owner);
   if (stack)
     {
       unsigned int min_strength = 100;
       int weak_idx = -1;
-      for (int i = 0; i < 4; i++)
+      for (int i = 0; i < getMaxNoOfProductionBases(); i++)
 	{
-	  if (d_basicprod[i] == NULL)
+	  if (d_prodbase[i] == NULL)
 	    continue;
-	  if (getArmy(i)->getStat(Army::STRENGTH,false) < min_strength)
+	  if (getProductionBase(i)->getStat(Army::STRENGTH,false) < 
+	      min_strength)
 	    {
 	      weak_idx = i;
-	      min_strength = getArmy(i)->getStat(Army::STRENGTH,false);
+	      min_strength = 
+		getProductionBase(i)->getStat(Army::STRENGTH,false);
 	    }
 	}
       if (weak_idx == -1)
 	return;
 
-      int savep = d_production;
-      setProduction(weak_idx);
+      int savep = d_active_production_slot;
+      setActiveProductionSlot(weak_idx);
       produceArmy();
-      setProduction(savep);
+      setActiveProductionSlot(savep);
       return;
     }
 }
@@ -549,13 +536,13 @@ void City::nextTurn()
     return;
 
   // check if an army should be produced
-  if (d_production >= 0 && --d_duration == 0) 
+  if (d_active_production_slot >= 0 && --d_duration == 0) 
     {
-      if (d_player->getGold() <= 0)
+      if (d_owner->getGold() <= 0)
 	{
 	  //dont make or vector the unit
 	  //and also stop production
-	  setProduction(-1);
+	  setActiveProductionSlot(-1);
 	  d_vectoring = false;
 	  d_vector.x = -1;
 	  d_vector.y = -1;
@@ -567,33 +554,35 @@ void City::nextTurn()
       if (d_vectoring)
 	{
 	  VectoredUnitlist *vul = VectoredUnitlist::getInstance();
-	  VectoredUnit *v = new VectoredUnit(d_pos, d_vector, 
-					     new Army(*(d_basicprod[d_production])),
-					     MAX_TURNS_FOR_VECTORING, getPlayer());
+	  VectoredUnit *v = new VectoredUnit
+	    (getPos(), d_vector, 
+	     new Army(*(d_prodbase[d_active_production_slot])),
+	     MAX_TURNS_FOR_VECTORING, d_owner);
 	  vul->push_back(v);
-	  setProduction(d_production);
-	  item->fillData(getArmytype(d_production), this, true);
+	  setActiveProductionSlot(d_active_production_slot);
+	  item->fillData(getProductionBase(d_active_production_slot), this, 
+			 true);
 	}
       else //or make it here
 	{
 	  Army *a = produceArmy();
-	  item->fillData(a->getType(), this, false);
+	  item->fillData(a, this, false);
 	}
       //FIXME: a cookie goes to the person who can figure out how
       //to get this action into the realplayer class.
-      getPlayer()->getActionlist()->push_back(item);
+      d_owner->getActionlist()->push_back(item);
     }
 }
 
-bool City::hasProduction(int type, Uint32 set) const
+bool City::hasProductionBase(int type, Uint32 set) const
 {
   if (type < 0)
     return false;
-  for (int i = 0; i < d_numbasic; i++)
+  for (int i = 0; i < d_numprodbase; i++)
     {
-      if (d_basicprod[i] == NULL)
+      if (d_prodbase[i] == NULL)
 	continue;
-      if (d_basicprod[i]->getType() == (unsigned int) type)
+      if (d_prodbase[i]->getType() == (unsigned int) type)
 	return true;
     }
 
@@ -605,23 +594,18 @@ int City::getArmytype(int slot) const
   if (slot < 0)
     return -1;
 
-  if (slot >= d_numbasic)
+  if (slot >= d_numprodbase)
     return -1;
-  if (d_basicprod[slot] == NULL)
+  if (d_prodbase[slot] == NULL)
     return -1;
-  return d_basicprod[slot]->getType();
+  return d_prodbase[slot]->getType();
 }
 
-const Army* City::getArmy(int slot) const
+const Army* City::getProductionBase(int slot) const
 {
   if (getArmytype(slot) == -1)
     return 0;
-  return d_basicprod[slot];
-}
-
-bool City::isFriend(Player* player) const
-{
-  return (d_player == player);
+  return d_prodbase[slot];
 }
 
 int City::getGoldNeededForUpgrade() const
@@ -638,9 +622,8 @@ int City::getGoldNeededForUpgrade() const
 
 void City::setVectoring(Vector<int> p) 
 {
-  d_vector.x=p.x; 
-  d_vector.y=p.y;
-  d_vectoring=true; 
+  d_vector = p;
+  d_vectoring = true;
 
   if (p.x == -1 || p.y == -1)
     {
@@ -653,35 +636,35 @@ void City::setVectoring(Vector<int> p)
 Army *City::produceArmy()
 {
   // add produced army to stack
-  if (d_production == -1)
+  if (d_active_production_slot == -1)
     return NULL;
 
   debug("produce_army()\n");
 
   // do not produce an army if the player has no gold.
   // unless it's the neutrals
-  if (d_player != Playerlist::getInstance()->getNeutral() && 
-      d_player->getGold() < 0) 
+  if (d_owner != Playerlist::getInstance()->getNeutral() && 
+      d_owner->getGold() < 0) 
     return NULL;
 
-  Army *a = new Army(*(getArmy(d_production)), d_player);
+  Army *a = new Army(*(getProductionBase(d_active_production_slot)), d_owner);
   GameMap::getInstance()->addArmy(this, a);
 
-  if (d_player == Playerlist::getInstance()->getNeutral()) 
+  if (d_owner == Playerlist::getInstance()->getNeutral()) 
     {
       //we're an active neutral city
       //check to see if we've made 5 or not.
       //stop producing if we've made 5 armies in our neutral city
-      Stack *s = d_player->getStacklist()->getObjectAt(d_pos);
+      Stack *s = d_owner->getStacklist()->getObjectAt(getPos());
       if (!s)
-	setProduction(d_production);
+	setActiveProductionSlot(d_active_production_slot);
       else if (s->size() < 5)
-	setProduction(d_production);
+	setActiveProductionSlot(d_active_production_slot);
       else
-	setProduction(-1);
+	setActiveProductionSlot(-1);
     }
   else // start producing next army of same type
-    setProduction(d_production);
+    setActiveProductionSlot(d_active_production_slot);
   return a;
 }
 
@@ -692,7 +675,7 @@ bool City::canAcceptVectoredUnit()
 bool City::canAcceptVectoredUnits(Uint32 number_of_units)
 {
   VectoredUnitlist *vul = VectoredUnitlist::getInstance();
-  if (vul->getNumberOfVectoredUnitsGoingTo(d_pos) + number_of_units >= 
+  if (vul->getNumberOfVectoredUnitsGoingTo(getPos()) + number_of_units >= 
       MAX_ARMIES_VECTORED_TO_ONE_CITY)
     return false;
   return true;
