@@ -1,3 +1,10 @@
+// Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005 Ulf Lorenz
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2005 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sigc++/functors/mem_fun.h>
 
@@ -74,7 +82,7 @@ int Citylist::countCities(Player* player) const
     {
         if ((*it).isBurnt())
           continue;
-        if ((*it).getPlayer() == player) cities++;
+        if ((*it).getOwner() == player) cities++;
     }
     
     return cities;
@@ -87,7 +95,7 @@ void Citylist::nextTurn(Player* p)
     // This iteration adds the city production to the player    
     for (iterator it = begin(); it != end(); it++)
     {
-        if ((*it).getPlayer() == p)
+        if ((*it).getOwner() == p)
             (*it).nextTurn();
     }
 
@@ -96,67 +104,61 @@ void Citylist::nextTurn(Player* p)
     // add the gold in the first run, because then the player may come above 0
     // gold, which means some cities produce units, others not.
     for (const_iterator it = begin(); it != end(); it++)
-        if ((*it).getPlayer() == p)
+        if ((*it).getOwner() == p)
             p->addGold((*it).getGold());
 }
+static bool isFogged(void *object)
+{
+  return ((City*)object)->isFogged();
+}
+
+static bool isBurnt(void *object)
+{
+  return ((City*)object)->isBurnt();
+}
+
+static bool isNotOwnedByNeutral(void *object)
+{
+  return ((City*)object)->getOwner() != Playerlist::getInstance()->getNeutral();
+}
+
+static bool isNotOwnedByActivePlayer(void *object)
+{
+  return ((City*)object)->getOwner() != Playerlist::getActiveplayer();
+}
+
+static bool isOwnedByActivePlayer(void *object)
+{
+  return ((City*)object)->getOwner() == Playerlist::getActiveplayer();
+}
+
+static bool isNotOwnedByEnemy(void *object)
+{
+  Player *p = Playerlist::getActiveplayer();
+  if (!p)
+    return false;
+  City *city = ((City*)object);
+  if (city->getOwner() != p &&
+	    p->getDiplomaticState(city->getOwner()) == Player::AT_WAR)
+    return false;
+  return true;
+}
 
 City* Citylist::getNearestEnemyCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-    Player* p = Playerlist::getInstance()->getActiveplayer();
-    
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        if ((*it).isBurnt())
-            continue;
-
-        if ((*it).getPlayer() != p &&
-	    p->getDiplomaticState((*it).getPlayer()) == Player::AT_WAR)
-        {        
-            Vector<int> p = (*it).getPos();
-            int delta = abs(p.x - pos.x);
-            if (delta < abs(p.y - pos.y))
-                delta = abs(p.y - pos.y);
-            if ((diff > delta) || (diff == -1))
-            {
-                diff = delta;
-                diffit = it;
-            }
-        }
-    }
-
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isNotOwnedByEnemy);
+  return getNearestObject(pos, &filters);
 }
 
+
 City* Citylist::getNearestForeignCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-    Player* p = Playerlist::getInstance()->getActiveplayer();
-    
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        if ((*it).isBurnt())
-            continue;
-
-        if ((*it).getPlayer() != p)
-        {        
-            Vector<int> p = (*it).getPos();
-            int delta = abs(p.x - pos.x);
-            if (delta < abs(p.y - pos.y))
-                delta = abs(p.y - pos.y);
-            if ((diff > delta) || (diff == -1))
-            {
-                diff = delta;
-                diffit = it;
-            }
-        }
-    }
-
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isOwnedByActivePlayer);
+  return getNearestObject(pos, &filters);
 }
 
 City* Citylist::getNearestCity(const Vector<int>& pos, int dist)
@@ -197,7 +199,7 @@ City* Citylist::getNearestCity(const Vector<int>& pos, Player *p)
         if ((*it).isBurnt())
             continue;
 
-        if ((*it).getPlayer() == p)
+        if ((*it).getOwner() == p)
         {
             Vector<int> p = (*it).getPos();
             int delta = abs(p.x - pos.x);
@@ -218,57 +220,17 @@ City* Citylist::getNearestCity(const Vector<int>& pos, Player *p)
 
 City* Citylist::getNearestCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-
-    for (iterator it = begin(); it != end(); ++it)
-    {
-          if ((*it).isBurnt())
-              continue;
-          
-          Vector<int> p = (*it).getPos();
-          int delta = abs(p.x - pos.x);
-          if (delta < abs(p.y - pos.y))
-              delta = abs(p.y - pos.y);
-          
-          if ((diff > delta) || (diff == -1))
-          {
-              diff = delta;
-              diffit = it;
-          }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  return getNearestObject(pos, &filters);
 }
 
 City* Citylist::getNearestVisibleCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-
-    for (iterator it = begin(); it != end(); ++it)
-    {
-          if ((*it).isBurnt())
-              continue;
-
-          if ((*it).isFogged() == true)
-              continue;
-          
-          Vector<int> p = (*it).getPos();
-          int delta = abs(p.x - pos.x);
-          if (delta < abs(p.y - pos.y))
-              delta = abs(p.y - pos.y);
-          
-          if ((diff > delta) || (diff == -1))
-          {
-              diff = delta;
-              diffit = it;
-          }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isFogged);
+  return getNearestObject(pos, &filters);
 }
 
 City* Citylist::getNearestVisibleCity(const Vector<int>& pos, int dist)
@@ -293,73 +255,29 @@ City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos, int dist)
   return NULL;
 }
 
+
 City* Citylist::getNearestVisibleFriendlyCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-    Player* p = Playerlist::getInstance()->getActiveplayer();
-    
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        if ((*it).isBurnt())
-            continue;
-
-	if ((*it).isFogged())
-	    continue;
-
-        if ((*it).getPlayer() == p)
-        {
-            Vector<int> p = (*it).getPos();
-            int delta = abs(p.x - pos.x);
-            if (delta < abs(p.y - pos.y))
-                delta = abs(p.y - pos.y);
-            
-            if ((diff > delta) || (diff == -1))
-            {
-                diff = delta;
-                diffit = it;
-            }
-        }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isFogged);
+  filters.push_back(isNotOwnedByActivePlayer);
+  return getNearestObject(pos, &filters);
 }
 
 City* Citylist::getNearestNeutralCity(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-    
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        if ((*it).isBurnt())
-            continue;
-
-        if ((*it).getPlayer() == Playerlist::getInstance()->getNeutral())
-        {
-            Vector<int> p = (*it).getPos();
-            int delta = abs(p.x - pos.x);
-            if (delta < abs(p.y - pos.y))
-                delta = abs(p.y - pos.y);
-            
-            if ((diff > delta) || (diff == -1))
-            {
-                diff = delta;
-                diffit = it;
-            }
-        }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  filters.push_back(isNotOwnedByNeutral);
+  return getNearestObject(pos, &filters);
 }
 
 
 City* Citylist::getFirstCity(Player* p)
 {
     for (iterator it = begin(); it != end(); it++)
-        if ((*it).getPlayer() == p)
+        if ((*it).getOwner() == p)
             return &(*it);
 
     return 0;
@@ -388,7 +306,7 @@ bool Citylist::load(std::string tag, XML_Helper* helper)
 	it--;
 	City *city = &*it;
 	Army *a = new Army (helper, Army::PRODUCTION_BASE);
-	city->addBasicProd(-1, a);
+	city->addProductionBase(-1, a);
 	return true;
       }
     if (tag == "city")
@@ -403,9 +321,9 @@ bool Citylist::load(std::string tag, XML_Helper* helper)
 void Citylist::changeOwnership(Player *old_owner, Player *new_owner)
 {
   for (iterator it = begin(); it != end(); it++)
-    if ((*it).getPlayer() == old_owner)
+    if ((*it).getOwner() == old_owner)
       {
-	(*it).setPlayer(new_owner);
+	(*it).setOwner(new_owner);
 	if ((*it).isCapital())
 	  if ((*it).getCapitalOwner() == old_owner)
 	    (*it).setCapitalOwner(new_owner);
@@ -418,7 +336,7 @@ void Citylist::stopVectoringTo(City *c)
     {
       if ((*it).isBurnt() == true)
 	continue;
-      if ((*it).getPlayer() != c->getPlayer())
+      if ((*it).getOwner() != c->getOwner())
 	continue;
       if ((*it).getVectoring() == Vector<int>(-1,-1))
 	continue;
@@ -439,7 +357,7 @@ Vector<int> Citylist::calculateCenterOfTerritory (Player *p)
   int count = 0;
   for (iterator it = begin(); it != end(); it++)
     {
-      if (p && (*it).getPlayer() == p)
+      if (p && (*it).getOwner() == p)
 	continue;
       Vector<int> pos = (*it).getPos();
       count++;
@@ -461,7 +379,7 @@ bool Citylist::isVectoringTarget(City *target)
 {
   for (iterator it = begin(); it != end(); it++)
     {
-      if ((*it).getPlayer() != target->getPlayer())
+      if ((*it).getOwner() != target->getOwner())
 	continue;
       if (target->contains((*it).getVectoring()))
 	return true;
@@ -474,64 +392,19 @@ std::list<City*> Citylist::getCitiesVectoringTo(City *target)
   std::list<City*> cities;
   for (iterator it = begin(); it != end(); it++)
     {
-      if ((*it).getPlayer() != target->getPlayer())
+      if ((*it).getOwner() != target->getOwner())
 	continue;
       if (target->contains((*it).getVectoring()))
 	cities.push_back(&(*it));
     }
   return cities;
 }
-City* Citylist::getNearestCity(City *city)
-{
-    int diff = -1;
-    iterator diffit;
-
-    for (iterator it = begin(); it != end(); ++it)
-    {
-          if ((*it).isBurnt())
-              continue;
-          
-	  Vector<int> pos = city->getPos();
-          Vector<int> p = (*it).getPos();
-          int delta = abs(p.x - pos.x);
-          if (delta < abs(p.y - pos.y))
-              delta = abs(p.y - pos.y);
-          
-          if ((diff > delta && delta != 0) || (diff == -1))
-          {
-              diff = delta;
-              diffit = it;
-          }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
-}
 
 City* Citylist::getNearestCityPast(const Vector<int>& pos, int dist)
 {
-    int diff = -1;
-    iterator diffit;
-
-    for (iterator it = begin(); it != end(); ++it)
-    {
-          if ((*it).isBurnt())
-              continue;
-          
-          Vector<int> p = (*it).getPos();
-          int delta = abs(p.x - pos.x);
-          if (delta < abs(p.y - pos.y))
-              delta = abs(p.y - pos.y);
-          
-          if ((diff > delta && delta >= dist) || (diff == -1))
-          {
-              diff = delta;
-              diffit = it;
-          }
-    }
-    
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isBurnt);
+  return getNearestObjectAfter(pos, dist, &filters);
 }
 
 // End of file
