@@ -1,3 +1,10 @@
+// Copyright (C) 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003, 2004, 2005, 2006 Andrea Paternesi
+// Copyright (C) 2004 David Sterba
+// Copyright (C) 2005 Bryan Duff
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,8 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 //This file contains the various macros used within lordsawar.
 
@@ -20,8 +27,8 @@
 
 #include <libintl.h>
 
-#define LORDSAWAR_SAVEGAME_VERSION "0.0.8"
-#define LORDSAWAR_CONFIG_VERSION "0.0.8"
+#define LORDSAWAR_SAVEGAME_VERSION "0.0.9"
+#define LORDSAWAR_CONFIG_VERSION "0.0.9"
 #define ENABLE_NLS 1 // Very important toactivate localization
 #define _(string) gettext(string) // Macro for the gettext
 #define __(astring) std::string(gettext(astring.c_str()))
@@ -39,7 +46,7 @@ const unsigned int TEMPLE_TYPES=1;
 const unsigned int RUIN_TYPES=2; 
 const unsigned int DIPLOMACY_TYPES=3; 
 const unsigned int ROAD_TYPES=12; 
-const unsigned int FOG_TYPES=13; 
+const unsigned int FOG_TYPES=15; 
 const unsigned int BRIDGE_TYPES=4; 
 const unsigned int CURSOR_TYPES=12; 
 const unsigned int MAX_ARMIES_VECTORED_TO_ONE_CITY = 4;
@@ -50,6 +57,23 @@ const unsigned int DIPLOMACY_STARTING_SCORE = 8;
 const unsigned int DIPLOMACY_MAX_SCORE = 15;
 const unsigned int DIPLOMACY_MIN_SCORE = 0;
 const unsigned int MAX_STACK_SIZE = 8;
+const unsigned int MAX_PRODUCTION_SLOTS_IN_A_CITY = 4;
+
+const unsigned int MAP_SIZE_TINY_WIDTH = 50;
+const unsigned int MAP_SIZE_TINY_HEIGHT = 75;
+const unsigned int MAP_SIZE_SMALL_WIDTH = 70;
+const unsigned int MAP_SIZE_SMALL_HEIGHT = 105;
+const unsigned int MAP_SIZE_NORMAL_WIDTH = 112;
+const unsigned int MAP_SIZE_NORMAL_HEIGHT = 156;
+
+const unsigned int PRODUCTION_SHIELD_WIDTH = 10;
+const unsigned int PRODUCTION_SHIELD_HEIGHT = 10;
+const unsigned int PRODUCTION_SHIELD_TYPES = 7;
+const unsigned int MOVE_BONUS_WIDTH = 32;
+const unsigned int MOVE_BONUS_HEIGHT = 20;
+const unsigned int MOVE_BONUS_TYPES = 6;
+const unsigned int SELECTOR_FRAMES = 6;
+const unsigned int SMALL_SELECTOR_FRAMES = 4;
 
 // from www.boost.org - derivation from this class makes the derived class
 // noncopyable
