@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef GUI_ARMYLIST_WINDOW_H
 #define GUI_ARMYLIST_WINDOW_H
@@ -39,6 +42,7 @@
 #include "../army.h"
 #include "../armyset.h"
 
+//! Armyset Editor.  Edit an Armyset.
 class ArmySetWindow: public sigc::trackable
 {
  public:
@@ -128,6 +132,7 @@ class ArmySetWindow: public sigc::trackable
     void on_save_armyset_as_activated();
     void on_quit_activated();
     void on_edit_armyset_info_activated();
+    void on_help_about_activated();
     void on_army_selected();
     void fill_army_info(Army *army);
 
