@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef EDITORBIGMAP_H
 #define EDITORBIGMAP_H
@@ -32,11 +36,10 @@ class City;
 class Ruin;
 class Signpost;
 class Temple;
-class Object;
+class UniquelyIdentified;
 
 
-/** Specialization of BigMap for the editor
-  */
+//! Scenario editor.  Specializatoin of the BigMap class for the editor.
 class EditorBigMap: public BigMap
 {
  public:
@@ -56,7 +59,7 @@ class EditorBigMap: public BigMap
     void mouse_leave_event();
 
     // something was selected
-    typedef std::vector<Object *> map_selection_seq;
+    typedef std::vector<UniquelyIdentified *> map_selection_seq;
     sigc::signal<void, map_selection_seq> objects_selected;
 
     // emitted whenever the user moves the mouse to a new tile
@@ -84,6 +87,7 @@ class EditorBigMap: public BigMap
     int tile_to_bridge_type(Vector<int> tile);
     void change_map_under_cursor();
     std::vector<Vector<int> > get_cursor_tiles();
+    int calculateRoadType (Vector<int> t);
 };
 
 #endif
