@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -26,6 +30,7 @@
 #include <gtkmm/eventbox.h>
 #include <gtkmm/image.h>
 #include <gtkmm/dialog.h>
+#include <gtkmm/aboutdialog.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/filechooserdialog.h>
 #include <gtkmm/menu.h>
@@ -47,7 +52,9 @@
 #include "../GraphicsCache.h"
 #include "../smallmap.h"
 #include "../GameScenario.h"
+#include "../CreateScenarioRandomize.h"
 #include "../armysetlist.h"
+#include "../Itemlist.h"
 #include "../playerlist.h"
 #include "../ai_dummy.h"
 
@@ -77,6 +84,8 @@
 #include "city-dialog.h"
 #include "map-info-dialog.h"
 #include "new-map-dialog.h"
+#include "itemlist-dialog.h"
+#include "rewardlist-dialog.h"
 
 
 MainWindow::MainWindow()
@@ -121,6 +130,8 @@ MainWindow::MainWindow()
 			      EditorBigMap::TERRAIN, 2);
     setup_pointer_radiobutton(xml, "draw_3", "button_3x3",
 			      EditorBigMap::TERRAIN, 3);
+    setup_pointer_radiobutton(xml, "draw_6", "button_6x6",
+			      EditorBigMap::TERRAIN, 6);
     setup_pointer_radiobutton(xml, "draw_stack", "button_stack",
 			      EditorBigMap::STACK, 1);
     setup_pointer_radiobutton(xml, "draw_ruin", "button_ruin",
@@ -170,6 +181,39 @@ MainWindow::MainWindow()
 				       &MainWindow::on_smooth_map_activated));
     xml->connect_clicked("smooth_screen_menuitem", sigc::mem_fun
 			 (this, &MainWindow::on_smooth_screen_activated));
+    xml->connect_clicked("edit_items_menuitem", 
+			 sigc::mem_fun(this, 
+				       &MainWindow::on_edit_items_activated));
+    xml->connect_clicked("edit_rewards_menuitem", 
+			 sigc::mem_fun(this, 
+				       &MainWindow::on_edit_rewards_activated));
+    xml->connect_clicked
+      ("random_all_cities_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_all_cities_activated));
+    xml->connect_clicked
+      ("random_unnamed_cities_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_unnamed_cities_activated));
+    xml->connect_clicked
+      ("random_all_ruins_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_all_ruins_activated));
+    xml->connect_clicked
+      ("random_unnamed_ruins_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_unnamed_ruins_activated));
+    xml->connect_clicked
+      ("random_all_temples_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_all_temples_activated));
+    xml->connect_clicked
+      ("random_unnamed_temples_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_unnamed_temples_activated));
+    xml->connect_clicked
+      ("random_all_signs_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_all_signs_activated));
+    xml->connect_clicked
+      ("random_unnamed_signs_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_random_unnamed_signs_activated));
+    xml->connect_clicked
+      ("help_about_menuitem", 
+       sigc::mem_fun(this, &MainWindow::on_help_about_activated));
 }
 
 MainWindow::~MainWindow()
@@ -230,7 +274,8 @@ void MainWindow::setup_terrain_radiobuttons()
 	item.button->signal_toggled().connect(
 	    sigc::mem_fun(this, &MainWindow::on_terrain_radiobutton_toggled));
 
-	SDL_Surface *surf = (*(*tile)[0])[0]->getPixmap();
+	//SDL_Surface *surf = (*(*tile)[0])[0]->getPixmap();
+	SDL_Surface *surf = (*(*(*tile).begin())->begin())->getPixmap();
 	Glib::RefPtr<Gdk::Pixbuf> pic = to_pixbuf(surf)->scale_simple(20, 20, Gdk::INTERP_NEAREST);
 	item.button->add(*manage(new Gtk::Image(pic)));
 
@@ -300,13 +345,14 @@ bool MainWindow::on_delete_event(GdkEventAny *e)
 
 void MainWindow::show_initial_map()
 {
-    set_filled_map(112, 156, Tile::WATER, "default", "default", "default");
+    set_filled_map(112, 156, Tile::WATER, "default", "default", "default",
+		   "Default");
     setup_terrain_radiobuttons();
     remove_tile_style_buttons();
     setup_tile_style_buttons(Tile::GRASS);
 }
 
-void MainWindow::set_filled_map(int width, int height, int fill_style, std::string tileset, std::string shieldset, std::string cityset)
+void MainWindow::set_filled_map(int width, int height, int fill_style, std::string tileset, std::string shieldset, std::string cityset, std::string armyset)
 {
     clear_map_state();
     d_width = width;
@@ -316,9 +362,11 @@ void MainWindow::set_filled_map(int width, int height, int fill_style, std::stri
     GameMap::setWidth(width);
     GameMap::setHeight(height);
     GameMap::getInstance(tileset, shieldset, cityset);
+    Itemlist::createStandardInstance();
 
     // sets up the lists
     game_scenario.reset(new GameScenario(_("Untitled"), _("No description"), true));
+    d_create_scenario_names.reset(new CreateScenarioRandomize());
     //zip past the player IDs (+1 for neutral)
     for (unsigned int i = 0; i < MAX_PLAYERS + 1; i++)
       fl_counter->getNextId();
@@ -326,8 +374,8 @@ void MainWindow::set_filled_map(int width, int height, int fill_style, std::stri
 
     // ...however we need to do some of the setup by hand. We need to create a
     // neutral player to give cities a player upon creation...
-    Uint32 armyset = Armysetlist::getInstance()->getArmysets()[0];
-    Player* neutral = new AI_Dummy(_("Neutral"), armyset, Player::get_color_for_neutral(), width, height, MAX_PLAYERS);
+    Uint32 armyset_id = Armysetlist::getInstance()->getArmysetId(armyset);
+    Player* neutral = new AI_Dummy(_("Neutral"), armyset_id, Player::get_color_for_neutral(), width, height, MAX_PLAYERS);
     neutral->setType(Player::AI_DUMMY);
     Playerlist::getInstance()->push_back(neutral);
     Playerlist::getInstance()->setNeutral(neutral);
@@ -352,7 +400,8 @@ void MainWindow::set_random_map(int width, int height,
 				int hills, int mountains,
 				int cities, int ruins, int temples,
 				int signposts, std::string tileset,
-				std::string shieldset, std::string cityset)
+				std::string shieldset, std::string cityset,
+				std::string armyset)
 {
     clear_map_state();
 
@@ -368,8 +417,8 @@ void MainWindow::set_random_map(int width, int height,
     
     // We need to create a neutral player to give cities a player upon
     // creation...
-    Uint32 armyset = Armysetlist::getInstance()->getArmysets()[0];
-    Player* neutral = new AI_Dummy(_("Neutral"), armyset, Player::get_color_for_neutral(), width, height, MAX_PLAYERS);
+    Uint32 armyset_id = Armysetlist::getInstance()->getArmysetId(armyset);
+    Player* neutral = new AI_Dummy(_("Neutral"), armyset_id, Player::get_color_for_neutral(), width, height, MAX_PLAYERS);
     neutral->setType(Player::AI_DUMMY);
     Playerlist::getInstance()->push_back(neutral);
     Playerlist::getInstance()->setNeutral(neutral);
@@ -403,8 +452,10 @@ void MainWindow::set_random_map(int width, int height,
     gen.makeMap(width, height, false);
     GameMap::getInstance()->fill(&gen);
 
+    Itemlist::createStandardInstance();
     // sets up the lists
     game_scenario.reset(new GameScenario(_("Untitled"), _("No description"), true));
+    d_create_scenario_names.reset(new CreateScenarioRandomize());
     
     // now fill the lists
     const Maptile::Building* build = gen.getBuildings(width, height);
@@ -414,7 +465,7 @@ void MainWindow::set_random_map(int width, int height,
 	    {
 	    case Maptile::CITY:
 		Citylist::getInstance()->push_back(City(Vector<int>(i,j)));
-		(*Citylist::getInstance()->rbegin()).setPlayer(
+		(*Citylist::getInstance()->rbegin()).setOwner(
 		    Playerlist::getInstance()->getNeutral());
 		break;
 	    case Maptile::TEMPLE:
@@ -447,6 +498,7 @@ void MainWindow::clear_map_state()
     bigmap.reset();
     smallmap.reset();
     game_scenario.reset();
+    d_create_scenario_names.reset();
     GraphicsCache::deleteInstance();
 }
 
@@ -555,10 +607,11 @@ void MainWindow::on_new_map_activated()
 			   d.map.hills, d.map.mountains,
 			   d.map.cities, d.map.ruins, d.map.temples, 
 			   d.map.signposts, d.map.tileset, 
-			   d.map.shieldset, d.map.cityset);
+			   d.map.shieldset, d.map.cityset, d.map.armyset);
 	else
 	    set_filled_map(d.map.width, d.map.height, d.map.fill_style, 
-			   d.map.tileset, d.map.shieldset, d.map.cityset);
+			   d.map.tileset, d.map.shieldset, d.map.cityset,
+			   d.map.armyset);
     }
 }
 
@@ -669,6 +722,7 @@ void MainWindow::on_fullscreen_activated()
     else
 	window->unfullscreen();
 }
+
 void MainWindow::remove_tile_style_buttons()
 {
   Glib::ListHandle<Gtk::Widget*> children = 
@@ -681,6 +735,7 @@ void MainWindow::remove_tile_style_buttons()
     }
   tile_style_items.clear();
 }
+
 void MainWindow::setup_tile_style_buttons(Tile::Type terrain)
 {
   Gtk::RadioButton::Group group;
@@ -705,9 +760,9 @@ void MainWindow::setup_tile_style_buttons(Tile::Type terrain)
   group = auto_item.button->get_group();
   tile_style_items.push_back(auto_item);
 
-  for (unsigned int i = 0; i < tile->size(); i++)
+  for (Tile::iterator it = tile->begin(); it != tile->end(); it++)
     {
-      TileStyleSet *tilestyleset = (*tile)[i];
+      TileStyleSet *tilestyleset = *it;
       //loop through tile style sets
       for (unsigned int j = 0; j < tilestyleset->size(); j++)
 	{
@@ -737,10 +792,12 @@ void MainWindow::setup_tile_style_buttons(Tile::Type terrain)
 
   terrain_tile_style_hbox->show_all();
 }
+
 void MainWindow::on_tile_style_radiobutton_toggled()
 {
   on_pointer_radiobutton_toggled();
 }
+
 void MainWindow::on_terrain_radiobutton_toggled()
 {
   remove_tile_style_buttons();
@@ -843,7 +900,7 @@ void MainWindow::on_mouse_on_tile(Vector<int> tile)
     mouse_position_label->set_markup(str);
 }
 
-void MainWindow::on_objects_selected(std::vector<Object *> objects)
+void MainWindow::on_objects_selected(std::vector<UniquelyIdentified *> objects)
 {
     assert(!objects.empty());
 
@@ -855,7 +912,7 @@ void MainWindow::on_objects_selected(std::vector<Object *> objects)
     {
 	// show a popup
 	Gtk::Menu *menu = manage(new Gtk::Menu);
-	for (std::vector<Object *>::iterator i = objects.begin(), end = objects.end();
+	for (std::vector<UniquelyIdentified *>::iterator i = objects.begin(), end = objects.end();
 	     i != end; ++i)
 	{
 	    Glib::ustring s;
@@ -880,7 +937,7 @@ void MainWindow::on_objects_selected(std::vector<Object *> objects)
     }
 }
 
-void MainWindow::popup_dialog_for_object(Object *object)
+void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
 {
     if (Stack *o = dynamic_cast<Stack *>(object))
     {
@@ -894,7 +951,7 @@ void MainWindow::popup_dialog_for_object(Object *object)
     }
     else if (City *o = dynamic_cast<City *>(object))
     {
-	CityDialog d(o);
+	CityDialog d(o, d_create_scenario_names.get());
 	d.set_parent_window(*window.get());
 	d.run();
 
@@ -904,19 +961,19 @@ void MainWindow::popup_dialog_for_object(Object *object)
     }
     else if (Ruin *o = dynamic_cast<Ruin *>(object))
     {
-	RuinDialog d(o);
+	RuinDialog d(o, d_create_scenario_names.get());
 	d.set_parent_window(*window.get());
 	d.run();
     }
     else if (Signpost *o = dynamic_cast<Signpost *>(object))
     {
-	SignpostDialog d(o);
+	SignpostDialog d(o, d_create_scenario_names.get());
 	d.set_parent_window(*window.get());
 	d.run();
     }
     else if (Temple *o = dynamic_cast<Temple *>(object))
     {
-	TempleDialog d(o);
+	TempleDialog d(o, d_create_scenario_names.get());
 	d.set_parent_window(*window.get());
 	d.run();
 
@@ -924,13 +981,168 @@ void MainWindow::popup_dialog_for_object(Object *object)
 	bigmap->draw();
     }
 }
+
 void MainWindow::on_smooth_map_activated()
 {
   GameMap::getInstance()->applyTileStyles(0, 0, GameMap::getHeight(), 
 					  GameMap::getWidth(), true);
   bigmap->draw();
 }
+
 void MainWindow::on_smooth_screen_activated()
 {
   bigmap->smooth_view();
 }
+
+void MainWindow::on_edit_items_activated()
+{
+  ItemlistDialog d;
+  d.set_parent_window(*window.get());
+  d.run();
+}
+
+void MainWindow::on_edit_rewards_activated()
+{
+  RewardlistDialog d;
+  d.set_parent_window(*window.get());
+  d.run();
+}
+
+void MainWindow::randomize_city(City *c)
+{
+  std::string name = d_create_scenario_names->popRandomCityName();
+  if (name != "")
+    c->setName(name);
+  c->setRandomArmytypes(true, 1);
+}
+
+void MainWindow::on_random_all_cities_activated()
+{
+  Citylist *cl = Citylist::getInstance();
+  for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
+    randomize_city(&*it);
+}
+
+void MainWindow::on_random_unnamed_cities_activated()
+{
+  Citylist *cl = Citylist::getInstance();
+  for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
+    {
+      if ((*it).getName() == DEFAULT_CITY_NAME)
+	randomize_city(&*it);
+    }
+}
+
+void MainWindow::randomize_ruin(Ruin *r)
+{
+  std::string name = d_create_scenario_names->popRandomRuinName();
+  if (name != "")
+    {
+      Location *l = r;
+      RenamableLocation *renamable_ruin = static_cast<RenamableLocation*>(l);
+      renamable_ruin->setName(name);
+    }
+}
+
+void MainWindow::on_random_all_ruins_activated()
+{
+  Ruinlist *rl = Ruinlist::getInstance();
+  for (Ruinlist::iterator it = rl->begin(); it != rl->end(); it++)
+    randomize_ruin(&*it);
+}
+
+void MainWindow::on_random_unnamed_ruins_activated()
+{
+  Ruinlist *rl = Ruinlist::getInstance();
+  for (Ruinlist::iterator it = rl->begin(); it != rl->end(); it++)
+    {
+      if ((*it).getName() == DEFAULT_RUIN_NAME)
+	randomize_ruin(&*it);
+    }
+}
+
+void MainWindow::on_random_all_temples_activated()
+{
+  Templelist *tl = Templelist::getInstance();
+  for (Templelist::iterator it = tl->begin(); it != tl->end(); it++)
+    {
+      std::string name = d_create_scenario_names->popRandomTempleName();
+      if (name != "")
+	{
+	  Location *l = &*it;
+	  RenamableLocation *renamable_temple = 
+	    static_cast<RenamableLocation*>(l);
+	  renamable_temple->setName(name);
+	}
+    }
+}
+
+void MainWindow::on_random_unnamed_temples_activated()
+{
+  Templelist *tl = Templelist::getInstance();
+  for (Templelist::iterator it = tl->begin(); it != tl->end(); it++)
+    {
+      if ((*it).getName() == DEFAULT_TEMPLE_NAME)
+	{
+	  std::string name = d_create_scenario_names->popRandomTempleName();
+	  if (name != "")
+	    {
+	      Location *l = &*it;
+	      RenamableLocation *renamable_temple = 
+		static_cast<RenamableLocation*>(l);
+	      renamable_temple->setName(name);
+	    }
+	}
+    }
+}
+
+void MainWindow::randomize_signpost(Signpost *signpost)
+{
+  std::string name = "";
+  if ((rand() % d_create_scenario_names->getNumSignposts())  == 0)
+    name = d_create_scenario_names->popRandomSignpost();
+  else
+    name = d_create_scenario_names->getDynamicSignpost(signpost);
+  if (name == "")
+    name = d_create_scenario_names->popRandomSignpost();
+  signpost->setName(name);
+}
+
+void MainWindow::on_random_all_signs_activated()
+{
+  Signpostlist *sl = Signpostlist::getInstance();
+  for (Signpostlist::iterator it = sl->begin(); it != sl->end(); it++)
+    randomize_signpost(&*it);
+}
+
+void MainWindow::on_random_unnamed_signs_activated()
+{
+  Signpostlist *sl = Signpostlist::getInstance();
+  for (Signpostlist::iterator it = sl->begin(); it != sl->end(); it++)
+    {
+      if ((*it).getName() == DEFAULT_SIGNPOST)
+	randomize_signpost(&*it);
+    }
+}
+
+void MainWindow::on_help_about_activated()
+{
+  std::auto_ptr<Gtk::AboutDialog> dialog;
+
+  Glib::RefPtr<Gnome::Glade::Xml> xml
+    = Gnome::Glade::Xml::create(get_glade_path() + "/../about-dialog.glade");
+
+  Gtk::AboutDialog *d;
+  xml->get_widget("dialog", d);
+  dialog.reset(d);
+  dialog->set_transient_for(*window.get());
+
+  dialog->set_version(PACKAGE_VERSION);
+  SDL_Surface *logo = File::getMiscPicture("castle_icon.png");
+  dialog->set_logo(to_pixbuf(logo));
+  dialog->show_all();
+  dialog->run();
+
+  return;
+}
+
