@@ -1,3 +1,10 @@
+// Copyright (C) 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2006 Andrea Paternesi
+// Copyright (C) 2004 Bryan Duff
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include "fight.h"
 #include <algorithm>
@@ -34,6 +42,7 @@
 // Helper class; the single units participating in the fight are saved with
 // additional information. This should be a struct, but I don't know how to
 // forward the declaration properly.
+//! A particpant in a Fight.
 class Fighter
 {
     public:
@@ -110,7 +119,7 @@ Fight::Fight(Stack* attacker, Stack* defender, FightType type)
             if (!s)
                 continue;
             
-            if (s->getPlayer() == defender->getPlayer()
+            if (s->getOwner() == defender->getOwner()
                 && s != (*d_defenders.begin()))
             {
                 // check if stack may participate
@@ -129,20 +138,7 @@ Fight::Fight(Stack* attacker, Stack* defender, FightType type)
 
 	
     std::list<Stack*>::iterator it;
-    if (d_type == FOR_KEEPS)
-      {
-	// Now some last initializing. Each unit has its battle number 
-	// increased.
-
-	for (it = d_attackers.begin(); it != d_attackers.end(); it++)
-	  for (Stack::iterator sit = (*it)->begin(); sit != (*it)->end(); sit++)
-	    (*sit)->setBattlesNumber((*sit)->getBattlesNumber() + 1);
-
-	for (it = d_defenders.begin(); it != d_defenders.end(); it++)
-	  for (Stack::iterator sit = (*it)->begin(); sit != (*it)->end(); sit++)
-	    (*sit)->setBattlesNumber((*sit)->getBattlesNumber() + 1);
-      }
-
+    
     //setup fighters
     it = d_defenders.begin();
     std::vector<Army*> def;
@@ -162,7 +158,24 @@ Fight::Fight(Stack* attacker, Stack* defender, FightType type)
 	d_att_close.push_back(f);
       }
 
-}                
+  fillInInitialHPs();
+
+  // Before the battle starts, calculate the bonuses
+  // bonuses remain even if the unit providing a stackwide bonus dies
+
+
+  calculateBonus();
+}
+
+Fight::Fight(std::list<Stack*> attackers, std::list<Stack*> defenders,
+             std::list<FightItem> history)
+{
+  d_attackers = attackers;
+  d_defenders = defenders;
+  d_actions = history;
+
+  fillInInitialHPs();
+}
 
 Fight::~Fight()
 {
@@ -182,16 +195,11 @@ Fight::~Fight()
       delete (*d_def_close.begin());
       d_def_close.erase(d_def_close.begin());
     }
-
 }
 
 void Fight::battle(bool intense)
 {
   d_intense_combat = intense;
-  // at the beginning of the battle, calculate the bonuses
-  // bonuses remain even if the unit providing a stackwide bonus dies
-
-  calculateBonus();
 
   // first, fight until the fight is over
   for (d_turn = 0; doRound(); d_turn++);
@@ -241,15 +249,41 @@ void Fight::battle(bool intense)
     }
 }
 
+Army *findArmyById(const std::list<Stack *> &l, Uint32 id)
+{
+  for (std::list<Stack *>::const_iterator i = l.begin(), end = l.end();
+       i != end; ++i) {
+    Army *a = (*i)->getArmyById(id);
+    if (a)
+      return a;
+  }
+
+  return 0;
+}
+
+void Fight::battleFromHistory()
+{
+  for (std::list<FightItem>::iterator i = d_actions.begin(),
+         end = d_actions.end(); i != end; ++i) {
+    FightItem &f = *i;
+
+    Army *a = findArmyById(d_attackers, f.id);
+    if (!a)
+      a = findArmyById(d_defenders, f.id);
+
+    a->damage(f.damage);
+  }
+}
+
 bool Fight::doRound()
 {
   if (MAX_ROUNDS && d_turn >= MAX_ROUNDS)
     return false;
 
-  debug ("Fight round #" <<d_turn)
+  debug ("Fight round #" <<d_turn);
 
-    //fight the first one in attackers with the first one in defenders
-    std::list<Fighter*>::iterator ffit = d_att_close.begin();
+  //fight the first one in attackers with the first one in defenders
+  std::list<Fighter*>::iterator ffit = d_att_close.begin();
   std::list<Fighter*>::iterator efit = d_def_close.begin();
 
   //have the attacker and defender try to hit each other
@@ -425,18 +459,19 @@ void Fight::calculateModifiedStrengths (std::list<Fighter*>friendly,
       City *c = Citylist::getInstance()->getNearestCity((*fit)->pos);
       if (c && mtile->getBuilding() == Maptile::CITY)
 	{
+	  int num_production_bases = c->getNoOfProductionBases();
 	  if (c->isBurnt()) 
 	    city_bonus = 0;
-	  else if (c->getNoOfBasicProd() <= 2 && c->getPlayer() ==
+	  else if (num_production_bases <= 2 && c->getOwner() ==
 		   Playerlist::getInstance()->getNeutral())
 	    city_bonus = 0;
-	  else if (c->getNoOfBasicProd() <= 2 && c->getPlayer() ==
+	  else if (num_production_bases <= 2 && c->getOwner() ==
 		   Playerlist::getInstance()->getActiveplayer())
 	    city_bonus = 1;
-	  else if (c->getNoOfBasicProd() > 2 && c->getPlayer() ==
+	  else if (num_production_bases > 2 && c->getOwner() ==
 		   Playerlist::getInstance()->getNeutral())
 	    city_bonus = 1;
-	  else if (c->getNoOfBasicProd() > 2 && c->getPlayer() ==
+	  else if (num_production_bases > 2 && c->getOwner() ==
 		   Playerlist::getInstance()->getActiveplayer())
 	    city_bonus = 2;
 	}
@@ -448,8 +483,6 @@ void Fight::calculateModifiedStrengths (std::list<Fighter*>friendly,
 	    city_bonus = 2;
 	}
 
-      // FIXME: implement towers.  they get a city bonus
-
       // does the attacker cancel our city bonus?
       for (fit = enemy.begin(); fit != enemy.end(); fit++)
 	{
@@ -658,3 +691,28 @@ void Fight::remove(Fighter* f)
   // if the fighter wa sin no list, we are rather careful and don't do anything
   debug("Fight: fighter without list!")
 }
+	
+Uint32 Fight::getModifiedStrengthBonus(Army *a)
+{
+  std::list<Fighter*>::iterator it;
+  for (it = d_att_close.begin(); it != d_att_close.end(); it++)
+    if ((*it)->army == a)
+      return (*it)->terrain_strength;
+  for (it = d_def_close.begin(); it != d_def_close.end(); it++)
+    if ((*it)->army == a)
+      return (*it)->terrain_strength;
+  return 0;
+}
+
+void Fight::fillInInitialHPs()
+{
+  for (std::list<Stack *>::iterator i = d_attackers.begin();
+       i != d_attackers.end(); ++i)
+    for (Stack::iterator j = (*i)->begin(); j != (*i)->end(); ++j)
+      initial_hps[(*j)->getId()] = (*j)->getHP();
+  
+  for (std::list<Stack *>::iterator i = d_defenders.begin();
+       i != d_defenders.end(); ++i)
+    for (Stack::iterator j = (*i)->begin(); j != (*i)->end(); ++j)
+      initial_hps[(*j)->getId()] = (*j)->getHP();
+}
