@@ -1,3 +1,6 @@
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -28,7 +32,6 @@
 #include "sound.h"
 #include "GraphicsCache.h"
 #include "GameScenario.h"
-#include "QuestsManager.h"
 #include "NextTurn.h"
 
 #include "gamebigmap.h"
@@ -42,7 +45,6 @@
 #include "ruinlist.h"
 #include "templelist.h"
 #include "signpostlist.h"
-#include "armysetlist.h"
 #include "city.h"
 #include "ruin.h"
 #include "signpost.h"
@@ -59,6 +61,9 @@
 #include "GameMap.h"
 #include "history.h"
 
+#include "herotemplates.h"
+
+Game *Game::current_game = 0;
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
 #define debug(x)
@@ -72,9 +77,6 @@ void Game::addPlayer(Player *p)
     (*it).disconnect();
   connections[p->getId()].clear();
 
-  connections[p->getId()].push_back
-    (p->sdyingStack.connect(sigc::mem_fun(this, &Game::stackDied)));
-
   if (p->getType() == Player::HUMAN)
     {
       connections[p->getId()].push_back
@@ -82,17 +84,18 @@ void Game::addPlayer(Player *p)
       connections[p->getId()].push_back
 	(p->snewMedalArmy.connect(sigc::mem_fun(this, &Game::newMedalArmy)));
       connections[p->getId()].push_back
-	(p->srecruitingHero.connect
-	 (sigc::bind<0>
-	  (sigc::mem_fun
-	   (hero_offers_service, &sigc::signal<bool, Player *, 
-	    Hero *, City *, int>::emit), p)));
+	(p->srecruitingHero.connect(sigc::mem_fun(this, &Game::recruitHero)));
+
       connections[p->getId()].push_back
 	(p->streachery.connect
 	 (sigc::bind<0>
 	  (sigc::mem_fun
 	   (stack_considers_treachery, 
 	    &sigc::signal<bool, Player *, Stack *, Player *, Vector<int> >::emit), p)));
+      connections[p->getId()].push_back
+	(p->hero_arrives_with_allies.connect
+         (sigc::mem_fun
+          (hero_arrives, &sigc::signal<void, int>::emit)));
     }
 	
       
@@ -123,10 +126,22 @@ void Game::addPlayer(Player *p)
      (sigc::mem_fun(advice_asked, &sigc::signal<void, float>::emit)));
 }
 
+#define NETWORK_TESTING 1
+
+#include "game-server.h"
+
+GameServer *game_server = 0;
+
 Game::Game(GameScenario* gameScenario)
     : d_gameScenario(gameScenario) 
 {
+    current_game = this;
     input_locked = false;
+
+#if NETWORK_TESTING
+    game_server = new GameServer();
+    game_server->start();
+#endif
     
     // init the bigmap
     bigmap.reset(new GameBigMap
@@ -191,7 +206,7 @@ Game::Game(GameScenario* gameScenario)
     center_view_on_city();
     update_control_panel();
 
-    loadHeroTemplates();
+    HeroTemplates::getInstance();
 }
 
 Game::~Game()
@@ -205,8 +220,16 @@ Game::~Game()
     }
     delete d_gameScenario;
     delete d_nextTurn;
+    
+    HeroTemplates::deleteInstance();
 }
 
+GameScenario *Game::getScenario()
+{
+  return current_game->d_gameScenario;
+}
+
+
 void Game::end_turn()
 {
     unselect_active_stack();
@@ -221,9 +244,9 @@ void Game::update_stack_info()
 {
     Stack* stack = Playerlist::getActiveplayer()->getActivestack();
 
-    if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
-        GameScenario::s_hidden_map == true)
-      return;
+    //if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+        //GameScenario::s_hidden_map == true)
+      //return;
     stack_info_changed.emit(stack);
 }
 
@@ -244,7 +267,7 @@ void Game::update_sidebar_stats()
     s.income = s.cities = 0;
     for (Citylist::iterator i = Citylist::getInstance()->begin(),
 	     end = Citylist::getInstance()->end(); i != end; ++i)
-	if (i->getPlayer() == player)
+	if (i->getOwner() == player)
 	{
 	    s.income += i->getGold();
 	    ++s.cities;
@@ -264,11 +287,13 @@ void Game::update_sidebar_stats()
 void Game::redraw()
 {
     if (bigmap.get())
+      {
 	bigmap->draw();
+      }
     if (smallmap.get())
       {
-	if (Playerlist::getActiveplayer()->getType() == Player::HUMAN ||
-	    GameScenario::s_hidden_map == false)
+	//if (Playerlist::getActiveplayer()->getType() == Player::HUMAN ||
+	    //GameScenario::s_hidden_map == false)
 	  smallmap->draw();
       }
 }
@@ -284,6 +309,8 @@ void Game::select_next_movable_stack()
 void Game::move_selected_stack_dir(int diffx, int diffy)
 {
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
+  if (!stack)
+    return;
   // Get rid of the old path if there is one
   if (stack->getPath()->size())
     stack->getPath()->flClear();
@@ -351,8 +378,10 @@ void Game::move_selected_stack_along_path()
     Playerlist::getActiveplayer()->stackSplit(stack);
 
   Playerlist::getActiveplayer()->stackMove(stack);
-		  
-  if (stack->canMove() == false)
+
+  //maybe we joined another stack
+  stack = Playerlist::getActiveplayer()->getActivestack();
+  if (stack && stack->canMove() == false)
     {
       Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
       unselect_active_stack();
@@ -499,36 +528,29 @@ void Game::search_selected_stack()
     }
 }
 
-// the parameter is currently not used (=0), but may be used for more detailed
-// descriptions later on
 void Game::stackUpdate(Stack* s)
 {
   if (!s)
     s = Playerlist::getActiveplayer()->getActivestack();
 
+  //FIXME: if player is not to be observed, bail now
   if (s)
-    smallmap->center_view(s->getPos(), true);
+    smallmap->center_view_on_tile(s->getPos(), true);
 
   redraw();
 
   update_stack_info();
   update_control_panel();
-}
 
-// s is currently unused, but can later be filled with reasonable data
-void Game::stackDied(Stack* s)
-{
-  unselect_active_stack();
-  redraw();
-  update_control_panel();
+  // sleep for a specified amount of time
+  SDL_Delay(Configuration::s_displaySpeedDelay);
 }
 
-
 Army::Stat Game::newLevelArmy(Army* a)
 {
   // don't show a dialog if computer or enemy's armies advance
-  if ((a->getPlayer()->getType() != Player::HUMAN) ||
-      (a->getPlayer() != Playerlist::getInstance()->getActiveplayer()))
+  if ((a->getOwner()->getType() != Player::HUMAN) ||
+      (a->getOwner() != Playerlist::getInstance()->getActiveplayer()))
     return Army::STRENGTH;
 
   return army_gains_level.emit(a);
@@ -537,9 +559,9 @@ Army::Stat Game::newLevelArmy(Army* a)
 void Game::newMedalArmy(Army* a)
 {
   // We don't want to have medal awards of computer players displayed
-  if (!a->getPlayer()
-      || (a->getPlayer()->getType() != Player::HUMAN)
-      || a->getPlayer() != Playerlist::getInstance()->getActiveplayer())
+  if (!a->getOwner()
+      || (a->getOwner()->getType() != Player::HUMAN)
+      || a->getOwner() != Playerlist::getInstance()->getActiveplayer())
     return;
 
   medal_awarded_to_army.emit(a);
@@ -556,7 +578,7 @@ void Game::on_city_queried (City* c, bool brief)
 {
   if (c)
     {
-      Player *player = c->getPlayer();
+      Player *player = c->getOwner();
 
       if (brief)
 	{
@@ -644,9 +666,7 @@ void Game::on_stack_queried (Stack* s)
 {
   if (s)
     {
-      Rectangle r = s->get_area();
-      //r.w *= s->size();
-      MapTipPosition mpos = bigmap->map_tip_position(r);
+      MapTipPosition mpos = bigmap->map_tip_position(s->getPos());
       stack_tip_changed.emit(s, mpos);
     }
   else
@@ -680,7 +700,7 @@ void Game::looting_city(City* city, int &gold)
   Citylist *clist = Citylist::getInstance();
   Playerlist *plist = Playerlist::getInstance();
   Player *attacker = plist->getActiveplayer();
-  Player *defender = city->getPlayer();
+  Player *defender = city->getOwner();
   int amt = (defender->getGold() / (2 * clist->countCities (defender)) * 2);
   // give (Enemy-Gold/(2Enemy-Cities)) to the attacker 
   // and then take away twice that from the defender.
@@ -693,30 +713,11 @@ void Game::looting_city(City* city, int &gold)
   return;
 }
 
-void Game::invading_city(City* city)
+void Game::invading_city(City* city, int gold)
 {
-  Playerlist *plist = Playerlist::getInstance();
-  Player *player = plist->getActiveplayer();
-  int gold = 0;
-
-  // See if this is the last city for that player, and alter the 
-  // diplomatic scores.
-  if (Citylist::getInstance()->countCities(city->getPlayer()) == 1)
-    {
-      if (city->getPlayer()->getDiplomaticRank() < 
-	  player->getDiplomaticRank())
-	player->deteriorateDiplomaticRelationship (2);
-      else if (city->getPlayer()->getDiplomaticRank() > 
-	  player->getDiplomaticRank())
-	player->improveDiplomaticRelationship (2, city->getPlayer());
-    }
-
-  // loot the city
-  // if the attacked city isn't neutral, loot some gold
-  if (city->getPlayer() != plist->getNeutral())
-    looting_city (city, gold);
-
-  if (!input_locked)
+  Player *player = Playerlist::getInstance()->getActiveplayer();
+  
+  if (player->getType() == Player::HUMAN)
     {
       redraw();
       CityDefeatedAction a = city_defeated.emit(city, gold);
@@ -750,7 +751,6 @@ void Game::invading_city(City* city)
 	city_visited.emit(city);
     }
 
-  //fixme: check for end of game here
   redraw();
   update_stack_info();
   update_sidebar_stats();
@@ -833,8 +833,8 @@ void Game::update_control_panel()
   if (stack)
     {
       can_move_selected_stack_along_path.emit
-	(stack->getPath()->size() > 0 && stack->enoughMoves() ||
-	 (stack->getPath()->size() && stack->getMovesExhaustedAtPoint() > 0));
+	(!stack->getPath()->empty() && stack->enoughMoves() ||
+	 (!stack->getPath()->empty() && stack->getPath()->getMovesExhaustedAtPoint() > 0));
 
       /*
        * a note about searching.
@@ -948,12 +948,12 @@ SmallMap &Game::get_smallmap()
 
 void Game::startGame()
 {
-
   debug ("start_game()");
   lock_inputs();
 
   d_nextTurn->start();
-  update_control_panel();
+  if (Playerlist::getInstance()->countPlayersAlive())
+    update_control_panel();
 }
 
 void Game::loadGame()
@@ -970,14 +970,20 @@ void Game::loadGame()
       update_stack_info();
       game_loaded.emit(player);
     }
+
+  d_nextTurn->setContinuingTurn();
+  d_nextTurn->start();
+#if 0
   else
     {
       lock_inputs();
       update_sidebar_stats();
       player->startTurn();
       d_nextTurn->endTurn();
-      update_control_panel();
+      if (Playerlist::getInstance()->countPlayersAlive())
+	update_control_panel();
     }
+#endif
 }
 
 void Game::stopGame()
@@ -991,220 +997,30 @@ bool Game::saveGame(std::string file)
   return d_gameScenario->saveGame(file);
 }
 
-
-/*
- *
- * what are the chances of a hero showing up?
- *
- * 1 in 6 if you have enough gold, where "enough gold" is...
- *
- * ... 1500 if the player already has a hero, then:  1500 is generally 
- * enough to buy all the heroes.  I forget the exact distribution of 
- * hero prices but memory says from 1000 to 1500.  (But, if you don't 
- * have 1500 gold, and the price is less, you still get the offer...  
- * So, calculate price, compare to available gold, then decided whether 
- * or not to offer...)
- *
- * ...500 if all your heroes are dead: then prices are cut by about 
- * a factor of 3.
- */
-void Game::maybeRecruitHero (Player *p)
-{
-  City *city;
-  Playerlist *plist = Playerlist::getInstance();
-  int gold_needed = 0;
-  if (Citylist::getInstance()->countCities(p) == 0)
-    return;
-  //give the player a hero if it's the first round.
-  //otherwise we get a hero based on chance
-  //a hero costs a random number of gold pieces
-  if (d_gameScenario->getRound() == 0)
-    gold_needed = 0;
-  else
-    {
-      bool exists = false;
-      Stacklist *stacklist = p->getStacklist();
-      for (Stacklist::iterator it = stacklist->begin(); 
-	   it != stacklist->end(); it++)
-	if ((*it)->hasHero())
-	  exists = true; 
-
-      gold_needed = (rand() % 500) + 1000;
-      if (exists == false)
-	gold_needed /= 3;
-    }
-
-  //we set the chance of some hero recruitment to, ehm, 10 percent
-  if (((((rand() % 6) == 0) && (gold_needed < p->getGold())) 
-       || gold_needed == 0)
-      && (p != plist->getNeutral()))
-    {
-      const Armysetlist* al = Armysetlist::getInstance();
-      int num = rand() % d_herotemplates[p->getId()].size();
-      Hero *templateHero = d_herotemplates[p->getId()][num];
-      const Army *heroType = al->getArmy (p->getArmyset(), 
-					  templateHero->getType());
-      Hero* newhero = new Hero(*heroType, templateHero->getName(), p);
-      newhero->setGender(Army::Gender(templateHero->getGender()));
-      if (gold_needed == 0)
-	city = Citylist::getInstance()->getFirstCity(p);
-      else
-	{
-	  std::vector<City*> cities;
-	  Citylist* cl = Citylist::getInstance();
-	  for (Citylist::iterator it = cl->begin(); it != cl->end(); ++it)
-	    if (!(*it).isBurnt() && (*it).getPlayer() == p)
-	      cities.push_back(&(*it));
-	  if (cities.empty())
-	    return;
-	  city = cities[rand() % cities.size()];
-	}
-
-      bool accepted = p->recruitHero(newhero, city, gold_needed);
-      if (accepted)
-	{
-	  History_HeroEmerges *item = new History_HeroEmerges();
-	  item->fillData(newhero, city);
-	  p->getHistorylist()->push_back(item);
-
-	  newhero->setPlayer(p);
-
-	  int alliesCount;
-	  GameMap::getInstance()->addArmy(city, newhero);
-	  /* now maybe add a few allies */
-	  if (gold_needed > 1300)
-	    alliesCount = 3;
-	  else if (gold_needed > 1000)
-	    alliesCount = 2;
-	  else if (gold_needed > 800)
-	    alliesCount = 1;
-	  else
-	    alliesCount = 0;
-
-	  if (alliesCount > 0)
-	    {
-	      const Army *army = Reward_Allies::randomArmyAlly();
-	      if (army)
-		{
-		  Reward_Allies::addAllies(p, city->getPos(), army,alliesCount);
-		  if (p->getType() == Player::HUMAN)
-		    hero_arrives.emit(alliesCount);
-		}
-	    }
-	  if (gold_needed == 0)
-	    {
-	      // Initially give the first hero the player's standard.
-	      std::string name = p->getName() + " " + _("Standard");
-	      Item *battle_standard = new Item (name, true, p);
-	      battle_standard->setBonus(Item::ADD1STACK);
-	      newhero->addToBackpack(battle_standard, 0);
-	    }
-	  p->withdrawGold(gold_needed);
-	  p->supdatingStack.emit(0);
-	}
-      else
-	delete newhero;
-    }
-  return;
-}
-
-int
-Game::loadHeroTemplates()
-{
-  FILE *fileptr = fopen (File::getMiscFile("heronames").c_str(), "r");
-  char *line = NULL;
-  size_t len = 0;
-  ssize_t read;
-  int retval;
-  int gender;
-  int side;
-  size_t bytesread = 0;
-  char *tmp;
-  const Armysetlist* al = Armysetlist::getInstance();
-  const Army* herotype;
-
-  // list all the army types that are heroes.
-  std::vector<const Army*> heroes;
-  Player *p = Playerlist::getInstance()->getNeutral();
-  for (unsigned int j = 0; j < al->getSize(p->getArmyset()); j++)
-    {
-      const Army *a = al->getArmy (p->getArmyset(), j);
-      if (a->isHero())
-	heroes.push_back(a);
-    }
-
-  if (fileptr == NULL)
-    return -1;
-  while ((read = getline (&line, &len, fileptr)) != -1)
-    {
-      bytesread = 0;
-      retval = sscanf (line, "%d%d%n", &side, &gender, &bytesread);
-      if (retval != 2)
-	{
-	  free (line);
-	  return -2;
-	}
-      while (isspace(line[bytesread]) && line[bytesread] != '\0')
-	bytesread++;
-      tmp = strchr (&line[bytesread], '\n');
-      if (tmp)
-	tmp[0] = '\0';
-      if (strlen (&line[bytesread]) == 0)
-	{
-	  free (line);
-	  return -3;
-	}
-      if (side < 0 || side > (int) MAX_PLAYERS)
-	{
-	  free (line);
-	  return -4;
-	}
-
-      herotype = heroes[rand() % heroes.size()];
-      Hero *newhero = new Hero (*herotype, "", NULL);
-      if (gender)
-	newhero->setGender(Hero::MALE);
-      else
-	newhero->setGender(Hero::FEMALE);
-      newhero->setName (&line[bytesread]);
-      d_herotemplates[side].push_back (newhero);
-    }
-  if (line)
-    free (line);
-  fclose (fileptr);
-  return 0;
-}
-
-bool Game::init_turn_for_player(Player* p)
+void Game::init_turn_for_player(Player* p)
 {
   Playerlist* pl = Playerlist::getInstance();
-  // FIXME: Currently this function only checks for a human player. You
-  // can also have it check for e.g. escape key pressed to interrupt
-  // an AI-only game to save/quit.
-
-  center_view_on_city();
-  if (Playerlist::isFinished())
-    return true; //closing game window while the computer is moving
 
+  if (GameScenario::s_hidden_map && p->getType() == Player::HUMAN)
+    {
+      smallmap->blank();
+      bigmap->blank();
+    }
   next_player_turn.emit(p, d_gameScenario->getRound() + 1);
+  center_view_on_city();
   if (p->getType() == Player::HUMAN)
     {
       unlock_inputs();
 
-
       update_sidebar_stats();
       update_stack_info();
       update_control_panel();
 
-      QuestsManager::getInstance()->nextTurn(p);
-
-      maybeRecruitHero(p);
-
-      if (d_gameScenario->getRound() == 0)
-	{
-	  Citylist *clist = Citylist::getInstance();
-	  city_visited.emit(clist->getFirstCity(p));
-	}
+      //if (d_gameScenario->getRound() == 0)
+	//{
+	  //Citylist *clist = Citylist::getInstance();
+	  //city_visited.emit(clist->getFirstCity(p));
+	//}
 
       // update the diplomacy icon if we've received a proposal
       bool proposal_received = false;
@@ -1223,35 +1039,10 @@ bool Game::init_turn_for_player(Player* p)
 	    }
 	}
       received_diplomatic_proposal.emit(proposal_received);
-
-      return true;
     }
   else
     {
       SDL_Delay(250);
-      QuestsManager::getInstance()->nextTurn(p);
-      maybeRecruitHero(p);
-      if (d_gameScenario->s_cusp_of_war == true &&
-	  d_gameScenario->getRound() == CUSP_OF_WAR_ROUND)
-	{
-	  for (Playerlist::iterator it = pl->begin(); it != pl->end(); ++it)
-	    if ((*it)->getType() == Player::HUMAN)
-	      {
-		if ((*it)->isDead())
-		  continue;
-		if (p->getDiplomaticState(*it) != Player::AT_WAR)
-		  {
-		    p->proposeDiplomacy (Player::PROPOSE_WAR, *it);
-		    (*it)->proposeDiplomacy (Player::PROPOSE_WAR, p);
-		    p->declareDiplomacy (Player::AT_WAR, *it);
-  
-		    History_DiplomacyWar *item1 = new History_DiplomacyWar();
-		    item1->fillData(*it);
-		    p->getHistorylist()->push_back(item1);
-		  }
-	      }
-	}
-	  return false;
     }
 }
 
@@ -1266,26 +1057,28 @@ void Game::on_player_died(Player *player)
 
 void Game::on_fight_started(Fight &fight)
 {
-  Player* pd = (*(fight.getDefenders().begin()))->getPlayer();
-  Player* pa = (*(fight.getAttackers().begin()))->getPlayer();
+#if 0
+  Player* pd = fight.getDefenders().front()->getOwner();
+  Player* pa = fight.getAttackers().front()->getOwner();
   if ((pa->getType() == Player::HUMAN || pd->getType() == Player::HUMAN) ||
       (pa->getType() != Player::HUMAN && pd->getType() != Player::HUMAN 
        && pd != Playerlist::getInstance()->getNeutral()))
-    {
+  {
 
-      if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
-	  GameScenario::s_hidden_map == true)
-	return;
-    }
-  else
-    {
-      //short circuit the battle sequence
-      fight.battle(GameScenario::s_intense_combat);
+    //short circuit the battle sequence
+    if (pa->getType() != Player::HUMAN && pd->getType() != Player::HUMAN &&
+        GameScenario::s_hidden_map == true)
       return;
-    }
+  }
+  else
+    return; //short circuit the battle sequence
+#endif
+  
+  //FIXME: zoom the map here if we're attacking an observable human, 
+  //from an unobservable computer player
   bigmap->setFighting(true);
   bigmap->draw();
-  fight_started.emit(fight, GameScenario::s_intense_combat);
+  fight_started.emit(fight);
   bigmap->setFighting(false);
   bigmap->draw();
 }
@@ -1296,13 +1089,19 @@ void Game::center_view_on_city()
 
   if (p == Playerlist::getInstance()->getNeutral())
     return;
+  if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+      GameScenario::s_hidden_map == true)
+    return;
+  //FIXME: if player is not to be observed, bail now
   // preferred city is a capital city that belongs to the player 
   for (Citylist::iterator i = Citylist::getInstance()->begin();
        i != Citylist::getInstance()->end(); i++)
     {
-      if (i->getPlayer() == p && i->isCapital())
+      if (i->getOwner() == p && i->isCapital() &&
+	  i->getCapitalOwner() == p)
 	{
-	  smallmap->center_view(i->getPos(), true);
+	  smallmap->center_view_on_tile(i->getPos(), 
+					!GameScenario::s_hidden_map);
 	  return;
 	}
     }
@@ -1311,19 +1110,23 @@ void Game::center_view_on_city()
   for (Citylist::iterator i = Citylist::getInstance()->begin();
        i != Citylist::getInstance()->end(); i++)
     {
-      if (i->getPlayer() == p)
+      if (i->getOwner() == p)
 	{
 	  if (Playerlist::isFinished())
 	    return;
-	  smallmap->center_view(i->getPos(), true);
+	  smallmap->center_view_on_tile(i->getPos(), 
+					!GameScenario::s_hidden_map);
 	  break;
 	}
     }
 }
 void Game::select_active_stack()
 {
+  //if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+      //GameScenario::s_hidden_map == true)
+    //return;
   Player *p = Playerlist::getInstance()->getActiveplayer();
-  smallmap->center_view(p->getActivestack()->getPos(), true);
+  smallmap->center_view_on_tile (p->getActivestack()->getPos(), true);
   bigmap->select_active_stack();
 }
 void Game::unselect_active_stack()
@@ -1333,7 +1136,7 @@ void Game::unselect_active_stack()
 
 bool Game::maybeTreachery(Stack *stack, Player *them, Vector<int> pos)
 {
-  Player *me = stack->getPlayer();
+  Player *me = stack->getOwner();
   bool treachery = me->treachery (stack, them, pos);
   if (treachery == false)
     return false;
@@ -1389,10 +1192,10 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
       //if this is a neutral city, and we're playing with 
       //active neutral cities, AND it hasn't already been attacked
       //then it's production gets turned on
-      Player *neu = city->getPlayer(); //neutral player
+      Player *neu = city->getOwner(); //neutral player
       if (GameScenario::s_neutral_cities == GameParameters::ACTIVE &&
 	  neu == Playerlist::getInstance()->getNeutral() &&
-	  city->getProductionIndex() == -1)
+	  city->getActiveProductionSlot() == -1)
 	{
 	  //great, then let's turn on the production.
 	  //well, we already made a unit, and we want to produce more
@@ -1401,12 +1204,12 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
 	  if (o)
 	    {
 	      int army_type = o->getStrongestArmy()->getType();
-	      for (int i = 0; i < 4; i++)
+	      for (int i = 0; i < city->getMaxNoOfProductionBases(); i++)
 		{
 		  if (city->getArmytype(i) == army_type)
 		    {
 		      // hey, we found the droid we were looking for
-		      city->setProduction(i);
+		      city->setActiveProductionSlot(i);
 		      break;
 		    }
 		}
@@ -1415,3 +1218,11 @@ void Game::on_city_fight_finished(City *city, Fight::Result result)
     }
   return;
 }
+    
+bool Game::recruitHero(Hero *hero, City *city, int gold)
+{
+  bool retval = hero_offers_service (city->getOwner(), hero, city, gold);
+  if (d_gameScenario->getRound() == 0)
+    city_visited.emit(city);
+  return retval;
+}
