@@ -1,3 +1,6 @@
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef GAME_H
 #define GAME_H
@@ -86,7 +90,7 @@ class Game
     // save current game, returns true if successful
     bool saveGame(std::string file);
 
-    GameScenario *getScenario() {return d_gameScenario;};
+    static GameScenario *getScenario();
     GameBigMap &get_bigmap();
     SmallMap &get_smallmap();
     
@@ -117,7 +121,7 @@ class Game
     sigc::signal<void, Stack *, MapTipPosition> stack_tip_changed;
     sigc::signal<void, Ruin*, Stack*, Reward*> ruin_searched;
     sigc::signal<void, Ruin*, Stack*> sage_visited;
-    sigc::signal<void, Fight &, bool> fight_started;
+    sigc::signal<void, Fight &> fight_started;
     sigc::signal<void, Stack *, Stack *> ruinfight_started;
     sigc::signal<void, float> advice_asked;
     sigc::signal<void, Fight::Result> ruinfight_finished;
@@ -145,12 +149,7 @@ class Game
     void addPlayer(Player *p);
 
  private:
-
-    /**
-    \brief The function reads in the heronames file and produces a
-    set of hero templates to be randomly selected from
-    */
-    int loadHeroTemplates();
+    static Game *current_game;
 
     // move the selected stack one tile in a given direction
     void move_selected_stack_dir(int diffx, int diffy);
@@ -167,9 +166,6 @@ class Game
     void lock_inputs();
     void unlock_inputs();
 
-    //! Possibly recruit a new hero at the beginning of a turn
-    void maybeRecruitHero(Player *p);
-
     //! Maybe peform treachery
     bool maybeTreachery(Stack *stack, Player *them, Vector<int> pos);
 
@@ -185,18 +181,15 @@ class Game
     void on_smallmap_changed(SDL_Surface *map);
     
     // misc. callbacks
-    void invading_city(City* city);
-    // returns true if the next turn loop should stop
-    bool init_turn_for_player(Player* p);
+    void invading_city(City* city, int gold);
+    void init_turn_for_player(Player* p);
     void on_player_died(Player *p);
     //! Callback when the army of a human player reaches a new level.
     Army::Stat newLevelArmy(Army* a);
     //! Callback when an army gets a new medal.
     void newMedalArmy(Army* a);
-    //! Called whenever a stack has moved, updates the map etc.
+    //! Called whenever a stack has changed, updates the map etc.
     void stackUpdate(Stack* s);
-    //! Called whenever a stack has died; updates bigmap as well
-    void stackDied(Stack* s);
     //! Called whenever players fight
     void on_fight_started(Fight &fight);
     //! Called whenever a player receives an offer of surrender
@@ -204,20 +197,17 @@ class Game
     void nextRound();
     //! Called after a player's stack attacks a city
     void on_city_fight_finished(City *city, Fight::Result result);
-
-
     
     void looting_city(City *city, int &gold);
     void unselect_active_stack();
     void select_active_stack();
+    bool recruitHero(Hero *hero, City *city, int gold);
 
     GameScenario* d_gameScenario;
     NextTurn* d_nextTurn;
     std::auto_ptr<GameBigMap> bigmap;
     std::auto_ptr<SmallMap> smallmap;
 
-    /* the contents of the heronames data file */
-    std::vector<Hero*> d_herotemplates[MAX_PLAYERS];
 
     bool input_locked;
 
