@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -40,6 +44,7 @@
 #include "GameMap.h"
 #include "GraphicsCache.h"
 #include "game.h"
+#include "FogMap.h"
 
 #include "timing.h"
 
@@ -155,7 +160,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    {
 	      //restrict going into enemy cities unless they're only
 	      //one square away
-	      if (c->getPlayer() != Playerlist::getActiveplayer())
+	      if (c->getOwner() != Playerlist::getActiveplayer())
 		{
 		  int delta = abs(tile.x - stack->getPos().x);
 		  if (delta <= 1)
@@ -179,8 +184,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	      Playerlist::getActiveplayer()->stackMove(stack);
 	      if (!Playerlist::getActiveplayer()->getActivestack())
 		{
-		  if (stack->canMove() == false)
-		    unselect_active_stack();
+		  unselect_active_stack();
 		  return;
 		}
 	      else
@@ -201,17 +205,17 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		  bool deselect = false;
 		  if (path_already_set)
 		    {
-		      if (stack->getPath()->size() > 0 && 
+		      if (!stack->getPath()->empty() && 
 			  stack->enoughMoves() == false)
 			deselect = true;
-		      else if (stack->getPath()->size() > 0 &&
-			       stack->getMovesExhaustedAtPoint() == 0)
+		      else if (!stack->getPath()->empty() &&
+			       stack->getPath()->getMovesExhaustedAtPoint() == 0)
 			deselect = true;
 		    }
 		  else
 		    {
-		      if (stack->getPath()->size() > 0 && 
-			  stack->getMovesExhaustedAtPoint() == 0)
+		      if (!stack->getPath()->empty() && 
+			  stack->getPath()->getMovesExhaustedAtPoint() == 0)
 			deselect = true;
 		      if ((d_cursor == GraphicsCache::SWORD ||
 			   d_cursor == GraphicsCache::HEART) &&
@@ -251,7 +255,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 			city_queried (c, false);
 		      else
 			{
-			  if (c->getPlayer() == Playerlist::getActiveplayer())
+			  if (c->getOwner() == Playerlist::getActiveplayer())
 			    city_queried (c, false);
 			}
 		    }
@@ -307,7 +311,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		  stack_queried.emit(st);
 		  mouse_state = SHOWING_STACK;
 		}
-	      else if (st->getPlayer() == Playerlist::getActiveplayer() && 
+	      else if (st->getOwner() == Playerlist::getActiveplayer() && 
 		       d_see_opponents_stacks == false)
 		{
 		  stack_queried.emit(st);
@@ -370,7 +374,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 	  City *c = Citylist::getInstance()->getObjectAt(tile);
 	  if (c)
 	    {
-	      if (c->getPlayer() == Playerlist::getActiveplayer())
+	      if (c->getOwner() == Playerlist::getActiveplayer())
 		d_cursor = GraphicsCache::FEET;
 	      else if (c->isBurnt() == true)
 		d_cursor = GraphicsCache::FEET;
@@ -385,8 +389,8 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 			d_cursor = GraphicsCache::QUESTION;
 		      else
 			{
-			  Player *me = stack->getPlayer();
-			  Player *them = c->getPlayer();
+			  Player *me = stack->getOwner();
+			  Player *them = c->getOwner();
 			  bool friendly = (me->getDiplomaticState(them) == 
 					   Player::AT_PEACE);
 			  if (friendly)
@@ -408,12 +412,36 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 	  else
 	    {
 	      Maptile *t = GameMap::getInstance()->getTile(tile);
-	      Stack *st;
-	      st = Stacklist::getObjectAt(tile);
-	      if (!st)
+	      Stack *st = Stacklist::getObjectAt(tile);
+	      if (st && st->getOwner() != Playerlist::getActiveplayer())
+	        {
+	          int delta = abs(stack->getPos().x - st->getPos().x);
+	          if (delta <= 1)
+	            delta = abs(stack->getPos().y - st->getPos().y);
+	          if (delta <= 1)
+	            {
+	              if (is_shift_key_down())
+	        	    d_cursor = GraphicsCache::QUESTION;
+	              else
+	                {
+	                  Player *me = stack->getOwner();
+	                  Player *them = st->getOwner();
+	                  bool friendly = (me->getDiplomaticState(them) == 
+	            		       Player::AT_PEACE);
+	                  if (friendly)
+	            	d_cursor = GraphicsCache::HEART;
+	                  else
+	            	d_cursor = GraphicsCache::SWORD;
+	                }
+	            }
+	          else
+	            d_cursor = GraphicsCache::HAND;
+	        }
+              else
 		{
-		  Stack *empty = new Stack (*stack);
-		  if (empty->getPath()->calculate(empty, tile) == 0)
+                  Path path;
+                  int moves = path.calculate(stack, tile);
+		  if (moves == 0)
 		    d_cursor = GraphicsCache::HAND;
 		  else
 		    {
@@ -424,36 +452,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		      else
 			d_cursor = GraphicsCache::FEET;
 		    }
-		  delete empty;
 		}
-	      else
-		{
-		  if (st->getPlayer() != Playerlist::getActiveplayer())
-		    {
-		      int delta = abs(stack->getPos().x - st->getPos().x);
-		      if (delta <= 1)
-			delta = abs(stack->getPos().y - st->getPos().y);
-		      if (delta <= 1)
-			{
-			  if (is_shift_key_down())
-		    	    d_cursor = GraphicsCache::QUESTION;
-			  else
-			    {
-			      Player *me = stack->getPlayer();
-			      Player *them = st->getPlayer();
-			      bool friendly = (me->getDiplomaticState(them) == 
-					       Player::AT_PEACE);
-			      if (friendly)
-				d_cursor = GraphicsCache::HEART;
-			      else
-				d_cursor = GraphicsCache::SWORD;
-			    }
-			}
-		      else
-			d_cursor = GraphicsCache::HAND;
-		    }
-		}
-
 	    }
 	}
     }
@@ -465,7 +464,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
       st = Playerlist::getActiveplayer()->getStacklist()->getObjectAt(tile);
       if (st)
 	{
-	  if (st->getPlayer() == Playerlist::getActiveplayer())
+	  if (st->getOwner() == Playerlist::getActiveplayer())
 	    d_cursor = GraphicsCache::TARGET;
 	  else
 	    d_cursor = GraphicsCache::HAND;
@@ -478,7 +477,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 	      City *c = Citylist::getInstance()->getObjectAt(tile);
 	      if (c->isBurnt() == true)
 		d_cursor = GraphicsCache::HAND;
-	      else if (c->getPlayer() == Playerlist::getActiveplayer())
+	      else if (c->getOwner() == Playerlist::getActiveplayer())
 		d_cursor = GraphicsCache::ROOK;
 	      else if (d_see_opponents_production == true)
 		d_cursor = GraphicsCache::ROOK;
@@ -580,8 +579,7 @@ void GameBigMap::after_draw()
 	  r2.y = pos.y + offset;
 	  r2.w = r2.h = wpsize;
 
-	  //fixme: check to see if we can move here
-	  canMoveThere = (pathcount < stack->getMovesExhaustedAtPoint());
+	  canMoveThere = (pathcount < stack->getPath()->getMovesExhaustedAtPoint());
 	  if (canMoveThere)
 	    r1.x = 0;
 	  else
@@ -598,7 +596,8 @@ void GameBigMap::after_draw()
     {
       // draw the selection
       Vector<int> p = stack->getPos();
-      if (is_inside(buffer_view, Vector<int>(p.x, p.y)))
+      if (is_inside(buffer_view, Vector<int>(p.x, p.y)) &&
+	  FogMap::isFogged(p) == false)
 	{
 	  static int bigframe = -1;
 	  static int smallframe = -1;
@@ -634,9 +633,9 @@ void GameBigMap::after_draw()
 	  else
 	    {
 	      if (num_selected > 1)
-		tmp = gc->getSelectorPic(0, bigframe, stack->getPlayer());
+		tmp = gc->getSelectorPic(0, bigframe, stack->getOwner());
 	      else
-		tmp = gc->getSelectorPic(1, smallframe, stack->getPlayer());
+		tmp = gc->getSelectorPic(1, smallframe, stack->getOwner());
 	      SDL_BlitSurface(tmp, 0, buffer, &r);
 	    }
 
