@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -94,7 +98,8 @@ BuyProductionDialog::BuyProductionDialog(City *c)
 	toggle->signal_button_release_event().connect(
 	    sigc::bind(sigc::mem_fun(*this, &BuyProductionDialog::on_production_button_event),
 		       toggle), false);
-	toggle->set_sensitive(city->isAlreadyBought(purchasables[i]) == false);
+	toggle->set_sensitive
+	  (city->hasProductionBase(purchasables[i]) == false);
     }
 
     ignore_toggles = false;
@@ -180,11 +185,12 @@ void BuyProductionDialog::set_buy_button_state()
 	can_buy = false;
     else
     {
-	int gold = city->getPlayer()->getGold();
+	int gold = city->getOwner()->getGold();
 	const Army *a = army_id_to_army();
 	
 	if (int(a->getProductionCost()) > gold ||
-	    city->hasProduction(selected_army, city->getPlayer()->getArmyset()))
+	    city->hasProductionBase(selected_army, 
+				    city->getOwner()->getArmyset()))
 	    can_buy = false;
     }
     
