@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -86,7 +90,7 @@ CityWindow::CityWindow(City *c, bool razing_possible,
     raze_button->signal_clicked().connect(
 	sigc::mem_fun(this, &CityWindow::on_raze_clicked));
 
-    for (int i = 1; i <= city->getMaxNoOfBasicProd(); ++i) {
+    for (int i = 1; i <= city->getMaxNoOfProductionBases(); ++i) {
 	Gtk::ToggleButton *toggle;
 	xml->get_widget(String::ucompose("production_toggle%1", i), toggle);
 	production_toggles.push_back(toggle);
@@ -161,9 +165,9 @@ void CityWindow::fill_in_city_info()
 
 void CityWindow::fill_in_production_toggles()
 {
-    Player *player = city->getPlayer();
+    Player *player = city->getOwner();
     unsigned int as = player->getArmyset();
-    int production_index = city->getProductionIndex();
+    int production_index = city->getActiveProductionSlot();
     int type;
     Glib::RefPtr<Gdk::Pixbuf> pic;
     GraphicsCache *gc = GraphicsCache::getInstance();
@@ -175,7 +179,7 @@ void CityWindow::fill_in_production_toggles()
     empty_pic->fill(0x00000000);
     
     ignore_toggles = true;
-    for (int i = 0; i < city->getMaxNoOfBasicProd(); i++)
+    for (int i = 0; i < city->getMaxNoOfProductionBases(); i++)
     {
 	Gtk::ToggleButton *toggle = production_toggles[i];
 	toggle->foreach(sigc::mem_fun(toggle, &Gtk::Container::remove));
@@ -202,7 +206,7 @@ void CityWindow::fill_in_production_toggles()
 
 void CityWindow::on_production_toggled(Gtk::ToggleButton *toggle)
 {
-    if (city->getPlayer() != Playerlist::getActiveplayer())
+    if (city->getOwner() != Playerlist::getActiveplayer())
     {
         toggle->set_active(false);
         return;
@@ -223,9 +227,9 @@ void CityWindow::on_production_toggled(Gtk::ToggleButton *toggle)
     bool is_empty = city->getArmytype(slot) == -1;
     
     if (is_empty)
-	city->getPlayer()->cityChangeProduction(city, -1);
+	city->getOwner()->cityChangeProduction(city, -1);
     else
-	city->getPlayer()->cityChangeProduction(city, slot);
+	city->getOwner()->cityChangeProduction(city, slot);
 
     on_hold_button->set_sensitive(!is_empty);
     
@@ -234,11 +238,11 @@ void CityWindow::on_production_toggled(Gtk::ToggleButton *toggle)
 
 void CityWindow::fill_in_production_info()
 {
-    Player *player = city->getPlayer();
+    Player *player = city->getOwner();
     unsigned int as = player->getArmyset();
     Glib::RefPtr<Gdk::Pixbuf> pic;
     GraphicsCache *gc = GraphicsCache::getInstance();
-    int slot = city->getProductionIndex();
+    int slot = city->getActiveProductionSlot();
     SDL_Surface *s
 	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL);
     Glib::RefPtr<Gdk::Pixbuf> empty_pic
@@ -260,7 +264,7 @@ void CityWindow::fill_in_production_info()
     }
     else
     {
-        const Army* a = city->getArmy(slot);
+        const Army* a = city->getProductionBase(slot);
 
 	// fill in first column
 	s1 += a->getName();
@@ -295,7 +299,7 @@ void CityWindow::fill_in_production_info()
     turns_left_label->set_markup("<i>" + s3 + "</i>");
     current_label->set_markup("<i>" + s4 + "</i>");
 
-    if (city->getPlayer () != Playerlist::getActiveplayer())
+    if (city->getOwner () != Playerlist::getActiveplayer())
       {
         turns_left_label->hide();
         current_label->hide();
@@ -342,7 +346,7 @@ bool CityWindow::on_production_button_event(GdkEventButton *e, Gtk::ToggleButton
 	}
 	assert(slot != -1);
 
-	const Army *army = city->getArmy(slot);
+	const Army *army = city->getProductionBase(slot);
 
 	if (army)
 	    army_info_tip.reset(new ArmyInfoTip(toggle, army));
@@ -360,7 +364,7 @@ bool CityWindow::on_production_button_event(GdkEventButton *e, Gtk::ToggleButton
 void CityWindow::on_on_hold_clicked() //stop button
 {
     city->setVectoring(Vector<int>(-1,-1));
-    city->getPlayer()->cityChangeProduction(city, -1);
+    city->getOwner()->cityChangeProduction(city, -1);
     on_hold_button->set_sensitive(false);
     ignore_toggles = true;
     for (unsigned int i = 0; i < production_toggles.size(); ++i)
@@ -385,12 +389,12 @@ void CityWindow::on_buy_clicked()
 	if  (slot == -1)
 	  {
 	    //no free slots available.  change the one we're on.
-	    slot = city->getProductionIndex();
+	    slot = city->getActiveProductionSlot();
 	    if (slot == -1) 
 	      slot = 0;
 	  }
-	city->getPlayer()->cityBuyProduction(city, slot, army);
-	city->getPlayer()->cityChangeProduction(city, slot);
+	city->getOwner()->cityBuyProduction(city, slot, army);
+	city->getOwner()->cityChangeProduction(city, slot);
 
 	fill_in_production_toggles();
 	fill_in_production_info();
