@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -184,11 +188,11 @@ void DestinationDialog::fill_in_vectoring_info()
   VectoredUnitlist *vul = VectoredUnitlist::getInstance();
   dialog->set_title(city->getName());
 
-  Player *player = city->getPlayer();
+  Player *player = city->getOwner();
   unsigned int as = player->getArmyset();
   Glib::RefPtr<Gdk::Pixbuf> pic;
   GraphicsCache *gc = GraphicsCache::getInstance();
-  int slot = city->getProductionIndex();
+  int slot = city->getActiveProductionSlot();
   SDL_Surface *s
     = GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL);
   Glib::RefPtr<Gdk::Pixbuf> empty_pic
@@ -222,7 +226,7 @@ void DestinationDialog::fill_in_vectoring_info()
     }
   else
     {
-      const Army* a = city->getArmy(slot);
+      const Army* a = city->getProductionBase(slot);
       pic = to_pixbuf(gc->getArmyPic(as, a->getType(), player, NULL));
       s1 = String::ucompose(_("%1t"), city->getDuration());
       turns_label->set_markup("<i>" + s1 + "</i>");
