@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, 2008, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -38,9 +42,8 @@
 #include "../army.h"
 #include "../GraphicsCache.h"
 
-FightWindow::FightWindow(Fight &fight, bool intense_combat)
+FightWindow::FightWindow(Fight &fight)
 {
-  bool d_intense_combat = intense_combat;
     Glib::RefPtr<Gnome::Glade::Xml> xml
 	= Gnome::Glade::Xml::create(get_glade_path() + "/fight-window.glade");
 
@@ -66,12 +69,14 @@ FightWindow::FightWindow(Fight &fight, bool intense_combat)
     // add the armies
     std::vector<Gtk::HBox *> close_hboxes;
     int close;
+    std::map<Uint32, Uint32> initial_hps = fight.getInitialHPs();
 
     // ... attackers
     close = 0;
     for (armies_type::iterator i = attackers.begin(); i != attackers.end(); ++i)
     {
-      add_army(*i, close_hboxes, attacker_close_vbox, close++, rows,
+      add_army(*i, initial_hps[(*i)->getId()],
+               close_hboxes, attacker_close_vbox, close++, rows,
 	       Gtk::ALIGN_LEFT);
     }
 
@@ -81,7 +86,8 @@ FightWindow::FightWindow(Fight &fight, bool intense_combat)
     close = 0;
     for (armies_type::iterator i = defenders.begin(); i != defenders.end(); ++i)
     {
-	add_army(*i, close_hboxes, defender_close_vbox, close++, rows,
+	add_army(*i, initial_hps[(*i)->getId()],
+                 close_hboxes, defender_close_vbox, close++, rows,
 		 Gtk::ALIGN_LEFT);
     }
     
@@ -90,16 +96,15 @@ FightWindow::FightWindow(Fight &fight, bool intense_combat)
     Player* p;
 
     Gtk::Image *defender_shield_image;
-    p = defenders.front()->getPlayer();
+    p = defenders.front()->getOwner();
     xml->get_widget("defender_shield_image", defender_shield_image);
     defender_shield_image->property_pixbuf()=to_pixbuf(gc->getShieldPic(2, p));
 
     Gtk::Image *attacker_shield_image;
-    p = attackers.front()->getPlayer();
+    p = attackers.front()->getOwner();
     xml->get_widget("attacker_shield_image", attacker_shield_image);
     attacker_shield_image->property_pixbuf()=to_pixbuf(gc->getShieldPic(2, p));
   
-    fight.battle(d_intense_combat);
     actions = fight.getCourseOfEvents();
     d_quick = false;
 }
@@ -210,7 +215,7 @@ int FightWindow::compute_max_rows(const armies_type &attackers,
     return old_height;
 }
 
-void FightWindow::add_army(Army *army,
+void FightWindow::add_army(Army *army, int initial_hp,
 			   std::vector<Gtk::HBox *> &hboxes,
 			   Gtk::VBox *vbox,
 			   int current_no, int max_rows,
@@ -225,7 +230,7 @@ void FightWindow::add_army(Army *army,
     
     // hit points graph
     Gtk::ProgressBar *progress = manage(new Gtk::ProgressBar);
-    progress->set_fraction(double(army->getHP()) / army->getStat(Army::HP));
+    progress->set_fraction(double(initial_hp) / army->getStat(Army::HP));
     progress->property_width_request() = army->getPixmap()->w;
     progress->property_height_request() = 12;
     army_box->pack_start(*progress, Gtk::PACK_SHRINK, 4);
@@ -251,9 +256,10 @@ void FightWindow::add_army(Army *army,
     // finally add an entry for later use
     ArmyItem item;
     item.army = army;
-    item.hp = army->getHP();
+    item.hp = initial_hp;
     item.bar = progress;
     item.image = image;
+    item.exploding = false;
     army_items.push_back(item);
 }
 
@@ -261,55 +267,59 @@ bool FightWindow::do_round()
 {
   GraphicsCache *gc = GraphicsCache::getInstance();
   SDL_Surface *expl = gc->getExplosionPic();
-  static Gtk::Image *explosion = NULL;
-  if (explosion)
+
+  // first we clear out any explosions
+  for (army_items_type::iterator i = army_items.begin(),
+         end = army_items.end(); i != end; ++i)
+  {
+    if (!i->exploding)
+      continue;
+    
+    Glib::RefPtr<Gdk::Pixbuf> empty_pic
+      = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, expl->w, expl->h);
+    empty_pic->fill(0x00000000);
+    i->image->property_pixbuf() = empty_pic;
+    i->exploding = false;
+    return Timing::CONTINUE;
+  }
+
+  while (action_iterator != actions.end())
+  {
+    FightItem &f = *action_iterator;
+	
+    ++action_iterator;
+
+    for (army_items_type::iterator i = army_items.begin(),
+           end = army_items.end(); i != end; ++i)
+      if (i->army->getId() == f.id)
+      {
+        i->hp -= f.damage;
+        if (i->hp < 0)
+          i->hp = 0;
+        double fraction = double(i->hp) / i->army->getStat(Army::HP);
+        i->bar->set_fraction(fraction);
+        if (fraction == 0.0)
+        {
+          i->bar->hide();
+          i->image->property_pixbuf() = to_pixbuf(expl);
+          i->exploding = true;
+        }
+		
+        break;
+      }
+
+    if (f.turn > round)
     {
-      Glib::RefPtr<Gdk::Pixbuf> empty_pic
-	= Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, expl->w, expl->h);
-      empty_pic->fill(0x00000000);
-      explosion->property_pixbuf() = empty_pic;
-      explosion = NULL;
+      ++round;
+
       return Timing::CONTINUE;
     }
+  }
 
-    while (action_iterator != actions.end())
-    {
-	FightItem &f = *action_iterator;
-	
-	++action_iterator;
-
-	for (army_items_type::iterator i = army_items.begin(),
-		 end = army_items.end(); i != end; ++i)
-	    if (i->army->getId() == f.id)
-	    {
-		i->hp -= f.damage;
-		if (i->hp < 0)
-		    i->hp = 0;
-		double fraction = double(i->hp) / i->army->getStat(Army::HP);
-		i->bar->set_fraction(fraction);
-		if (fraction == 0.0)
-		  {
-		    i->bar->hide();
-		    //i->image->get_pixbuf()->fill(0x00000000);
-		    i->image->property_pixbuf() = to_pixbuf(expl);
-		    explosion = i->image;
-		  }
-		
-		break;
-	    }
-
-	if (f.turn > round)
-	{
-	    ++round;
-
-	    return Timing::CONTINUE;
-	}
-    }
-
-    window->hide();
-    main_loop->quit();
+  window->hide();
+  main_loop->quit();
     
-    return Timing::STOP;
+  return Timing::STOP;
 }
 void FightWindow::on_key_release_event(GdkEventKey* event)
 {
