@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -124,6 +128,7 @@ GamePreferencesDialog::GamePreferencesDialog()
     Gtk::FileFilter map_filter;
     map_filter.add_pattern("*.map");
     map_filter.set_name(_("LordsAWar map files (*.map)"));
+    load_map_filechooser->set_current_folder(Configuration::s_savePath);
     load_map_filechooser->set_filter(map_filter);
 
 
@@ -298,6 +303,7 @@ void GamePreferencesDialog::on_random_map_toggled()
 	    (*c)->set_sensitive(true);
 	    (*e)->set_sensitive(true);
 	  }
+	start_game_button->set_sensitive(true);
       }
     else
       {
@@ -331,6 +337,10 @@ void GamePreferencesDialog::on_random_map_toggled()
 		(*e)->set_name((*i).name);
 	      }
 	  }
+	else
+	  {
+	    start_game_button->set_sensitive(false);
+	  }
       }
 
     load_map_filechooser->set_sensitive(!random_map);
@@ -452,21 +462,21 @@ void GamePreferencesDialog::on_start_game_clicked()
 	g.map_path = "";
 	switch (map_size_combobox->get_active_row_number()) {
 	case MAP_SIZE_SMALL:
-	    g.map.width = 70;
-	    g.map.height = 105;
+	    g.map.width = MAP_SIZE_SMALL_WIDTH;
+	    g.map.height = MAP_SIZE_SMALL_HEIGHT;
 	    g.map.ruins = 20;
 	    break;
 	
 	case MAP_SIZE_TINY:
-	    g.map.width = 50;
-	    g.map.height = 75;
+	    g.map.width = MAP_SIZE_TINY_WIDTH;
+	    g.map.height = MAP_SIZE_TINY_HEIGHT;
 	    g.map.ruins = 15;
 	    break;
 	
 	case MAP_SIZE_NORMAL:
 	default:
-	    g.map.width = 112;
-	    g.map.height = 156;
+	    g.map.width = MAP_SIZE_NORMAL_WIDTH;
+	    g.map.height = MAP_SIZE_NORMAL_HEIGHT;
 	    g.map.ruins = 35;
 	    break;
 	}
@@ -733,7 +743,6 @@ void GamePreferencesDialog::on_map_chosen()
   if (selected_filename.empty())
     return;
   //try to read the file, grab the players
-  printf ("got it : %s\n", selected_filename.c_str());
   XML_Helper helper(selected_filename, std::ios::in, Configuration::s_zipfiles);
 
   load_map_parameters.players.clear();
