@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, 2008, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -32,6 +36,7 @@
 #include <gdkmm/cursor.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/dialog.h>
+#include <gtkmm/aboutdialog.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/entry.h>
 #include <gtkmm/filechooserdialog.h>
@@ -55,6 +60,7 @@
 #include "ruin-rewarded-dialog.h"
 #include "hero-offer-dialog.h"
 #include "surrender-dialog.h"
+#include "surrender-refused-dialog.h"
 #include "quest-report-dialog.h"
 #include "quest-assigned-dialog.h"
 #include "quest-completed-dialog.h"
@@ -69,6 +75,7 @@
 #include "triumphs-dialog.h"
 #include "diplomacy-report-dialog.h"
 #include "diplomacy-dialog.h"
+#include "stack-info-dialog.h"
 
 #include "../ucompose.hpp"
 #include "../defs.h"
@@ -85,6 +92,7 @@
 #include "../stacklist.h"
 #include "../signpostlist.h"
 #include "../playerlist.h"
+#include "../citylist.h"
 #include "../hero.h"
 #include "../temple.h"
 #include "../city.h"
@@ -246,6 +254,7 @@ GameWindow::GameWindow()
     xml->get_widget("end_turn_menuitem", end_turn_menuitem);
     xml->get_widget("move_all_menuitem", move_all_menuitem);
     xml->get_widget("disband_menuitem", disband_menuitem);
+    xml->get_widget("stack_info_menuitem", stack_info_menuitem);
     xml->get_widget("signpost_menuitem", signpost_menuitem);
     xml->get_widget("search_menuitem", search_menuitem);
     xml->get_widget("inspect_menuitem", inspect_menuitem);
@@ -274,6 +283,8 @@ GameWindow::GameWindow()
 			 sigc::mem_fun(*this, &GameWindow::on_production_report_activated));
     xml->connect_clicked("triumphs_menuitem",
 			 sigc::mem_fun(*this, &GameWindow::on_triumphs_activated));
+    xml->connect_clicked("help_about_menuitem",
+			 sigc::mem_fun(*this, &GameWindow::on_help_about_activated));
     d_quick_fights = false;
 }
 
@@ -428,18 +439,27 @@ void GameWindow::new_game(GameParameters g)
 	g.map_path = path;
     }
 
-    setup_game(g.map_path);
+    bool success = setup_game(g.map_path);
+    if (!success)
+      return;
     setup_signals();
     game->startGame();
+    //FIXME: we don't get here until the game ends.
+    //is that right?
 }
 
 void GameWindow::load_game(std::string file_path)
 {
     current_save_filename = file_path;
-    setup_game(file_path);
+    bool success = setup_game(file_path);
+    if (!success)
+      return;
     setup_signals();
     game->loadGame();
-    game->redraw();
+    //FIXME: we don't get here until the game ends.
+    //is that right?
+    if (Playerlist::getInstance()->countPlayersAlive())
+      game->redraw();
 }
 
 void GameWindow::setup_button(Gtk::Button *button,
@@ -479,6 +499,9 @@ void GameWindow::setup_signals()
    (sigc::mem_fun (*this, &GameWindow::on_diplomacy_button_clicked)));
   connections.push_back 
     (game->received_diplomatic_proposal.connect 
+     (sigc::mem_fun(*this, &GameWindow::change_diplomacy_button_image)));
+  connections.push_back 
+    (game->can_end_turn.connect 
      (sigc::mem_fun(*this, &GameWindow::update_diplomacy_button)));
 
   setup_button(defend_button,
@@ -535,6 +558,9 @@ void GameWindow::setup_signals()
   setup_menuitem(disband_menuitem,
 		 sigc::mem_fun(*this, &GameWindow::on_disband_activated),
 		 game->can_disband_stack);
+  setup_menuitem(stack_info_menuitem,
+		 sigc::mem_fun(*this, &GameWindow::on_stack_info_activated),
+		 game->can_deselect_selected_stack);
   setup_menuitem(signpost_menuitem,
 		 sigc::mem_fun(*this, &GameWindow::on_signpost_activated),
 		 game->can_change_signpost);
@@ -685,13 +711,8 @@ void GameWindow::setup_signals()
 
 }
 
-void GameWindow::update_diplomacy_button (bool proposals_present)
+void GameWindow::change_diplomacy_button_image (bool proposals_present)
 {
-  if (GameScenario::s_diplomacy == false)
-    {
-      diplomacy_button->set_sensitive (false);
-      return;
-    }
   /* switch up the image. */
   if (proposals_present)
     diplomacy_button->property_image() = new Gtk::Image(d_button_images[8]);
@@ -699,7 +720,22 @@ void GameWindow::update_diplomacy_button (bool proposals_present)
     diplomacy_button->property_image() = new Gtk::Image(d_button_images[0]);
 }
 
-void GameWindow::setup_game(std::string file_path)
+void GameWindow::update_diplomacy_button (bool sensitive)
+{
+  if (Playerlist::getActiveplayer()->getType() != Player::HUMAN)
+    {
+      diplomacy_button->set_sensitive (false);
+      return;
+    }
+  if (GameScenario::s_diplomacy == false)
+    {
+      diplomacy_button->set_sensitive (false);
+      return;
+    }
+  diplomacy_button->set_sensitive(sensitive);
+}
+
+bool GameWindow::setup_game(std::string file_path)
 {
   stop_game();
 
@@ -707,15 +743,19 @@ void GameWindow::setup_game(std::string file_path)
   GameScenario* game_scenario = new GameScenario(file_path, broken);
 
   if (broken)
-    // FIXME: we should not die here, but simply return to the splash screen
-    show_fatal_error(_("Map was broken when re-reading. Exiting..."));
-
+  {
+    on_message_requested("Corrupted saved game file.");
+    game_ended.emit();
+    return false;
+  }
+  
   Sound::getInstance()->haltMusic(false);
   Sound::getInstance()->enableBackground();
 
   game.reset(new Game(game_scenario));
 
   show_shield_turn();
+  return true;
 }
 
 bool GameWindow::on_delete_event(GdkEventAny *e)
@@ -881,13 +921,21 @@ void GameWindow::on_save_game_as_activated()
 
 void GameWindow::on_quit_activated()
 {
-  // FIXME: ask
-  bool end = true;
+  std::auto_ptr<Gtk::Dialog> dialog;
+  Glib::RefPtr<Gnome::Glade::Xml> xml
+    = Gnome::Glade::Xml::create(get_glade_path() + "/game-quit-dialog.glade");
+  Gtk::Dialog *d;
+  xml->get_widget("dialog", d);
+  dialog.reset(d);
+  dialog->set_transient_for(*window.get());
 
-  if (end) {
-    stop_game();
-    game_ended.emit();
-  }
+  int response = dialog->run();
+
+  if (response == 0) //end the game
+    {
+      stop_game();
+      game_ended.emit();
+    }
 }
 
 void GameWindow::on_quests_activated()
@@ -952,6 +1000,18 @@ void GameWindow::on_signpost_activated()
   return;
 }
 
+  
+void GameWindow::on_stack_info_activated()
+{
+  StackInfoDialog d(currently_selected_stack);
+  d.set_parent_window(*window.get());
+  d.run();
+  //on_stack_info_changed(currently_selected_stack);
+  //FIXME, armies don't stay selected in the right way.  to reproduce:
+  //go in with all three selected.  deselect middle one in stackinfodialog,
+  //then return.
+}
+
 void GameWindow::on_disband_activated()
 {
   std::auto_ptr<Gtk::Dialog> dialog;
@@ -1135,6 +1195,27 @@ void GameWindow::on_triumphs_activated()
   d.run();
 }
 
+void GameWindow::on_help_about_activated()
+{
+  std::auto_ptr<Gtk::AboutDialog> dialog;
+
+  Glib::RefPtr<Gnome::Glade::Xml> xml
+    = Gnome::Glade::Xml::create(get_glade_path() + "/about-dialog.glade");
+
+  Gtk::AboutDialog *d;
+  xml->get_widget("dialog", d);
+  dialog.reset(d);
+  dialog->set_transient_for(*window.get());
+
+  dialog->set_version(PACKAGE_VERSION);
+  SDL_Surface *logo = File::getMiscPicture("castle_icon.png");
+  dialog->set_logo(to_pixbuf(logo));
+  dialog->show_all();
+  dialog->run();
+
+  return;
+}
+
 void GameWindow::on_diplomacy_report_activated()
 {
   DiplomacyReportDialog d(Playerlist::getActiveplayer());
@@ -1176,23 +1257,11 @@ void GameWindow::on_game_over(Player *winner)
   Gtk::Image *image;
   xml->get_widget("image", image);
 
-  SDL_Surface *win_unmasked = File::getMiscPicture("win.jpg", false);
-  // mask pics need a special format
-  SDL_Surface* tmp = File::getMiscPicture("win_mask.png");
-  SDL_Surface *mask = SDL_CreateRGBSurface(
-					   SDL_SWSURFACE, tmp->w, tmp->h, 32, 0xFF000000, 0xFF0000, 0xFF00, 0xFF);
-  SDL_SetAlpha(tmp, 0, 0);
-  SDL_BlitSurface(tmp, 0, mask, 0);
-  SDL_FreeSurface(tmp);
-
-  SDL_Surface* win = GraphicsCache::getInstance()->applyMask(
-							     win_unmasked, mask, Playerlist::getActiveplayer());
+  SDL_Surface *win = File::getMiscPicture("win.png", false);
 
   image->property_pixbuf() = to_pixbuf(win);
 
   SDL_FreeSurface(win);
-  SDL_FreeSurface(win_unmasked);
-  SDL_FreeSurface(mask);
 
   Gtk::Label *label;
   xml->get_widget("label", label);
@@ -1374,11 +1443,12 @@ void GameWindow::on_stack_info_changed(Stack *s)
     show_stats();
   else
     {
-      if (s->getPlayer()->getType() == Player::HUMAN)
+      if (s->getOwner()->getType() == Player::HUMAN)
 	show_stack(s);
       else
 	show_stats();
     }
+  return;
 }
 
 void GameWindow::show_stats()
@@ -1427,16 +1497,16 @@ void GameWindow::show_stack(Stack *s)
       Gtk::ToggleButton *toggle = new Gtk::ToggleButton;
       toggle->add(*toggle_box);
       toggle->set_active(army->isGrouped());
-      toggle->signal_toggled().connect(
-				       sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_toggled),
-						  toggle, army));
+      toggle->signal_toggled().connect
+	(sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_toggled),
+		    toggle, army));
       toggle->add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
-      toggle->signal_button_press_event().connect(
-						  sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_button_event),
-							     toggle, army), false);
-      toggle->signal_button_release_event().connect(
-						    sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_button_event),
-							       toggle, army), false);
+      toggle->signal_button_press_event().connect
+	(sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_button_event),
+		    toggle, army), false);
+      toggle->signal_button_release_event().connect
+	(sigc::bind(sigc::mem_fun(*this, &GameWindow::on_army_button_event),
+		    toggle, army), false);
       // add it
       stack_info_box->pack_start(*toggle, Gtk::PACK_SHRINK);
       army_buttons.push_back(toggle);
@@ -1444,7 +1514,7 @@ void GameWindow::show_stack(Stack *s)
     }
 
   fill_in_group_info(s);
-  ensure_one_army_button_active();
+  ensure_one_army_button_active(); 
   stack_info_container->show_all();
 }
 
@@ -1520,7 +1590,7 @@ void GameWindow::hide_map_tip()
 
 void GameWindow::on_sage_visited (Ruin *ruin, Stack *stack)
 {
-  SageDialog d(stack->getFirstHero()->getPlayer(), 
+  SageDialog d(stack->getFirstHero()->getOwner(), 
 	       static_cast<Hero*>(stack->getFirstHero()), ruin);
   d.set_parent_window(*window.get());
   Reward *reward = d.run();
@@ -1578,7 +1648,7 @@ void GameWindow::on_ruin_searched(Ruin *ruin, Stack *stack, Reward *reward)
   else if (reward->getType() == Reward::MAP)
     {
       Reward_Map *map = dynamic_cast<Reward_Map*>(reward);
-      s += String::ucompose("the %2.", map->getLocation()->getName());
+      s += String::ucompose("the %2.", map->getName());
     }
 
   label->set_text(s);
@@ -1651,9 +1721,9 @@ void GameWindow::on_ruinfight_finished(Fight::Result result)
   dialog->run();
 }
 
-void GameWindow::on_fight_started(Fight &fight, bool intense_combat)
+void GameWindow::on_fight_started(Fight &fight)
 {
-  FightWindow d(fight, intense_combat);
+  FightWindow d(fight);
 
   d.set_parent_window(*window.get());
   d.run(&d_quick_fights);
@@ -1707,8 +1777,11 @@ void GameWindow::on_surrender_answered (bool accepted)
     on_message_requested
       (_("You graciously and benevolently accept their offer."));
   else
-    on_message_requested
-      (_("Off with their heads!  I want it ALL!"));
+    {
+      SurrenderRefusedDialog d;
+      d.set_parent_window(*window.get());
+      d.run();
+    }
 }
 
 bool GameWindow::on_stack_considers_treachery (Player *me, Stack *stack, 
@@ -1837,6 +1910,10 @@ hero_has_quest_here (Stack *s, City *c, bool *pillage, bool *sack, bool *raze, b
   for (std::vector<Quest*>::iterator i = questlist.begin();
        i != questlist.end(); ++i)
     {
+      if ((*i) == NULL)
+	continue;
+      if ((*i)->isPendingDeletion() == true)
+	continue;
       switch ((*i)->getType())
 	{
 	case Quest::CITYSACK:
@@ -1970,13 +2047,13 @@ CityDefeatedAction GameWindow::on_city_defeated(City *city, int gold)
 	}
     }
 
-  if (city->getNoOfBasicProd() <= 0) {
+  if (city->getNoOfProductionBases() <= 0) {
     Gtk::Button *b;
     xml->get_widget("pillage_button", b);
     b->hide();
   }
 
-  if (city->getNoOfBasicProd() <= 1) {
+  if (city->getNoOfProductionBases() <= 1) {
     Gtk::Button *b;
     xml->get_widget("sack_button", b);
     b->hide();
@@ -2024,7 +2101,7 @@ void GameWindow::on_city_pillaged(City *city, int gold, int pillaged_army_type)
 {
   GraphicsCache *gc = GraphicsCache::getInstance();
   std::auto_ptr<Gtk::Dialog> dialog;
-  Player *player = city->getPlayer();
+  Player *player = city->getOwner();
   unsigned int as = player->getArmyset();
 
   Glib::RefPtr<Gnome::Glade::Xml> xml
@@ -2075,7 +2152,7 @@ void GameWindow::on_city_pillaged(City *city, int gold, int pillaged_army_type)
 void GameWindow::on_city_sacked(City *city, int gold, std::list<Uint32> sacked_types)
 {
   GraphicsCache *gc = GraphicsCache::getInstance();
-  Player *player = city->getPlayer();
+  Player *player = city->getOwner();
   unsigned int as = player->getArmyset();
   std::auto_ptr<Gtk::Dialog> dialog;
 
@@ -2252,28 +2329,32 @@ void GameWindow::on_next_player_turn(Player *player, unsigned int turn_number)
   show_shield_turn();
   if (player->getType() != Player::HUMAN)
     return;
-  if (Configuration::s_showNextPlayer == false)
-    return;
-  Glib::RefPtr<Gnome::Glade::Xml> xml
-    = Gnome::Glade::Xml::create(get_glade_path() + "/next-player-turn-dialog.glade");
+  if (Configuration::s_showNextPlayer == true)
+    {
+      Glib::RefPtr<Gnome::Glade::Xml> xml
+	= Gnome::Glade::Xml::create(get_glade_path() + 
+				    "/next-player-turn-dialog.glade");
 
-  Gtk::Dialog *d;
-  xml->get_widget("dialog", d);
-  dialog.reset(d);
-  dialog->set_transient_for(*window.get());
+      Gtk::Dialog *d;
+      xml->get_widget("dialog", d);
+      dialog.reset(d);
+      dialog->set_transient_for(*window.get());
 
-  Gtk::Image *image;
-  xml->get_widget("image", image);
-  image->property_file() = File::getMiscFile("various/ship.png");
+      Gtk::Image *image;
+      xml->get_widget("image", image);
+      image->property_file() = File::getMiscFile("various/ship.png");
 
-  Gtk::Label *label;
-  xml->get_widget("label", label);
-  Glib::ustring s = String::ucompose(_("%1\nTurn %2"), player->getName(), 
-				     turn_number);
-  label->set_text(s);
+      Gtk::Label *label;
+      xml->get_widget("label", label);
+      Glib::ustring s = String::ucompose(_("%1\nTurn %2"), player->getName(), 
+					 turn_number);
+      label->set_text(s);
 
-  dialog->show_all();
-  dialog->run();
+      dialog->show_all();
+      dialog->run();
+      show();
+    }
+	
 }
 
 void GameWindow::on_medal_awarded_to_army(Army *army)
