@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -153,7 +157,7 @@ void HeroDialog::on_drop_clicked()
     if (i)
     {
 	Item *item = (*i)[item_columns.item];
-	hero->getPlayer()->heroDropItem (hero, item, pos);
+	hero->getOwner()->heroDropItem (hero, item, pos);
 	(*i)[item_columns.status] = _("On the ground");
 	on_selection_changed();
 	fill_in_info_labels();
@@ -168,7 +172,7 @@ void HeroDialog::on_pickup_clicked()
 	Item *item = (*i)[item_columns.item];
         if (item->getPlanted() == true)
           item->setPlanted(false);
-	hero->getPlayer()->heroPickupItem (hero, item, pos);
+	hero->getOwner()->heroPickupItem (hero, item, pos);
 	(*i)[item_columns.status] = _("In backpack");
 	on_selection_changed();
 	fill_in_info_labels();
@@ -195,7 +199,6 @@ void HeroDialog::fill_in_info_labels()
     Uint32 bonus = 0;
     Glib::ustring s;
     // fill in first column
-    // FIXME: put in real numbers
     std::list<Item*> backpack = hero->getBackpack();
     for (std::list<Item*>::iterator i = backpack.begin(), end = backpack.end();
 	i != end; ++i)
