@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -105,15 +108,18 @@ HistoryReportDialog::HistoryReportDialog(Player *p, HistoryReportType type)
 
   generatePastCityCounts();
   city_chart = new LineChart(past_citycounts, d_colours, 
-			     Citylist::getInstance()->size());
+			     Citylist::getInstance()->size(),
+			     _("Cities"), _("Turns"));
   city_alignment->add(*manage(city_chart));
 
   generatePastGoldCounts();
-  gold_chart = new LineChart(past_goldcounts, d_colours, 0);
+  gold_chart = new LineChart(past_goldcounts, d_colours, 0, 
+			     _("Gold Pieces"), _("Turns"));
   gold_alignment->add(*manage(gold_chart));
 
   generatePastWinningCounts();
-  rank_chart = new LineChart(past_rankcounts, d_colours, 100);
+  rank_chart = new LineChart(past_rankcounts, d_colours, 100,
+			     _("Score"), _("Turns"));
   winner_alignment->add(*manage(rank_chart));
 
   fill_in_turn_info((Uint32)turn_scale->get_value());
@@ -177,7 +183,7 @@ void HistoryReportDialog::generatePastEventlists()
 		case History::DIPLOMATIC_TREACHERY:
 		case History::DIPLOMATIC_WAR:
 		case History::DIPLOMATIC_PEACE:
-		  (*hit[id])->setPlayer(*pit);
+		  (*hit[id])->setOwner(*pit);
 		  elist->push_back(*hit[id]);
 		  break;
 		case History::START_TURN:
@@ -228,13 +234,13 @@ void HistoryReportDialog::generatePastCitylists()
     }
 
   //start off with an initial city list where all cities are neutral owned
-  ObjectList<City> *clist = new ObjectList<City>();
+  LocationList<City> *clist = new LocationList<City>();
   Citylist *cl = Citylist::getInstance();
   for (Citylist::iterator it = cl->begin(); it != cl->end(); ++it)
     clist->push_back(*it);
-  for (ObjectList<City>::iterator it = clist->begin(); it != clist->end(); ++it)
+  for (LocationList<City>::iterator it = clist->begin(); it != clist->end(); ++it)
     {
-      (*it).setPlayer(Playerlist::getInstance()->getNeutral());
+      (*it).setOwner(Playerlist::getInstance()->getNeutral());
       (*it).setBurnt(false);
     }
 
@@ -263,11 +269,11 @@ void HistoryReportDialog::generatePastCitylists()
 		  Uint32 city_id;
 		  city_id = dynamic_cast<History_CityWon*>(*hit[id])->getCityId();
 		  //find city with this city id in clist
-		  ObjectList<City>::iterator cit = clist->begin();
+		  LocationList<City>::iterator cit = clist->begin();
 		  for (; cit != clist->end(); ++cit)
 		    if ((*cit).getId() == city_id)
 		      {
-			(*cit).setPlayer(*pit);
+			(*cit).setOwner(*pit);
 			break;
 		      }
 		}
@@ -276,12 +282,12 @@ void HistoryReportDialog::generatePastCitylists()
 		  Uint32 city_id;
 		  city_id = dynamic_cast<History_CityRazed*>(*hit[id])->getCityId();
 		  //find city with this city id in clist
-		  ObjectList<City>::iterator cit = clist->begin();
+		  LocationList<City>::iterator cit = clist->begin();
 		  for (; cit != clist->end(); ++cit)
 		    if ((*cit).getId() == city_id)
 		      {
 			//change the owner to neutral 
-			(*cit).setPlayer(Playerlist::getInstance()->getNeutral());
+			(*cit).setOwner(Playerlist::getInstance()->getNeutral());
 			(*cit).setBurnt(true);
 			break;
 		      }
@@ -296,7 +302,7 @@ void HistoryReportDialog::generatePastCitylists()
 	}
       //and add it to the list
       past_citylists.push_back(clist);
-      ObjectList<City> *new_clist = new ObjectList<City>(*clist);
+      LocationList<City> *new_clist = new LocationList<City>(*clist);
       clist = new_clist;
       if (last_turn == true)
 	break;
@@ -439,7 +445,7 @@ void HistoryReportDialog::on_switch_page(GtkNotebookPage *page, guint number)
 void HistoryReportDialog::addHistoryEvent(History *event)
 {
   GraphicsCache *gc = GraphicsCache::getInstance();
-  Player *p = event->getPlayer();
+  Player *p = event->getOwner();
 
   Glib::ustring s = "";
   Gtk::TreeIter i = events_list->append();
@@ -621,10 +627,10 @@ void HistoryReportDialog::generatePastCityCounts()
       for (unsigned int i = 0; i < past_citylists.size(); i++)
 	{
 	  Uint32 total_cities = 0;
-	  ObjectList<City>::iterator it = past_citylists[i]->begin();
+	  LocationList<City>::iterator it = past_citylists[i]->begin();
 	  for (; it != past_citylists[i]->end(); it++)
 	    {
-	      if ((*it).getPlayer() == *pit)
+	      if ((*it).getOwner() == *pit)
 		total_cities++;
 	    }
 	  line.push_back(total_cities);
