@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef HISTORY_REPORT_DIALOG_H
 #define HISTORY_REPORT_DIALOG_H
@@ -30,7 +33,7 @@
 
 #include "line-chart.h"
 
-#include "../ObjectList.h"
+#include "../LocationList.h"
 #include "../historymap.h"
 #include "../history.h"
 #include "../player.h"
@@ -69,7 +72,7 @@ class HistoryReportDialog: public sigc::trackable
     Gtk::Alignment *gold_alignment;
     Gtk::Alignment *winner_alignment;
 
-    std::vector<ObjectList<City>* > past_citylists;
+    std::vector<LocationList<City>* > past_citylists;
     LineChart *city_chart;
     std::vector<std::list<History *> > past_eventlists;
     std::list<std::list<Uint32> > past_citycounts;
