@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -42,11 +45,9 @@ ItemBonusDialog::ItemBonusDialog()
     items_treeview->append_column("", items_columns.name);
     items_treeview->append_column("Bonus", items_columns.bonus);
 
-    Itemlist::createInstance();
     Itemlist::iterator iter = Itemlist::getInstance()->begin();
     for (;iter != Itemlist::getInstance()->end(); iter++)
       addItem((*iter).second);
-    Itemlist::deleteInstance();
 }
 
 void ItemBonusDialog::set_parent_window(Gtk::Window &parent)
