@@ -1,3 +1,6 @@
+//  Copyright (C) 2007, Ole Laursen
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +13,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -20,6 +24,7 @@
 #include "preferences-dialog.h"
 
 #include "glade-helpers.h"
+#include "image-helpers.h"
 #include "../ucompose.hpp"
 #include "../defs.h"
 #include "../Configuration.h"
@@ -29,6 +34,7 @@
 #include "../citylist.h"
 #include "../ruinlist.h"
 #include "../ai_fast.h"
+#include "../GraphicsCache.h"
 
 
 PreferencesDialog::PreferencesDialog()
@@ -47,12 +53,14 @@ PreferencesDialog::PreferencesDialog()
     xml->get_widget("music_volume_hbox", music_volume_hbox);
     xml->get_widget("players_vbox", players_vbox);
     
+    GraphicsCache *gc = GraphicsCache::getInstance();
     Playerlist* pl = Playerlist::getInstance();
     for (Playerlist::iterator i = pl->begin(); i != pl->end(); ++i)
       {
 	if (*i == Playerlist::getInstance()->getNeutral())
 	  continue;
 	Gtk::HBox *player_hbox = new Gtk::HBox();
+	Gtk::Image *image = new Gtk::Image(to_pixbuf(gc->getShieldPic(2, *i)));
 	Gtk::ComboBoxText *type = new Gtk::ComboBoxText();
 	type->append_text(_("Human"));
 	type->append_text(_("Computer"));
@@ -62,6 +70,8 @@ PreferencesDialog::PreferencesDialog()
 	  type->set_active(1);
 	if ((*i)->isDead())
 	  type->set_sensitive(false);
+	player_hbox->pack_start(*manage(image), Gtk::PACK_SHRINK, 
+				Gtk::PACK_SHRINK);
 	player_hbox->pack_start(*manage(type), Gtk::PACK_SHRINK, 10);
 	player_types.push_back(type);
 	Gtk::Label *player_name = new Gtk::Label((*i)->getName());
