@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #ifndef REPORT_DIALOG_H
 #define REPORT_DIALOG_H
@@ -28,7 +31,7 @@
 #include <gtkmm/treeview.h>
 #include <gtkmm/alignment.h>
 
-#include "../ObjectList.h"
+#include "../LocationList.h"
 #include "../vectormap.h"
 #include "../citymap.h"
 #include "../armymap.h"
