@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -45,9 +48,9 @@ RuinReportDialog::RuinReportDialog(Vector<int> pos)
 
   xml->get_widget("map_image", map_image);
 
-  Location *l = NULL;
+  NamedLocation *l = NULL;
   Ruin *ruin = Ruinlist::getInstance()->getNearestRuin(pos);
-  Temple *temple = Templelist::getInstance()->getNearestTemple(pos);
+  Temple *temple = Templelist::getInstance()->getNearestObject(pos);
   if (temple && !ruin)
     l = temple;
   else if (ruin && !temple)
@@ -115,7 +118,7 @@ bool RuinReportDialog::on_map_mouse_button_event(GdkEventButton *e)
 
 void RuinReportDialog::fill_in_ruin_info()
 {
-  Location *l = ruinmap->getLocation();
+  NamedLocation *l = ruinmap->getNamedLocation();
   name_label->set_text(l->getName());
   Ruin *ruin = Ruinlist::getInstance()->getObjectAt(l->getPos());
   Temple *temple = Templelist::getInstance()->getObjectAt(l->getPos());
