@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <config.h>
 
@@ -147,8 +150,26 @@ void SageDialog::addReward(Reward *reward)
     case Reward::ALLIES:
       break;
     case Reward::MAP:
-      (*i)[rewards_columns.name] = 
-	static_cast<Reward_Map*>(reward)->getLocation()->getName();
+	{
+	  std::string name;
+	  Reward_Map *m = static_cast<Reward_Map*>(reward);
+	  name  = m->getName();
+	  if (name == "")
+	    {
+	      switch (rand() % 6)
+		{
+		case 0: name = "parchment map"; break;
+		case 1: name = "vellum map"; break;
+		case 2: name = "paper map"; break;
+		case 3: name = "torn paper map"; break;
+		case 4: name = "dusty map"; break;
+		case 5: name = "blood-stained map"; break;
+		}
+	    }
+	    
+	  (*i)[rewards_columns.name] = name;
+	}
+
       break;
     case Reward::RUIN:
 	{
