@@ -1,3 +1,7 @@
+// Copyright (C) 2006, 2007 Ulf Lorenz
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +14,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <assert.h>
 #include <math.h>
@@ -30,6 +35,7 @@
 #include "sdl-draw.h"
 #include "GraphicsCache.h"
 #include "FogMap.h"
+#include "GameScenario.h"
 
 OverviewMap::OverviewMap()
 {
@@ -84,106 +90,124 @@ bool OverviewMap::isShadowed(Uint32 type, int i, int j)
   return false;
 }
 
-void OverviewMap::draw_tile_pixel(Maptile *t, int i, int j)
+void OverviewMap::draw_tile_pixel(SDL_Surface *surface, Tile::Pattern pattern,
+				  SDL_Color first_color, SDL_Color second_color,
+				  SDL_Color third_color,
+				  int i, int j, bool shadowed)
 {
-  SDL_Color c = t->getColor();
-  Tile::Pattern p = t->getPattern();
+  SDL_Color c = first_color;
   
-  Uint32 first = SDL_MapRGB(static_surface->format, c.r, c.g, c.b);
-  switch (p)
+  Uint32 first = SDL_MapRGB(surface->format, c.r, c.g, c.b);
+  switch (pattern)
     {
       case Tile::SOLID:
-        draw_pixel(static_surface, i, j, first);
+        draw_pixel(surface, i, j, first);
         break;
       case Tile::STIPPLED:
         {
-          SDL_Color s = t->getSecondColor();
-          Uint32 second = SDL_MapRGB(static_surface->format, s.r, s.g, s.b);
+          SDL_Color s = second_color;
+          Uint32 second = SDL_MapRGB(surface->format, s.r, s.g, s.b);
          
           if ((i+j) % 2 == 0)
-            draw_pixel(static_surface, i, j, first);
+            draw_pixel(surface, i, j, first);
           else
-            draw_pixel(static_surface, i, j, second);
+            draw_pixel(surface, i, j, second);
         }
         break;
       case Tile::RANDOMIZED:
         {
-          SDL_Color s = t->getSecondColor();
-          Uint32 second = SDL_MapRGB(static_surface->format, s.r, s.g, s.b);
-          SDL_Color th = t->getThirdColor();
-          Uint32 third = SDL_MapRGB(static_surface->format, th.r, th.g, th.b);
+          SDL_Color s = second_color;
+          Uint32 second = SDL_MapRGB(surface->format, s.r, s.g, s.b);
+          SDL_Color th = third_color;
+          Uint32 third = SDL_MapRGB(surface->format, th.r, th.g, th.b);
           int num = rand() % 3;
           if (num == 0)
-            draw_pixel(static_surface, i, j, first);
+            draw_pixel(surface, i, j, first);
           else if (num == 1)
-            draw_pixel(static_surface, i, j, second);
+            draw_pixel(surface, i, j, second);
           else
-            draw_pixel(static_surface, i, j, third);
+            draw_pixel(surface, i, j, third);
         }
         break;
       case Tile::SUNKEN:
-        if (isShadowed(t->getType(), i, j) == false)
-          draw_pixel(static_surface, i, j, first);
+        if (shadowed == false)
+          draw_pixel(surface, i, j, first);
         else
           {
-            SDL_Color s = t->getSecondColor();
-            Uint32 shadow_color = SDL_MapRGB(static_surface->format, s.r, s.g, 
-                                             s.b);
-            draw_pixel(static_surface, i, j, shadow_color);
+            SDL_Color s = second_color;
+            Uint32 shadow_color = SDL_MapRGB(surface->format, s.r, s.g, s.b);
+            draw_pixel(surface, i, j, shadow_color);
           }
         break;
       case Tile::TABLECLOTH:
           {
-            SDL_Color s = t->getSecondColor();
-            Uint32 second = SDL_MapRGB(static_surface->format, s.r, s.g, s.b);
-            SDL_Color th = t->getThirdColor();
-            Uint32 third = SDL_MapRGB(static_surface->format, th.r, th.g, th.b);
+            SDL_Color s = second_color;
+            Uint32 second = SDL_MapRGB(surface->format, s.r, s.g, s.b);
+            SDL_Color th = third_color;
+            Uint32 third = SDL_MapRGB(surface->format, th.r, th.g, th.b);
             if (i % 4 == 0 && j % 4 == 0)
-              draw_pixel(static_surface, i, j, first);
+              draw_pixel(surface, i, j, first);
             else if (i % 4 == 0 && j % 4 == 1)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 0 && j % 4 == 2)
-              draw_pixel(static_surface, i, j, first);
+              draw_pixel(surface, i, j, first);
             else if (i % 4 == 0 && j % 4 == 3)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 1 && j % 4 == 0)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 1 && j % 4 == 1)
-              draw_pixel(static_surface, i, j, third);
+              draw_pixel(surface, i, j, third);
             else if (i % 4 == 1 && j % 4 == 2)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 1 && j % 4 == 3)
-              draw_pixel(static_surface, i, j, third);
+              draw_pixel(surface, i, j, third);
             else if (i % 4 == 2 && j % 4 == 0)
-              draw_pixel(static_surface, i, j, first);
+              draw_pixel(surface, i, j, first);
             else if (i % 4 == 2 && j % 4 == 1)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 2 && j % 4 == 2)
-              draw_pixel(static_surface, i, j, first);
+              draw_pixel(surface, i, j, first);
             else if (i % 4 == 2 && j % 4 == 3)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 3 && j % 4 == 0)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 3 && j % 4 == 1)
-              draw_pixel(static_surface, i, j, third);
+              draw_pixel(surface, i, j, third);
             else if (i % 4 == 3 && j % 4 == 2)
-              draw_pixel(static_surface, i, j, second);
+              draw_pixel(surface, i, j, second);
             else if (i % 4 == 3 && j % 4 == 3)
-              draw_pixel(static_surface, i, j, third);
+              draw_pixel(surface, i, j, third);
           }
         break;
     }
 }
+
+void OverviewMap::draw_tile_pixel(Maptile *t, int i, int j)
+{
+  bool shadowed = isShadowed(t->getType(), i, j);
+  draw_tile_pixel(static_surface, t->getPattern(), 
+		  t->getColor(), t->getSecondColor(), t->getThirdColor(),
+		  i, j, shadowed);
+}
+
+int OverviewMap::calculateResizeFactor()
+{
+  if (GameMap::getWidth() <= MAP_SIZE_TINY_WIDTH && 
+      GameMap::getHeight() <= MAP_SIZE_TINY_HEIGHT)
+    return 4;
+  else if (GameMap::getWidth() <= MAP_SIZE_SMALL_WIDTH && 
+	   GameMap::getHeight() <= MAP_SIZE_SMALL_HEIGHT)
+    return 3;
+  else
+    return 2;
+}
+
 void OverviewMap::resize()
 {
-  /* fixme: these values must align with gui/game-preferences.cpp */
-  if (GameMap::getWidth() <= 50 && GameMap::getWidth() <= 75)
-    resize(GameMap::get_dim() * 4);
-  else if (GameMap::getWidth() <= 70 && GameMap::getWidth() <= 105)
-    resize(GameMap::get_dim() * 3);
-  else
-    resize(GameMap::get_dim() * 2);
+  int factor = calculateResizeFactor();
+  resize(GameMap::get_dim() * factor);
 }
+
 void OverviewMap::resize(Vector<int> max_dimensions)
 {
     if (surface)
@@ -243,15 +267,6 @@ void OverviewMap::redraw_tiles(Rectangle tiles)
     draw();
 }
 
-bool OverviewMap::isFogged(Vector <int> pos)
-{
-  //is this tile visible, or not?
-  FogMap *fogmap = Playerlist::getActiveplayer()->getFogMap();
-  if (fogmap->getFogTile(pos) == FogMap::CLOSED)
-    return true;
-  return false;
-}
-
 void OverviewMap::draw_terrain_pixels(Rectangle r)
 {
     GameMap *gm = GameMap::getInstance();
@@ -334,7 +349,7 @@ void OverviewMap::draw()
           Vector <int> pos;
           pos.x = i;
           pos.y = j;
-          if (isFogged(pos) == true)
+          if (FogMap::isFogged(pos) == true)
             {
               pos = mapToSurface(pos);
               draw_filled_rect(surface, pos.x, pos.y,
@@ -398,7 +413,7 @@ void OverviewMap::draw_cities (bool all_razed)
       if (it->isBurnt() == true || all_razed == true)
         tmp = gc->getSmallRuinedCityPic();
       else
-        tmp = gc->getShieldPic(0, it->getPlayer());
+        tmp = gc->getShieldPic(0, it->getOwner());
   
       Vector<int> pos = it->getPos();
       pos = mapToSurface(pos);
