@@ -1,3 +1,11 @@
+// Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2000, Anluan O'Brien
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2004, 2005 Andrea Paternesi
+// Copyright (C) 2006, 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +18,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sigc++/functors/mem_fun.h>
 #include <assert.h>
@@ -27,6 +36,10 @@
 #include "GameMap.h"
 #include "vector.h"
 #include "xmlhelper.h"
+#include "FogMap.h"
+#include "player.h"
+#include "portlist.h"
+#include "port.h"
 
 using namespace std;
 
@@ -34,20 +47,20 @@ using namespace std;
 #define debug(x)
 
 Stack::Stack(Player* player, Vector<int> pos)
-    : Object(pos), d_player(player), d_defending(false), d_parked(false),
-    d_deleting(false), d_moves_exhausted_at_point(0)
+    : UniquelyIdentified(), Movable(pos), Ownable(player), d_defending(false), 
+    d_parked(false), d_deleting(false)
 {
     d_path = new Path();
 }
 
 Stack::Stack(Stack& s)
-    : Object(s), d_player(s.d_player), d_defending(s.d_defending),
-     d_parked(s.d_parked), d_deleting(false),
-     d_moves_exhausted_at_point(s.d_moves_exhausted_at_point)
+    : UniquelyIdentified(s), Movable(s), Ownable(s), 
+    d_defending(s.d_defending), d_parked(s.d_parked), d_deleting(false)
 {
     clear();
     d_path = new Path();
     //deep copy the other stack's armies
+    
     for (iterator sit = s.begin(); sit != s.end(); sit++)
     {
         Army* a;
@@ -59,27 +72,19 @@ Stack::Stack(Stack& s)
 	  }
 	else
 	  {
-	    a = new Army((**sit), (*sit)->getPlayer());
+	    a = new Army((**sit), (*sit)->getOwner());
 	    push_back(a);
 	  }
     }
 }
 
 Stack::Stack(XML_Helper* helper)
-: Object(helper), d_deleting(false)
+  : UniquelyIdentified(helper), Movable(helper), Ownable(helper), 
+    d_deleting(false)
 {
   helper->getData(d_defending, "defending");
   helper->getData(d_parked, "parked");
 
-  int i;
-  helper->getData(i, "player");
-  if (i == -1)
-    d_player = 0;
-  else
-    d_player = Playerlist::getInstance()->getPlayer(i);
-
-  helper->getData(d_moves_exhausted_at_point, "moves_exhausted_at_point");
-
   helper->registerTag("path", sigc::mem_fun((*this), &Stack::load));
   helper->registerTag("army", sigc::mem_fun((*this), &Stack::load));
   helper->registerTag("hero", sigc::mem_fun((*this), &Stack::load));
@@ -97,31 +102,85 @@ Stack::~Stack()
 void Stack::setPlayer(Player* p)
 {
   // we need to change the armies' loyalties as well!!
-  d_player = p;
+  setOwner(p);
   for (iterator it = begin(); it != end(); it++)
-    (*it)->setPlayer(p);
+    (*it)->setOwner(p);
 }
 
-bool Stack::moveOneStep()
+void Stack::moveOneStep()
 {
   debug("move_one_step()");
 
-  d_pos = **d_path->begin();
+  Vector<int> dest = *getPath()->front();
+  moveToDest(dest);
+
+  //now remove first point of the path
+  d_path->eraseFirstPoint();
+}
+
+void Stack::moveToDest(Vector<int> dest)
+{
+  Vector<int> pos = getPos();
+    
+  Uint32 maptype = GameMap::getInstance()->getTile(dest.x,dest.y)->getMaptileType();
+  City* to_city = Citylist::getInstance()->getObjectAt(dest.x, dest.y);
+  City* on_city = Citylist::getInstance()->getObjectAt(pos.x, pos.y);
+  Port* on_port = Portlist::getInstance()->getObjectAt(pos.x, pos.y);
+  bool on_water = (GameMap::getInstance()->getTile(pos.x,pos.y)->getMaptileType() == Tile::WATER);
+  bool to_water = (GameMap::getInstance()->getTile(dest.x,dest.y)->getMaptileType() == Tile::WATER);
+  bool ship_load_unload = false;
+  //here we mark the armies as being on or off a boat
+  if (!isFlying())
+    {
+      if ((on_water && to_city) || 
+	  (on_water && on_port && !to_water) ||
+	  ((on_city || on_port) && to_water))
+	{
+	  ship_load_unload = true;
+	  for (Stack::iterator it = begin(); it != end(); it++)
+	    {
+	      if (to_water && 
+		  ((*it)->getStat(Army::MOVE_BONUS) & Tile::WATER) == 0)
+		(*it)->setInShip(true);
+	      else
+		(*it)->setInShip(false);
+	    }
+	}
+    }
+  else
+    {
+      for (Stack::iterator it = begin(); it != end(); it++)
+	(*it)->setInShip(false);
+    }
+
+  //how many moves does the stack need to travel to dest?
+  int needed_moves = calculateTileMovementCost(dest);
+
+  for (Stack::iterator it = begin(); it != end(); it++)
+    {
+      if (ship_load_unload)
+	(*it)->decrementMoves((*it)->getMoves());
+      else 
+	{
+	  //maybe the army has a natural movement ability
+	  if ((*it)->getStat(Army::MOVE_BONUS) == maptype && needed_moves > 1)
+	    (*it)->decrementMoves(2);
+	  else
+	    (*it)->decrementMoves(needed_moves);
+	}
+    }
+
+  //update position and status
+  setPos(dest);
 
   setFortified(false);
   setDefending(false);
   setParked(false);
 
-  //now remove first point of the path
-  d_path->flErase(d_path->begin());
-
-  //and decrement the point at which we exhaust our path
-  if (getMovesExhaustedAtPoint())
-    setMovesExhaustedAtPoint(getMovesExhaustedAtPoint()-1);
-  return true;
+  //update fogmap
+  getOwner()->getFogMap()->alterFogRadius(dest, getMaxSight(), FogMap::OPEN);
 }
 
-
 bool Stack::isGrouped()
 {
   if (empty())
@@ -161,7 +220,7 @@ Uint32 Stack::getGroupMoves() const
 int Stack::getMinTileMoves() const
 {
   GameMap *map = GameMap::getInstance();
-  Rectangle bounds = GameMap::get_boundary();
+  Rectangle bounds = map->get_boundary();
 
   std::vector<Vector<int> > tiles;
   tiles.push_back(Vector<int>(getPos().x + 1, getPos().y - 1));
@@ -208,34 +267,22 @@ void Stack::decrementMoves(Uint32 moves)
 Army* Stack::getStrongestArmy() const
 {
   assert(!empty());
-  Army *strongest = 0;
-  Uint32 highest_strength = 0;
-
-  for (const_iterator it = begin(); it != end(); ++it)
-    {
-      // if hero
-      if ((*it)->isHero())
-	{
-	  return *it;
-	}
-      else if ((*it)->getStat(Army::STRENGTH) > highest_strength)
-	{
-	  highest_strength = (*it)->getStat(Army::STRENGTH);
-	  strongest = *it;
-	}
-    }
-  return strongest;
+  return getStrongestArmy(false);
 }
 
 Army* Stack::getStrongestHero() const
+{
+  return getStrongestArmy(true);
+}
+
+Army* Stack::getStrongestArmy(bool hero) const
 {
   Army *strongest = 0;
   Uint32 highest_strength = 0;
 
   for (const_iterator it = begin(); it != end(); ++it)
     {
-      // if hero
-      if ((*it)->isHero())
+      if ((*it)->isHero() && hero || !hero)
 	{
 	  if ((*it)->getStat(Army::STRENGTH) > highest_strength)
 	    {
@@ -247,6 +294,16 @@ Army* Stack::getStrongestHero() const
   return strongest;
 }
 
+Army *Stack::getArmyById(Uint32 id) const
+{
+  for (Stack::const_iterator i = begin(), e = end(); i != e; ++i)
+    if ((*i)->getId() == id)
+      return *i;
+  
+  return 0;
+}
+
+
 void Stack::group()
 {
   if (empty())
@@ -335,6 +392,7 @@ bool Stack::canMove() const
   int tile_moves = getMinTileMoves();
   int group_moves = getGroupMoves();
 
+  assert (tile_moves != -1);
   return group_moves > 0 && tile_moves >= 0 && group_moves >= tile_moves;
 }
 
@@ -350,11 +408,6 @@ Army* Stack::getFirstUngroupedArmy() const
   return 0;
 }
 
-bool Stack::isFriend(Player* player) const
-{
-  return d_player == player;
-}
-
 Uint32 Stack::getMaxSight() const
 {
   Uint32 max = 0;
@@ -395,7 +448,7 @@ void Stack::nextTurn()
 	std::list<Item*>::const_iterator item;
 	for (item = backpack.begin(); item != backpack.end(); item++)
 	  {
-	    Player *p = d_player;
+	    Player *p = d_owner;
 	    if ((*item)->getBonus(Item::ADD2GOLDPERCITY))
 	      p->addGold(2 * Citylist::getInstance()->countCities(p));
 	    if ((*item)->getBonus(Item::ADD3GOLDPERCITY))
@@ -413,8 +466,8 @@ void Stack::nextTurn()
     {
       (*it)->resetMoves();
       // TODO: should be moved in a more appropriate place => class Player
-      if (d_player)
-	d_player->withdrawGold((*it)->getUpkeep());
+      if (d_owner)
+	d_owner->withdrawGold((*it)->getUpkeep());
       (*it)->heal();
     }
 
@@ -429,17 +482,15 @@ bool Stack::save(XML_Helper* helper) const
 
   retval &= helper->openTag("stack");
   retval &= helper->saveData("id", d_id);
-  if (d_player)
-    retval &= helper->saveData("player", d_player->getId());
+  retval &= helper->saveData("x", getPos().x);
+  retval &= helper->saveData("y", getPos().y);
+  if (d_owner)
+    retval &= helper->saveData("owner", d_owner->getId());
   else
-    retval &= helper->saveData("player", -1);
-  retval &= helper->saveData("x", d_pos.x);
-  retval &= helper->saveData("y", d_pos.y);
+    retval &= helper->saveData("owner", -1);
   retval &= helper->saveData("defending", d_defending);
   retval &= helper->saveData("parked", d_parked);
 
-  retval &= helper->saveData("moves_exhausted_at_point", 
-			     d_moves_exhausted_at_point);
 
   //save path
   retval &= d_path->save(helper);
@@ -465,7 +516,7 @@ bool Stack::load(std::string tag, XML_Helper* helper)
   if (tag == "army")
     {
       Army* a = new Army(helper);
-      a->setPlayer(d_player);
+      a->setOwner(d_owner);
       push_back(a);
 
       return true;
@@ -474,7 +525,7 @@ bool Stack::load(std::string tag, XML_Helper* helper)
   if (tag == "hero")
     {
       Hero* h = new Hero(helper);
-      h->setPlayer(d_player);
+      h->setOwner(d_owner);
       push_back(h);
 
       return true;
@@ -609,8 +660,8 @@ Uint32 getFightOrder(std::list<Uint32> values, Uint32 value)
 
 bool Stack::armyCompareFightOrder (const Army *lhs, const Army *rhs)  
 {
-  std::list<Uint32> lhs_fight_order = lhs->getPlayer()->getFightOrder();
-  std::list<Uint32> rhs_fight_order = rhs->getPlayer()->getFightOrder();
+  std::list<Uint32> lhs_fight_order = lhs->getOwner()->getFightOrder();
+  std::list<Uint32> rhs_fight_order = rhs->getOwner()->getFightOrder();
   Uint32 lhs_rank = getFightOrder (lhs_fight_order, lhs->getType());
   Uint32 rhs_rank = getFightOrder (rhs_fight_order, rhs->getType());
   //if (lhs_rank == rhs_rank)
