@@ -1,3 +1,10 @@
+// Copyright (C) 2000, 2001, 2002, 2003 Michael Bartl
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2004, 2005 Andrea Paternesi
+// Copyright (C) 2004 John Farrell
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007, 2008 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +17,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sigc++/functors/mem_fun.h>
 
@@ -43,11 +51,6 @@ Stack* Stacklist::getObjectAt(int x, int y)
 
 }
 
-Stack* Stacklist::getObjectAt(Vector<int> point)
-{
-    return getObjectAt(point.x, point.y);
-}
-
 Vector<int> Stacklist::getPosition(Uint32 id)
 {
     for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
@@ -104,6 +107,14 @@ void Stacklist::nextTurn()
       {
         (*it)->nextTurn();
       }
+    for (iterator it = begin(); it != end(); it++)
+      for (iterator jit = begin(); jit != end(); jit++)
+	if (*jit != *it)
+	  if ((*jit)->getId() == (*it)->getId())
+	    {
+	      fprintf (stderr, "duplicate army id %d found\n", (*it)->getId());
+	      exit (1);
+	    }
 }
 
 vector<Stack*> Stacklist::defendersInCity(City *city)
@@ -113,12 +124,12 @@ vector<Stack*> Stacklist::defendersInCity(City *city)
     vector<Stack*> stackvector;
     Vector<int> pos = city->getPos();
 
-    for (int i = pos.x; i < pos.x + 2; i++)
+    for (unsigned int i = pos.x; i < pos.x + city->getSize(); i++)
     {
-        for (int j = pos.y; j < pos.y + 2; j++)
+        for (unsigned int j = pos.y; j < pos.y + city->getSize(); j++)
         {
 	    Stack *stack;
-	    stack = city->getPlayer()->getStacklist()->getOwnObjectAt(i, j);
+	    stack = city->getOwner()->getStacklist()->getOwnObjectAt(i, j);
             if (stack)
             {
 	      stackvector.push_back(stack);
@@ -195,7 +206,7 @@ Stack* Stacklist::getNextMovable()
     for (; it != end(); ++it)
     {
 	Stack *s = *it;
-        if (s->getPlayer() == player && !s->getDefending() && 
+        if (s->getOwner() == player && !s->getDefending() && 
 	    !s->getParked() && s->canMove())
 	    return s;
     }
@@ -209,7 +220,7 @@ Stack* Stacklist::getNextMovable()
     for (it = begin(); *it != d_activestack; ++it)
     {
 	Stack *s = *it;
-        if (s->getPlayer() == player && !s->getDefending() &&
+        if (s->getOwner() == player && !s->getDefending() &&
 	    !s->getParked() && s->canMove())
 	    return s;
     }
@@ -221,6 +232,24 @@ Stack* Stacklist::getNextMovable()
 	return d_activestack;
 }
 
+Stack *Stacklist::getStackById(Uint32 id)
+{
+  for (Stacklist::iterator i = begin(), e = end(); i != e; ++i)
+    if ((*i)->getId() == id)
+      return *i;
+    
+  return 0;
+}
+
+Stack *Stacklist::getArmyStackById(Uint32 army)
+{
+  for (Stacklist::iterator i = begin(), e = end(); i != e; ++i)
+    if ((*i)->getArmyById(army))
+      return *i;
+  
+  return 0;
+}
+
 void Stacklist::flClear()
 {
     d_activestack = 0;
