@@ -1,3 +1,8 @@
+// Copyright (C) 2000, 2001, 2003 Michael Bartl
+// Copyright (C) 2001, 2003, 2004, 2005 Ulf Lorenz
+// Copyright (C) 2007, 2008 Ben Asselstine
+// Copyright (C) 2007 Ole Laursen
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +15,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <sigc++/functors/mem_fun.h>
 
@@ -82,53 +88,18 @@ bool Templelist::load(std::string tag, XML_Helper* helper)
     return true;
 }
 
-Temple * Templelist::getNearestTemple(const Vector<int>& pos)
+static bool isFogged(void *t)
 {
-    int diff = -1;
-    iterator diffit;
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        Vector<int> p = (*it).getPos();
-        int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
-
-        if ((diff > delta) || (diff == -1))
-        {
-            diff = delta;
-            diffit = it;
-        }
-    }
-    if (diff == -1) return 0;
-    return &(*diffit);
-}
-Temple* Templelist::getNearestTemple(const Vector<int>& pos, int dist)
-{
-  Temple *t = getNearestTemple(pos);
-  if (t->getPos().x <= pos.x + dist && t->getPos().x >= pos.x - dist &&
-      t->getPos().y <= pos.y + dist && t->getPos().y >= pos.y - dist)
-    return t;
-  return NULL;
+  return ((Temple*)t)->isFogged();
 }
 
 Temple * Templelist::getNearestVisibleTemple(const Vector<int>& pos)
 {
-    int diff = -1;
-    iterator diffit;
-    for (iterator it = begin(); it != end(); ++it)
-    {
-        Vector<int> p = (*it).getPos();
-        int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
-        if (it->isFogged())
-          continue;
-
-        if ((diff > delta) || (diff == -1))
-        {
-            diff = delta;
-            diffit = it;
-        }
-    }
-    if (diff == -1) return 0;
-    return &(*diffit);
+  std::list<bool (*)(void *)> filters;
+  filters.push_back(isFogged);
+  return getNearestObject(pos, &filters);
 }
+
 Temple* Templelist::getNearestVisibleTemple(const Vector<int>& pos, int dist)
 {
   Temple *t = getNearestVisibleTemple(pos);
