@@ -1,3 +1,5 @@
+//  Copyright (C) 2007, 2008 Ben Asselstine
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +12,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include "vectormap.h"
 
@@ -96,14 +99,14 @@ void VectorMap::draw_cities (std::list<City*> citylist, Uint32 type)
 	    }
 	  else
 	    {
-              if ((*it)->getProductionIndex() == -1)
+              if ((*it)->getActiveProductionSlot() == -1)
                 prod = false;
               else
                 prod = true;
 	    }
 	  break;
 	case CLICK_SELECTS:
-          if ((*it)->getProductionIndex() == -1)
+          if ((*it)->getActiveProductionSlot() == -1)
             prod = false;
           else
             prod = true;
@@ -151,7 +154,7 @@ void VectorMap::draw_lines (std::list<City*> srcs, std::list<City*> dests)
 	continue;
       if ((*it)->isFogged() == true)
         continue;
-      City *c = cl->getObjectAt((*it)->getVectoring());
+      City *c = cl->getNearestObjectBefore((*it)->getVectoring(), 2);
       if (c)
         end = c->getPos();
       else
@@ -181,7 +184,7 @@ void VectorMap::after_draw()
   std::list<City*> srcs; //source cities
     
   Vector<int> flag;
-  flag = GameMap::getInstance()->findPlantedStandard(city->getPlayer());
+  flag = GameMap::getInstance()->findPlantedStandard(city->getOwner());
   planted_standard = flag;
 
   //only show cities that can accept more vectoring when
@@ -195,10 +198,10 @@ void VectorMap::after_draw()
   // draw special shield for every city that player owns.
   for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
     {
-      if ((*it).getPlayer() == Playerlist::getActiveplayer())
+      if ((*it).getOwner() == Playerlist::getActiveplayer())
         {
       
-          if ((*it).getProductionIndex() == -1)
+          if ((*it).getActiveProductionSlot() == -1)
             prod = false;
           else
             prod = true;
@@ -279,7 +282,7 @@ void VectorMap::after_draw()
 	{
 	  if ((*it).isFogged() == true)
 	    continue;
-	  if ((*it).getPlayer() != city->getPlayer())
+	  if ((*it).getOwner() != city->getOwner())
 	    continue;
 	  if ((*it).getVectoring() == Vector<int>(-1, -1))
 	    continue;
