@@ -1,3 +1,7 @@
+// Copyright (C) 2002, 2003 Michael Bartl
+// Copyright (C) 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
+// Copyright (C) 2003, 2004, 2005 Andrea Paternesi
+//
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
 //  the Free Software Foundation; either version 2 of the License, or
@@ -10,7 +14,8 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+//  02110-1301, USA.
 
 #include <iostream>
 #include <sstream>
@@ -125,6 +130,20 @@ XML_Helper::XML_Helper(std::string filename, std::ios::openmode mode, bool zip)
     }
 }
 
+XML_Helper::XML_Helper(std::ostream* output)
+  : d_inbuf(0), d_outbuf(0), d_fout(0), d_fin(0), d_out(0), d_in(0),
+    d_last_opened(""), d_version(""), d_failed(false), d_zip(false)
+{
+  d_out = output;
+}
+
+XML_Helper::XML_Helper(std::istream* input)
+  : d_inbuf(0), d_outbuf(0), d_fout(0), d_fin(0), d_out(0), d_in(0),
+    d_last_opened(""), d_version(""), d_failed(false), d_zip(false)
+{
+  d_in = input;
+}
+
 XML_Helper::~XML_Helper()
 {
     if (d_tags.size() != 0)
